// Scintilla source code edit control
/** @file UniConversion.cxx
 ** Functions to handle UFT-8 and UCS-2 strings.
 **/
// Copyright 1998-2001 by Neil Hodgson <neilh@scintilla.org>
// The License.txt file describes the conditions under which this software may be distributed.

#include <stdlib.h>

#include "UniConversion.h"

// returns the expected UTF-8 length of a UCS-2 string
unsigned int UTF8Length(const wchar_t *uptr,unsigned int tlen) {
  const wchar_t *uend=uptr+tlen;
	unsigned int len = 0;
	for (; uptr<uend; uptr++) {
		if (*uptr < 0x80) len++;
		else if (*uptr < 0x800) len+=2;
		else len +=3;
	}
	return len;
}

// converts a UCS-2 string to a UTF-8 string. Obtain len from UTF8Length()
void UTF8FromUCS2(const wchar_t *uptr, unsigned int tlen, char *putf, unsigned int len) {
  const wchar_t *uend=uptr+tlen;
  unsigned char *uputf=(unsigned char *)putf,*upend=uputf+len;
	for (; uptr<uend && uputf<upend; uptr++) {
		wchar_t uch = *uptr;
		if (uch < 0x80) {
			*uputf++ = (unsigned char)uch;
		} else if (uch < 0x800) {
		  if (uputf+1>=upend) break;
			*uputf++ = (unsigned char)(0xC0 | (uch >> 6));
			*uputf++ = (unsigned char)(0x80 | (uch & 0x3f));
		} else {
		  if (uputf+2>=upend) break;
			*uputf++ = (unsigned char)(0xE0 | (uch >> 12));
			*uputf++ = (unsigned char)(0x80 | ((uch >> 6) & 0x3f));
			*uputf++ = (unsigned char)(0x80 | (uch & 0x3f));
		}
	}
}

// returns the expected UCS2 length of a UTF-8 string
unsigned int UCS2Length(const char *s, unsigned int len) {
  const unsigned char *sx=(unsigned char *)s,*endx=sx+len;
	unsigned int ulen=0;
	for (;sx<endx;sx++) if (*sx < 0x80 || *sx > (0x80+0x40)) ulen++;
	return ulen;
}

// converts a UTF-8 string to a UCS-2 string. Obtain tlen from UCS2Length()
unsigned int UCS2FromUTF8(const char *s, unsigned int len, wchar_t *tbufo, unsigned int tlen) {
  const unsigned char *sx=(unsigned char *)s,*endx=sx+len;
  wchar_t *tbuf=tbufo,*endt=tbuf+tlen;
	while (sx<endx && tbuf<endt) {
		if (*sx < 0x80) {
			*tbuf++ = *sx;
			sx++;
		} else if (*sx < (0x80 + 0x40 + 0x20)) {
		  *tbuf++ = ((*sx & 0x1F) << 6) | (sx[1] & 0x7F);
		  sx+=2;
		} else {
		  *tbuf++ = ((*sx & 0xF) << 12) | ((sx[1] & 0x7F) << 6) | (sx[2] & 0x7F);
		  sx+=3;
		}
	}
	return(tbuf-tbufo);
}
