//this file is part of notepad++
//Copyright (C)2003 Don HO ( donho@altern.org )
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//http://www.gnu.org/copyleft/gpl.html
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// Notepad ++ http://notepad-plus.sourceforge.net/uk/site.htm
// Scintilla: http://www.scintilla.org/
//
// To install this plugin, copy the DLL and any required files to the
// plugins folder in the Notepad++ install folder.
//
// NPPTextFX (C) 2005 by Chris Severance

// What can you do to help?
//  Look for bugs in existing functions.
//  Expand routines for more functionality and more specific programming language support.
//  Think of new esoteric things that are hard or impossible with Copy-Paste or regex Search-Replace
//    you want done in an editor and add it. If you're lucky, you can reuse lots of existing code.
//  Come up with better titles for menu items.
//  Arrange the menus to make more sense.
//  Hassle Don Ho until he adds the features we need to make better plugins.
//  Hassle Don Ho to put most of these functions into his editor. Many of these things shouldn't be plugins.
//  Adapt functions for UNICODE.
//  Improve the help texts.
//  Make it run on more compilers.
//  Make the examples in the demo text file better
//  Beg other Notepad writers to make a plugin system and adapt this code to it.
//  Take functions that now use clipboard text because it's all I can get to and write proper dialog inputs for them.
// "new feature" is found throughout the code where new features have been thought of but not implemented

// Projects for Dev-C++ 4.9.9.2 and CodeBlocks are included
// This compiles under MinGW or Digital Mars as a .c or .cpp file
// This compiles under MS Visual C++ 2003 Free or Borland C++ 5.51 Free as a .cpp file

// Users wanting to run MS Visual C++ 2003 Free must download both the toolkit and the SDK
// Install the SDK base option only and move include & lib over to the include&lib in the Visual Toolkit folder.
// http://msdn.microsoft.com/visualc/vctoolkit2003/
// http://www.microsoft.com/msdownload/platformsdk/sdkupdate/ (I prefer the ISO install)
// http://www.microsoft.com/whdc/devtools/debugging/installx86.mspx

// The project compiles easiest when placed in the Notepad ++ plugins folder

// Quite a few goto's are used in this code. I strive to remove gotos
// but I use them when they are more clear and concise than the alternatives.
// Modern compilers seem to optimize around them just fine.
// Donald Knuth's "Structured Programming with go to Statements"
// http://www.stevemcconnell.com/ccgoto.htm

/* This was designed as a C file but only MinGW and Digital Mars handle variable declarations within a function for .c sources. */
/* Switching to .cpp opened up the rest of the compilers */

/* For Codeblocks-BC Borland we must fix the underscore in front of the exported symbols problem */
/* You must go to Compile | Compiler Options | Borland C++ Compiler 5.5 | Other | Advanced Options | Link Object Files to Dynamic Library */
/* and change the text box to $linker -Tpd $link_options $libdirs c0d32 $link_objects, $exe_output,, $libs,$exe_output.BCDEF, $link_resobjects */

/* For CodeBlocks-WC OpenWatcom we must fix DLL generation and the underscore in front of the exported symbols problems */
/* You must go to Compile | Compiler Options | OpenWatcom (W32) Compiler | Other | Advanced Options | Link Object Files to Dynamic Library */
/* and change the text box to $linker -nologo -dll -out:$exe_output $libs $link_objects -def:$exe_output.BCDEF */
/* Then go to Compile | Compiler Options | OpenWatcom (W32) Compiler | Programs */
/* and change the Dynamic Linker to: link386.exe */
/* option -3s "386 stack calling conventions" must be selected */

/* For Codeblocks-DMC Digital Mars we must fix the problem where the .DEF file doesn't get created during compile to pass to the linker if it's not all done at once */
/* If a .DMCDEF isn't already provided, create one by running: */
/* c:\dm\bin\dmc.exe -mn -WD -DBUILD_DLL -I"C:\dm\include" NPPTextFX.cpp shell32.lib kernel32.lib */
/* and rename it to NPPTextFX.DLL.DMCDEF */
/* Then you must go to Compile | Compiler Options | Digital Mars Compiler | Other | Advanced Options | Link Object Files to Dynamic Library */
/* and change the text box to $linker $link_options $link_objects,$exe_output,,$libs,$exe_output.DMCDEF,$link_resobjects */

/* Indent Lines Sticky Margin was my first routine that could go low/high performace so I had it GetTickCount() with NPPTextFX.CPP==178K on a P3-500.
   High,Low,Size,Time Performace // few optimizations turned on
  1800,2600,108K,10s Borland-C (NPPDEBUG=0) // merge duplicate strings
  1500,4300,108K,10s Borland-C (NPPDEBUG=1) // merge duplicate strings
  1600,1700,110K, 4s DMC       (NPPDEBUG=0) // This is pretty impressive for a work-in-progress compiler
  2200,4800,110K, 4s DMC       (NPPDEBUG=1)
  3200,5800,110K,25s MSVC      (NPPDEBUG=1)
  5800,8100, 52K,37s MinGW     (NPPDEBUG=1) // optimizations turned off
  5600,9500,106K,14s OpenWatcom (NPPDEBUG=1)
  Borland-C & DMC may have a larger runtime size but they are blowing the other compilers away for runtime speed and compile times.
  Borland-C has the most useful warnings.
  DMC will be used for releases from now on.
  Looks like switching to DMC will give me much greater performance gains than converting any more functions to dual buffers high performance.
*/

// Exclude rarely-used stuff from Windows headers
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shellapi.h>

#ifdef __BORLANDC__ /* { */
#pragma warn -pia /* Borland-C: possibly incorrect assignment */
//#pragma warn -ccc	/* Borland-C: condition is always true/false */
#pragma warn -rch	/* Borland-C: Unreachable code */
#endif /* } */

#include <shlwapi.h> /* DMC: Needs to have it's includes and libs updated to get shlwapi.h */
#include <shlobj.h>  /* MSVC: needs shlwapi.h copied from MS Visual Studio to work */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include "PluginInterface.h"

#ifndef NELEM /* { */
#define NELEM(xyzzy) (sizeof(xyzzy)/sizeof(xyzzy[0]))
#endif /* } */
#ifdef PLAT_WIN /* { */
#define NPPWINDOWS 1 /* These 0 or 1 definitions allow us to use the constants in C code for the macros */
#else /* } else { */
#define NPPWINDOWS 0
#endif /* } */
#ifndef NPPDEBUG
#define NPPDEBUG 1 /* set to 1 to turn on string bounds checkers and error boxes */
#endif
// This is the name which will be displayed in Plugins Menu
#define PLUGIN_NAME "TextFX"
#define SUPPORT_PATH "NPPTextFX" /* thanks to smprintfpath we don't need to put a slash on the end of this */
BOOL onNT=FALSE;
#define IsNT() onNT
HANDLE loadonce=NULL;
CLIPFORMAT cfColumnSelect;
struct NppData nppData;
char *pluginpath=NULL;
#define HighPerformance 0 /* define this to disable adaptive code; remove define to enable adaptive code */
#ifndef HighPerformance
BOOL HighPerformance;
#define HighPerformanceon 0 /* an if() can use HighPerformanceon if it wants to be on when HighPerformance is not adaptive */
#else
#define HighPerformanceon 1
#endif

#define LaunchURL(lurl) ShellExecute(nppData._nppHandle,"open",lurl,NULL,".",0);

/* Here we enable or disable code imported from other projects. You may need this because:
   The import source code is no longer useful, available, or legal to use in certain markets.
   The import source code originally used was not provided and it's too hard to figure out how to retrofit replacement source code.
   The imported function generates unacceptably large compiled code that can be eliminated for some platforms.
   The import function cannot be implemented on some platforms.
   The import is already implemented by another program for some platforms.
   The import function is causing problems and removing it would help pinpoint the problem source.
   The import function does not compile and run with the desired compiler.
   You want to know what parts of the code were affected by the addition of a particular import.
   */
#ifndef ENABLE_TIDYDLL
#define ENABLE_TIDYDLL 1 /* libTidy.DLL must be compiled with __cdecl calls as done in the enclosed Code::Blocks project */
#define ENABLE_PUSHPOP 0
#endif
//----------------------------------------------------------//
// Here're the definition of functions for our functionalities
//----------------------------------------------------------//

#ifdef _MSC_VER /* { */
#define vsnprintf _vsnprintf /* why doesn't Microsoft provide this now standard function? */
#endif /* } */
#if NPPDEBUG /* { */
#if (defined(__DMC__) || defined(_MSC_VER)) /* { */
#define snprintf _snprintf /* snprintf is unsafe so we only want to use our snprintfX() */
#endif /* } */
#else /* } else { */
#define snprintf snprintf_unsafe
#endif /* } */
#define sprintf sprintf_unsafe
#define strncpy strncpy_unsafe

// SECTION: Beginning of library functions

#if NPPDEBUG
/* We control all malloc/realloc/free buffers. We add 3 bytes to the buffer size and place
   1 check code at the beginning and 2 at the end. If any of those are modifed then we know
   that the caller has gone beyond the bounds. */
struct _MALLOC_STACK {
  void *buf;
  unsigned len;
  const char *title,*dummy; /* dummy makes it 16 bytes */
} mallocstack[100];

// wants the fake buf and fake ct
// returns the real buf
void *freesafebounds(void *bf,unsigned ct,const char *title) {
  if (bf) {
    unsigned char *bft=(unsigned char *)bf;
    --bft;
    bf = (void *)bft;
    if (bft[0] != 255)  MessageBox(0,"Buffer went beyond buffer begin",title, MB_OK|MB_ICONSTOP);
    if (bft[ct+1]!=255) MessageBox(0,"Buffer went beyond wrote 1+ buffer end",title, MB_OK|MB_ICONSTOP);
    if (bft[ct+2]!=254 )  MessageBox(0,"Buffer went beyond wrote 2+ buffer end",title, MB_OK|MB_ICONSTOP);
  }
  return(bf);
}

// wants the real buf and fake ct
// returns the fake buf
void *reallocsafebounds(void *bf,unsigned ct) {
  void *bf2;
  bf2=bf?realloc(bf,ct+3):malloc(ct+3);
  if (bf2) {
    unsigned char *bft2=(unsigned char *)bf2;
    bft2[0]   =255;
    bft2[ct+1]=255;
    bft2[ct+2]=254;
    bft2++;
    bf2 = (void *)bft2;
    //if (!bf) memset(bf2,'@',ct);
  }
  return(bf2);
}

void freesafe(void *bf,const char *title) {
  if (bf) {
    int i; for(i=0; i<NELEM(mallocstack); i++) if (mallocstack[i].buf==bf) {
      free(freesafebounds(mallocstack[i].buf,mallocstack[i].len,title));
      //mallocstack[i].buf=NULL;
      memset(mallocstack+i,0,sizeof(mallocstack[0]));
      return;
    }
    MessageBox(0,"free() buffer never allocated",title, MB_OK|MB_ICONSTOP);
    return;
  }
  MessageBox(0,"free() NULL buffer",title, MB_OK|MB_ICONSTOP);
}

void *reallocsafeX(void *bf,unsigned ct,char *title,int allownull) {
  void *bf2=NULL;
  if (ct) {
    if (bf) {
      int i; for(i=0; i<NELEM(mallocstack); i++) if (mallocstack[i].buf==bf) goto fnd;
      MessageBox(0,"Buffer realloc() not originally realloc()",title, MB_OK|MB_ICONSTOP);
      return(NULL);
      if (0) {
fnd:    if (ct==mallocstack[i].len) MessageBox(0,"Buffer realloc() to same size",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
        bf2=freesafebounds(mallocstack[i].buf,mallocstack[i].len,title);
      }
    } else if (!allownull) MessageBox(0,"Attempt to realloc NULL buf",title, MB_OK|MB_ICONSTOP);
    bf2=reallocsafebounds(bf2,ct);
    if (bf2) {
      int i;
      for(i=0; i<NELEM(mallocstack); i++) if (mallocstack[i].buf==bf) {mallocstack[i].buf=bf2; mallocstack[i].len=bf2?ct:0; if (!mallocstack[i].title) mallocstack[i].title=bf2?title:NULL; goto fnd2;}
      MessageBox(0,"no more space in malloc stack",title, MB_OK|MB_ICONSTOP);
    }
  } else if (bf) freesafe(bf,title);
fnd2:
  return(bf2);
}

#define mallocsafeinit() memset(mallocstack,0,sizeof(mallocstack))

void mallocsafedone(void) {
  unsigned badct; int i; for(badct=0, i=0; i<NELEM(mallocstack); i++) if (mallocstack[i].buf) {badct++; MessageBox(0,mallocstack[i].title,"mallocsafedone", MB_OK|MB_ICONSTOP); free(freesafebounds(mallocstack[i].buf,mallocstack[i].len,"mallocsafedone"));}
  if (badct) {
    char debug[256]; snprintf(debug,sizeof(debug),"%u malloc() buffers not free'd\n",badct); MessageBox(0,debug,"mallocsafedone", MB_OK|MB_ICONSTOP);
  }
  mallocsafeinit();
}

#define mallocsafe(ct,ti) reallocsafeX(NULL,ct,ti,1)
#define reallocsafe(bf,ct,ti) reallocsafeX(bf,ct,ti,0)
#define malloc malloc_unsafe
#define realloc realloc_unsafe
#define calloc calloc_unsafe /* I never use calloc() but just in case someone tries */
#define free free_unsafe
#define strdup strdup_unsafe

char *strdupsafe(const char *source,char *title) {
    unsigned ls=strlen(source)+1;
  char *rv=(char *)mallocsafe(ls,title);
  if (rv) memcpy(rv,source,ls);
  return(rv);
}

#if 1
#define testmallocsafe()
#else
void testmallocsafe(void) {
  char *test,*test2;
  mallocsafeinit();
  if (test=(char *)mallocsafe(0,"test")) MessageBox(0,"Improper return #1",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
  freesafe(test,"test"); // free NULL buf
  if (!(test=(char *)mallocsafe(10,"test"))) MessageBox(0,"Improper return #2",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
  memset(test,'@',10);
  if (!(test=(char *)reallocsafe(test,10,"test"))) MessageBox(0,"Improper return #3",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
  test[-1]='x'; // buffer beyond bounds
  test2=(char *)reallocsafe(NULL,11,"test2"); // realloc NULL buff
  test2[11]='x';
  freesafe(test,"test"); // no error
  freesafe(test,"test"); // free never allocated
  freesafe(NULL,"test");
  mallocsafedone();
}
#endif

#else
#define mallocsafe(ct,ti) malloc(ct)
#define reallocsafe(bf,ct,ti) realloc(bf,ct)
#define freesafe(bf,ti) free(bf)
#define strdupsafe(bf,ti) strdup(bf)
#endif

size_t roundtonextpower(size_t numo) { /* round to next power */
  size_t pow,num;
  num=numo;
  if (num>=32768) {pow=15; num >>= 15;}
  else if (num>1024) {pow=10; num >>=10;}
  else if (num>128) {pow=5; num>>=5;}
  else pow=0;
  while(num>1) {pow++; num>>=1;}
  num=1<<pow;
  if (num<numo) num<<=1;
  return(num);
}

// returns the number of bytes *dest was changed by so the caller can update their pointers too, including when *dest becomes NULL by accident.
// If you are certain that *dest started non null, you can be sure that it hasn't become NULL if the return value is 0
/* Constant realloc()'s are abusive to memory because realloc() must copy the data to a new malloc()'d area.
   On the other hand, slack space on buffers that will never use it is wasteful.
   Choose strategy to minimize realloc()'s while making good use of memory */
#define ARMSTRATEGY_INCREASE 0 //increase buffer with slack space when necessary; good for a constantly expanding buffer
#define ARMSTRATEGY_INCREASE 0 //increase buffer with slack space when necessary; good for a constantly expanding buffer
#define ARMSTRATEGY_MAINTAIN 1 //increase buffer only the minimum amount, buffer will not be reduced if too large; good for a buffer that will be reused with some large and some small allocations
#define ARMSTRATEGY_REDUCE 2   //increase buffer only the minimum amount, reduce buffer if too large; good for buffers of known size or that will only be used once
// If choosing ARMSTRATEGY_REDUCE for a new buffer it is usually better to just use malloc() yourself and not call armrealloc()
// clear=1, memset new space to 0
size_t _armrealloc_cutoff=16384; /* sizeof(buf)<=, next size doubled, >, added */
int armrealloc(char **dest,unsigned *destsz,unsigned newsize,int strategy,int clear
#if NPPDEBUG
,char *title
#define armreallocsafe armrealloc
#define THETITLE title
#else
#define armreallocsafe(dt,ds,ns,st,cl,ti) armrealloc(dt,ds,ns,st,cl)
#define THETITLE "armrealloc" /* the macros discards this */
#endif
) {
  unsigned rv=0,oldsz=(*dest)?(*destsz):0;
  char *dest1;
  if (strategy==ARMSTRATEGY_REDUCE && *destsz != newsize) { *destsz = newsize; goto doit;} // without this goto, the conditionals were so convoluted that they never worked
  if (strategy==ARMSTRATEGY_INCREASE){
    if (newsize < 64) newsize=64;
    if (!*dest && newsize < *destsz) newsize=*destsz;
  }
  if (!*dest || *destsz <  newsize) {
    *destsz = (strategy==ARMSTRATEGY_INCREASE)?roundtonextpower(newsize):newsize;
doit:
    dest1=*dest;
    dest1= (dest1)?(char *)reallocsafe(dest1,*destsz,THETITLE):(char *)mallocsafe(*destsz,THETITLE);
      //if (dest1) strcpy((dest1+*destsz-1),"#$"); // $ is beyond string
    rv=dest1-*dest;
    if (dest1 && clear && *destsz>oldsz) memset(dest1+oldsz,0,*destsz-oldsz);
    *dest=dest1;
  }
  return(rv);
}
#undef THETITLE

/* strcpyarm() append-realloc-malloc; This function is designed to
   continually add onto the end of a string. It copies source to *dest+*destlen
   and *destlen += strlen(source). If the new data including \0 would exceed *destsz, the buffer is realloc()'d
   larger and *destsz will be updated with the new size. One of the handy side affects of the arm routines
   is that buffers are no longer created on the stack where they can endanger other stack variables.
   Hackers will find it difficult to overrun a buffer and when they do, all they will trash is the heap.
   returns the number of bytes *dest was changed by during a realloc()
   if source==NULL then no action is taken and a 0 is returned
   *dest must be NULL or a malloc()'d pointer. *dest must not be a stack or data pointer
   if *dest==NULL, a new buffer will be malloc()'d to hold source, *destlen will start at 0
     you can force the new buffer to be larger than source by setting *destsz to the desired starting size
     *destsz should be set to 0 if you do not wish to specify a size for the initial buffer
   if *dest is not NULL, *destsz is always the actual size of the
     malloc()'d buffer so should never be altered while in use unless you free() and reallocate *dest yourself.
   destlen==NULL disables append
     source will always be placed at *dest
     the buffer will always be allocated to it's minimum possible size
     when destlen!=NULL, the buffer will usually have slack space on the end for more things to be added
   dest,destsz cannot be NULL
   On rare occasion, a realloc() will fail and a once valid *dest may be sent back as NULL,
     when this happens *destsz,*destlen will be left at their most recent values though
     they are no longer valid. The next call will start *destlen at 0 and will attempt to allocate *destsz or strlen(source) bytes of memory depending on destlen
   maxlen is the maximum number of characters to copy from source, or (unsigned)-1 to copy all characters. */
// someday we may need a strncpyarmfree() which frees source after it is used
int strncpyarm(char **dest,size_t *destsz,size_t *destlen,const char *source,size_t maxlen
#if NPPDEBUG
,char *title
#define strncpyarmsafe strncpyarm
#define strcpyarmsafe(buf,bufsz,bufl,scsrc,ti) strncpyarm(buf,bufsz,bufl,scsrc,(unsigned)-1,ti)
#define THETITLE title
#else
#define strncpyarmsafe(dt,ds,dl,st,ml,ti) strncpyarm(dt,ds,dl,st,ml)
#define strcpyarmsafe(buf,bufsz,bufl,scsrc,ti) strncpyarm(buf,bufsz,bufl,scsrc,(unsigned)-1)
#define THETITLE "strncpyarm" /* the macros discards this */
#endif
) {
  size_t destlen1,slen;
  int rv=0;
  if (source) { // maxlen==0 is perfectly valid
    slen=strlen(source);
    if (slen>maxlen) slen=maxlen;
    destlen1=(*dest && destlen)?*destlen:0;
    if (*dest && !destlen1 && *destsz!=slen+1) {
      freesafe(*dest,THETITLE);
      *dest=NULL;
      *destsz=slen+1;
    }
    rv=armreallocsafe(dest,destsz,destlen1+slen+1,destlen?ARMSTRATEGY_INCREASE:ARMSTRATEGY_REDUCE,0,THETITLE);
    if (*dest) {
      if (slen) {
        memcpy(*dest+destlen1,source,slen);
        destlen1 += slen;
      }
      *(*dest+destlen1)='\0';
      if (destlen) *destlen=destlen1;
    }
  }
  return(rv);
}
#undef THETITLE

int memcpyarm(char **dest,size_t *destsz,size_t *destlen,const char *source,size_t slen
#if NPPDEBUG
,char *title
#define memcpyarmsafe memcpyarm
#define THETITLE title
#else
#define memcpyarmsafe(dt,ds,dl,st,ml,ti) memcpyarm(dt,ds,dl,st,ml)
#define THETITLE "memcpyarm" /* the macros discards this */
#endif
) {
  size_t destlen1;
  int rv=0;
  if (source) { // slen=0 must at least allocate memory
    destlen1=(*dest && destlen)?*destlen:0;
    if (*dest && !destlen1 && *destsz!=slen) {
      freesafe(*dest,THETITLE);
      *dest=NULL;
      *destsz=slen;
    }
    rv=armreallocsafe(dest,destsz,destlen1+slen,destlen?ARMSTRATEGY_INCREASE:ARMSTRATEGY_REDUCE,0,THETITLE);
    if (*dest) {
      if (slen) {
        memcpy(*dest+destlen1,source,slen);
        destlen1 += slen;
      }
      if (destlen) *destlen=destlen1;
    }
  }
  return(rv);
}
#undef THETITLE

int memsetarm(char **dest,size_t *destsz,size_t *destlen,int chr,size_t slen
#if NPPDEBUG
,char *title
#define memsetarmsafe memsetarm
#define THETITLE title
#else
#define memsetarmsafe(dt,ds,dl,st,ml,ti) memsetarm(dt,ds,dl,st,ml)
#define THETITLE "memcpyarm" /* the macros discards this */
#endif
) {
  size_t destlen1;
  int rv=0;
    destlen1=(*dest && destlen)?*destlen:0;
    if (*dest && !destlen1 && *destsz!=slen) {
      freesafe(*dest,THETITLE);
      *dest=NULL;
      *destsz=slen;
    }
    rv=armreallocsafe(dest,destsz,destlen1+slen,destlen?ARMSTRATEGY_INCREASE:ARMSTRATEGY_REDUCE,0,THETITLE);
    if (*dest) {
      if (slen) {
        memset(*dest+destlen1,chr,slen);
        destlen1 += slen;
      }
      if (destlen) *destlen=destlen1;
    }
  return(rv);
}
#undef THETITLE

/* snprintfX() attempts to perfectly duplicate what sprintf() does
   without the unsafeness or undesirable behaviour of sprintf()
   or the bugs and problems of various snprintf()'s
   Call: snprintfX(buf,sizeof(buf),"format string",...);
   Returns: number of non null characters written.
   Unlike the official snprintf():
     * snprintfX() writes no more than sizeof(buf)-1 non null characters plus a guaranteed \0 ending.
     * snprintfX() always returns the exact number of non null characters written
     * snprintfX() does not return a -1 if an error occurs or more buffer space is needed
     * snprintfX() does not return the total number bytes that would be
         needed to store the entire string when there is not enough space
   If you need any of that functionality, you must call the real functions and deal with the bugs as vsarmprintf() does */
size_t snprintfX(char *buffer,size_t buffersz,const char *format,...) {
  size_t rv=0;
  va_list ap;
  if (buffer && buffersz) {
    va_start(ap, format);
    buffer[0]='\0';
    rv=vsnprintf(buffer,buffersz,format,ap);
    buffer[buffersz-1]='\0';
    if (rv==(unsigned)-1) rv=strlen(buffer);
    if (rv>=buffersz) rv=buffersz-1;
    va_end(ap);
  }
  return(rv);
}

/* sprintf() using the same buffer system as strncpyarm()
   see strncpyarm() for parameter meanings
   http://libslack.org/manpages/snprintf.3.html
   smprintf() handles the broken vsnprintf() implementations that return -1 when there isn't enough space
     or ones that ensure or don't ensure a \0 at the end
   The v...() routines are rarely used directly. They are made so the non v routine wrappers and custom wrappers can be built.*/
size_t vsarmprintf(char **dest,size_t *destsz,size_t *destlen,const char *format,va_list ap2) {
  size_t vs=0,destlen1,destsz1=*destsz;
  char *dest1=*dest;
  va_list ap;

  destlen1=destlen?*destlen:0;
  if (!dest1) {
    destlen1=0;
    if (destsz1<8) destsz1=8;
    dest1=(char *)mallocsafe(destsz1,"vsarmprintf");
  }
  if (dest1 && destsz1<8+destlen1) { //guarantee at least 8 characters into which vsnprintf() can write
    destsz1=8+destlen1;
    dest1=(char *)reallocsafe(dest1,destsz1,"vsarmprintf");
  }
  goto bottest; while(dest1 && destsz1 && !(dest1[destsz1-1]='\0') && ((vs=vsnprintf(dest1+destlen1,destsz1-destlen1,format,ap))>(destsz1-destlen1) || dest1[destsz1-1])) {
    if (vs == (unsigned)-1) destsz1=roundtonextpower(destsz1*2);
    else destsz1=destlen?roundtonextpower(destlen1+vs+1):destlen1+vs+1;
#if NPPWINDOWS /* { no recursive calls please */
    //char debug[256]; snprintfX(debug,sizeof(debug),"vsnprintf() returns %u; Trying length %u",vs,destsz1); MessageBox(nppData._nppHandle,debug,PLUGIN_NAME, MB_OK|MB_ICONSTOP);
#endif /* } */
    if (!destlen1) {
      freesafe(dest1,"vsarmprintf");
      dest1=(char *)mallocsafe(destsz1,"vsarmprintf");
    } else dest1=(char *)reallocsafe(dest1,destsz1,"vsarmprintf");
bottest:
#ifdef __WATCOMC__
    memcpy(&ap[0],&ap2[0],sizeof(ap[0])); /* It appears that vsnprintf() increments ap[0] which we don't want. What a buggy compiler! */
#else
    ap=ap2;
#endif
    //memset(dest1+destlen1,'@',destsz1-destlen1); dest1[destsz1]='?'; // beyond string
  }
  if (dest1) {
#if NPPDEBUG /* { */
    if (vs != strlen(dest1+destlen1)) { /* no recursive calls please */
      char debug[256]; snprintfX(debug,sizeof(debug),"vs=%u strlen(dest1)=%u",vs,strlen(dest1)); MessageBox(nppData._nppHandle,debug,PLUGIN_NAME, MB_OK|MB_ICONSTOP);
      vs = strlen(dest1+destlen1);
    }
    if (destsz1<=vs+destlen1) {
      char debug[256]; snprintfX(debug,sizeof(debug),"Buffer too short, destlen1=%u vs=%u",destsz1,vs); MessageBox(nppData._nppHandle,debug,PLUGIN_NAME, MB_OK|MB_ICONSTOP);
    }
#endif /* } */
    if (!destlen && destsz1>vs+1+destlen1) {
      destsz1=vs+1+destlen1;
      dest1=(char *)reallocsafe(dest1,destsz1,"vsarmprintf");
#if NPPWINDOWS /* { no recursive calls please */
      //char debug[256]; snprintfX(debug,sizeof(debug),"Final length (including \\0) was %u",vs+1); MessageBox(nppData._nppHandle,debug,PLUGIN_NAME, MB_OK|MB_ICONSTOP);
#endif /* } */
    }
  }
  if (!(*dest=dest1)) vs=0;
  *destsz=destsz1;
  if (destlen) *destlen=destlen1+vs;
  return(vs);
}

size_t sarmprintf(char **dest,size_t *destsz,size_t *destlen,const char *format,...) {
  size_t rv;
  va_list ap;
  va_start(ap,format);
  rv=vsarmprintf(dest,destsz,destlen,format,ap);
  va_end(ap);
  return(rv);
}

/* snprintf() falls short of the mark.
   What we really need is smprintf(...) AKA mprintf(...).
   Like strdup() it malloc()'s sufficient space and places the sprintf()
   results in it. The caller will get the entire result without needing to
   overguess and waste space. The caller will free the pointer. The caller
   may create a special routine that uses and free's the pointer all at once. */
char *smprintf(const char *format,...) {
  char *rv=NULL;
#if NPPDEBUG
  size_t rvsz=0;
#else
  size_t rvsz=256;
#endif
  va_list ap;
  va_start(ap,format);
  vsarmprintf(&rv,&rvsz,NULL,format,ap);
  va_end(ap);
  return(rv);
}

/* vsmprintfpath() gets much reduced functionality from smprintf()
   but it gains a couple of features needed for path construction
   and it probably runs a lot faster than the official smprintf()
   so it's better for simple format strings. Simple format strings
   without any of the special functions are interchangable with
   sprintf() variants.
   If you need full sprintf() functionality, use smprintf() or snprintfX() above. */
/* Control characters:
   Supported: exactly as shown; no modifiers allowed!
     %%=%
     %c=character
     %s=string
     %d,%i=integer
     %o=octal
     %u=unsigned
     %x,%X hex
   New and Improved Functionality:
//new feature: %bn# print number in any base, n can be *
//new feature: optional characters also cancel next character if the same
//new feature: %S a string that needs to be free()'d after it is used
//new feature: %s prints (null) when NULL is passed in
//new feature: %#n(...) repeat ... n times
//new feature: %D hex dump of string
     %?_, optional character _ immediately after ? is only inserted if the previous character is not a _
     %C, optional character next in va_args is only inserted if the previous character is not the same
       special note: if the character is a space then it is only inserted if the previous character is not whitespace
     %#n* ; %#n#_ where n is a number or * and _ is a character;
       inserts n characters _ of your choice: example: %#57#? inserts 57 question marks. n may be a * to extract the number from the arglist
       if the second # is a * then the character is drawn from the arglist
       %#** (both from arglist) or %#57* (char from arglist)
   Not Supported:
     Any internal modifiers
     float types
   Example: smprintfpath("%c%?:%s%?\\%s%?\\%s",'C',"\\WINDOWS\","TEMP","TEMPFILE.TXT")
     -> C:\WINDOWS\TEMP\TEMPFILE.TXT
*/
size_t vsarmprintfpath(char **dest,unsigned *destsz,unsigned *destlen,const char *format,va_list ap2) {
  size_t tlen=0,llen;
  char *cx;
  char *argst,*dest1,lastch,checkch,cmd[3],tempspace[128];
  unsigned pass,argu;
  size_t destlen1=(*dest && destlen)?*destlen:0;
  for(pass=0; pass<2; pass++) {
    va_list ap;
#ifdef __WATCOMC__
    memcpy(&ap[0],&ap2[0],sizeof(ap[0]));
#else
     ap=ap2;
#endif
    dest1=*dest+destlen1; // not used on the first pass, prevents compiler warnings
    for(lastch='\0',cx=(char *)format; *cx; cx++) {
      if (*cx!='%') {
pctch:
        lastch=*cx;
        goto dochar;
      } else switch(*++cx) {
        // better idea: insert the % symbol and go on to the next character
      case '\0': cx--; break; /* feed the \0 back to kill the for loop */
      case '%': lastch  = '%'; goto dochar;
      case 'c': lastch  = (int)va_arg(ap,unsigned); goto dochar;
      case 'C': checkch = (int)va_arg(ap,unsigned); goto checkchar;
      case '?':
        cx++;
        checkch=*cx;
checkchar:
        if (!(checkch==' ' && isspace(lastch) || lastch==checkch)) {
          lastch=checkch;
dochar:   if (!pass) {
            tlen++;
          } else {
            *dest1=lastch;
            dest1++;
            *dest1='\0';
          }
        }
        break;
      case 'd':
      case 'i':
      case 'o':
      case 'u':
      case 'x':
      case 'X':
        argu = va_arg(ap,unsigned);
        memcpy(cmd,"%c",3);
        cmd[1]=*cx;
        snprintfX(tempspace,sizeof(tempspace),cmd,argu);
        argst=tempspace;
        goto dostring;  /* this saves code but wastes a strlen on a known short string */
      case 's':
        argst = va_arg(ap,char *);
dostring:
        if (!pass) {
          if (llen=strlen(argst)) {
            lastch=argst[llen-1];
            tlen+=llen;
          }
        } else {
          while(*dest1=*argst) {dest1++; argst++;} // stpcpy() isn't always available
          //memcpy(dest,argst,strlen(argst)+1);  %S support
          if (dest1>*dest) lastch=*(dest1-1);
        }
        break;
      case '#':
        cx++;
        if (!*cx) continue;
        else if (*cx=='*') {
          llen=va_arg(ap,unsigned);
          cx++;
          if (!*cx) continue;
        } else llen=strtol(cx,&cx,0);
        if (llen) {
          if (*cx=='*') {
            lastch=va_arg(ap,int);
          } else {
            cx++;
            lastch=*cx;
          }
          if (!pass) {
            tlen+=llen;
          } else {
            memset(dest1,lastch,llen);
            dest1 += llen;
            *dest1='\0';
          }
        } else cx++;
        break;
default: goto pctch;
      }
    }
    if (!pass) {
      armreallocsafe(dest,destsz,destlen1+tlen+1,destlen?ARMSTRATEGY_INCREASE:ARMSTRATEGY_REDUCE,0,"vsarmprintfpath");
      if (!*dest) break;
    }
  }
  if (destlen) *destlen += tlen;
  return(tlen);
}

size_t sarmprintfpath(char **dest,size_t *destsz,size_t *destlen,const char *format,...) {
  size_t rv;
  va_list ap;
  va_start(ap,format);
  rv=vsarmprintfpath(dest,destsz,destlen,format,ap);
  va_end(ap);
  return(rv);
}

char *smprintfpath(const char *format,...) {
  char *rv=NULL;
#if NPPDEBUG
  size_t rvsz=0;
#else
  size_t rvsz=256;
#endif
  va_list ap;
  va_start(ap,format);
  vsarmprintfpath(&rv,&rvsz,NULL,format,ap);
  va_end(ap);
  return(rv);
}

/* Just like MessageBox() except that lpText must be NULL or a malloc()'d pointer that will be free()'d after display.
   This allows you to display message text of unknown size without creating and freeing a pointer.
   If lpText is NULL meaning the malloc() failed, a nondescript default error message is printed instead.
   Ideally lpText will be the result of a strdup(), smprintf(), smprintfpath(), or other malloc() function.
   Example: MessageBoxFree(hWnd,smprintf("Message of unknown length:\r\n%s",str),"Caption",MB_OK); */
int MessageBoxFree(HWND hWnd,TCHAR *lpText,LPCTSTR lpCaption,UINT uType) {
  int rv=MessageBox(hWnd,lpText?lpText:"a memory problem occured",lpCaption,uType);
  if (lpText) freesafe(lpText,"MessageBoxFree");
  return(rv);
}

#if NPPDEBUG /* { */
void testfree(int tofree,char *res,char *realstring,size_t rv) {
  if (res) {
    if (strcmp(res,realstring)) MessageBox(nppData._nppHandle,res,"String Mismatch", MB_OK|MB_ICONSTOP);
    if (tofree) freesafe(res,"testfree");
  } else MessageBox(nppData._nppHandle,"malloc() failed",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
  if (rv != (unsigned)-1 && rv != strlen(realstring)) MessageBoxFree(nppData._nppHandle,smprintf("Mismatch: rv=%u strlen()=%u",rv,strlen(realstring)),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
}

void testsprintfuncs(void) {
  //char *res;
  size_t rv;
  char buf[8];
  char *zbuf,*zbuf2; unsigned zbufsz,zbufsz2,temp,strategy,domalloc,want; int dx;

  for(temp=4; temp<=4096; temp<<=1) for(dx=-1; dx!=2; dx++) for(strategy=0; strategy<=2; strategy++) for(domalloc=0; domalloc<=1; domalloc++) {
    zbufsz=temp+dx;
    zbuf=domalloc?(char *)mallocsafe(zbufsz,"testsprintfuncs"):NULL;
    armreallocsafe(&zbuf,&zbufsz,temp+dx,strategy,0,"testsprintfuncs");
    want=strategy==ARMSTRATEGY_INCREASE?roundtonextpower(temp+dx):temp+dx;
    if (domalloc) want=temp+dx;
    if (strategy==ARMSTRATEGY_INCREASE && want<64) want=64;
    if (zbufsz != want) MessageBoxFree(nppData._nppHandle,smprintf("Mismatch: strategy:%u temp+dx=%u want=%u zbufsz=%u zbufwas=%s",strategy,temp+dx,want,zbufsz,domalloc?"...":"NULL"),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
    freesafe(zbuf,"testsprintfuncs");
  }

  for(temp=1; temp<130; temp++) {
    zbuf=NULL; zbufsz=0;
    sarmprintf(&zbuf,&zbufsz,NULL,"%*s",temp,"X");
    zbuf2=NULL;zbufsz2=0;
    strcpyarmsafe(&zbuf2,&zbufsz2,NULL,zbuf,"testsprintfuncs");
    freesafe(zbuf2,"testsprintfuncs");
    freesafe(zbuf,"testsprintfuncs");
    //zbuf=NULL; zbufsz=0;
    zbuf=NULL; zbufsz=0;
    sarmprintf(&zbuf,&zbufsz,NULL,"%*s",131-temp,"X");
    zbuf2=NULL;zbufsz2=0;
    strcpyarmsafe(&zbuf2,&zbufsz2,NULL,zbuf,"testsprintfuncs");
    freesafe(zbuf2,"testsprintfuncs");
    freesafe(zbuf,"testsprintfuncs");
  }

  //test with smprintf() initial buffer size of 8
  testfree(1,smprintf("Howdy %d partner",57),"Howdy 57 partner",(unsigned)-1); // too long
  testfree(1,smprintf("123456"),"123456",(unsigned)-1); // too short
  testfree(1,smprintf("x2%s","ex"),"x2ex",(unsigned)-1); // too short
  testfree(1,smprintf("1234567"),"1234567",(unsigned)-1); // just right
  testfree(1,smprintfpath("%s%?\\%s%?\\%s","A","B\\","C"),"A\\B\\C",(unsigned)-1);
  rv=snprintfX(buf,sizeof(buf),"123456789ABCDEF");
  testfree(0,buf,"1234567",rv);
  rv=snprintfX(buf,sizeof(buf),"123");
  testfree(0,buf,"123",rv);
}
#endif /* } */

// When enabled, this will ensure that string lengths are being maintained properly by the highly detailed
// string transforms. When they are known to be working, turning off NPPDEBUG will disable all this
// slow code.
#if NPPDEBUG && 1 /* { binary safe functions no longer permit this testing */
void *memmovetest(void *dest,void *src,size_t n) {
  size_t m;
  m=strlen((const char *)src);
  if (m+1 != n) {
#if NPPWINDOWS /* { */
      MessageBoxFree(nppData._nppHandle,smprintf("memmovetest wrong SLN strlen(+1):%u != SLN:%u",strlen((const char *)src)+1,n),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
#else /* } else { */ /* Borland C for DOS */
     printf("Wrong SLN:%u != strlen(+1):%d\n",n,m+1);
#endif /* } */
     n=m;
  }
  return(memmove(dest,src,n));
}
#else /* } else { */
#define memmovetest memmove
#endif /* } */

// unlike other arm routines, destlen is required here because memmovearm() is not interested in appending anything.
// returns the number of bytes *dest was changed by during a realloc() so the caller can adjust their pointers
int memmovearm(char **dest,unsigned *destsz,unsigned *destlen,char *destp,char *sourcep
#if NPPDEBUG
,int notest
#endif
){  unsigned rv=0;
  if (destp != sourcep) {
    if (rv=armreallocsafe(dest,destsz,*destlen+1+destp-sourcep,ARMSTRATEGY_INCREASE,0,"memmovearm")) {
      destp += rv;
      sourcep += rv;
    }
    if (*dest) {
      //memset(*dest+*destlen,'@',*destsz-*destlen); *(*dest+*destlen)='%'; // % is \0
#if NPPDEBUG
      if (notest) memmove(destp,sourcep,(*destlen)-(sourcep-*dest)+1); else
#endif
      memmovetest(destp,sourcep,(*destlen)-(sourcep-*dest)+1);
      *destlen += destp-sourcep;
    }
  }
  return(rv);
}
#if NPPDEBUG
#define memmovearmtest memmovearm
#else
#define memmovearmtest(dt,ds,dl,dp,sp,nt) memmovearm(dt,ds,dl,dp,sp)
#endif

// because so many c string functions must call strlen() before operating, these mem...
// functions are more efficient on huge buffers. If you wish to provide a \0 terminated C-string
// calculate the strlen() yourself as most c string functions do every time they
// are called.
void memcqspnstart(const char *find,unsigned findl,unsigned *quick) {
  unsigned q;
  for(q=0; q<256/(sizeof(unsigned)*8); q++) quick[q]=0;
  while(findl) {
    quick[(unsigned)*(unsigned char *)find/(sizeof(unsigned)*8)] |= 1<<(unsigned)*(unsigned char *)find%(sizeof(unsigned)*8);
    find++;
    findl--;
  }
}

// strcqspn quick version for large find strings
// end=buf+buflen
// for C strings this is the position of the \0
// for buffers, this is 1 character beyond the end (same place)
// returns end when the search fails
char *memcqspn(char *buf,char *end,unsigned *quick) {
  for(; buf<end; buf++) {
    if (quick[(unsigned)*(unsigned char *)buf/(sizeof(unsigned)*8)] & 1<<((unsigned)*(unsigned char *)buf%(sizeof(unsigned)*8))) return(buf);
  }
  return(end);
}

char *memqspn(char *buf,char *end,unsigned *quick) {
  for(; buf<end; buf++) {
    if (!(quick[(unsigned)*(unsigned char *)buf/(sizeof(unsigned)*8)] & 1<<((unsigned)*(unsigned char *)buf%(sizeof(unsigned)*8)))) return(buf);
  }
  return(end);
}

// strcspn: returns the first location in buf of any character in find
char *memcspn(char *buf,char *end,const char *find,unsigned findl) {
  if (findl>1) {
    for(; buf<end; buf++) {
      if (memchr(find,*buf,findl)) return(buf);
    }
  } else if (findl==1 && buf<end) {
    char *rv;
    if (rv=(char *)memchr(buf,*find,end-buf)) return(rv);
  }
  return(end);
}

// strspn: returns the first location in buf of any character in find
char *memspn(char *buf,char *end,const char *find,unsigned findl) {
  if (findl>1) {
    for(; buf<end; buf++) {
      if (!memchr(find,*buf,findl)) return(buf);
    }
  } else if (findl==1 && buf<end) {
    while(buf<end && *buf==*find) buf++; // if I had time I'd implement this as REPE SCASB
    return(buf);
  }
  return(end);
}

char *memstr(char *buf,char *end,const char *find,unsigned findl) {
  if (findl>1) {
    while((buf=(char *)memchr(buf,*find,end-buf)) && buf<end-findl) {
      if (!memcmp(buf+1,find+1,findl-1)) return(buf);
      buf++;
    }
  } else if (findl==1 && buf<end) {
    char *rv;
    if (rv=(char *)memchr(buf,*find,end-buf)) return(rv);
  }
  return(end);
}

// check to see if a number is a power of two
// returns the power+1 such that 1<<(powcheck(num)-1)=num
// returns 0 if the num is not a power of 2, such as when num==0
unsigned powcheck(unsigned num) {
  unsigned test=1,rv=0,pow=0;

  if (num) do {
    pow++;
    if (num==test) {rv=pow; break;}
    test <<=1;
  } while(test);
  return(rv);
}

// SUBSECTION Sort routines
/* I discovered the need for a safer sort than qsort which can blow up the
 * stack with cleverly designed input. I also discovered the need for a
 * stable sort. Merge was chosen for the stable sort and Introsort was
 * chosen for the stack safe qsort. A special version for width==sizeof(int)
 * is provided to better optimize the most common record width. Should
 * anything go wrong, the routines bail to the c-lib qsort(). All the
 * sort routines are designed to be call compatible with qsort(). */
#if 1 && NPPDEBUG
#define movect(num) moves+=(num);
#define setmoves(num) moves=(num);
#define getmoves() (moves)
  unsigned long moves;
#else
#define movect(num)
#define setmoves(num)
#define getmoves() (0LU)
#endif
#define INTROSORTENABLE 1 /* set to 1 for introsort, 0 for original qsort */
#define INTROSTACKLEVEL 16 /* The stack depth permitted before switching to heapsort */
// http://linux.wku.edu/~lamonml/kb.html (GNU GPL version 2)
void _mergesortint(unsigned *pbase,unsigned *ptempo,size_t left,size_t mid,size_t right,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
#define width 1
  unsigned *pleft,*plefte,*pmid,*pright,*ptemp;

  pleft=pbase+left*width;
  pmid=pbase+mid*width;
  plefte=pmid-width;
  ptemp=ptempo;
  pright=pbase+right*width;

  if (pleft <= plefte && pmid <= pright) while(1) {
    if (fcmp(pleft,pmid)<=0) {
      *ptemp=*pleft;
      movect(1)
      ptemp+=width;
      pleft+=width;
      if (pleft>plefte) break;
    } else {
      *ptemp=*pmid;
      movect(1)
      ptemp+=width;
      pmid+=width;
      if (pmid>pright) break;
    }
  }
  while(pleft<=plefte) {
      *ptemp=*pleft;
      movect(1)
      ptemp+=width;
      pleft+=width;
  }
  while(pmid<=pright) {
      *ptemp=*pmid;
      movect(1)
      ptemp+=width;
      pmid+=width;
  }
  memcpy(pbase+left*width,ptempo,(char *)ptemp-(char *)ptempo);
  movect(ptemp-ptempo);
#undef width
}

// this verion isn't much less efficient. The only multiplication is at the beginning
void _mergesort(void *base,void *temp,size_t left,size_t mid,size_t right,size_t width,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  char *pleft,*plefte,*pmid,*pright,*ptemp;

  pleft=(char *)base+left*width;
  pmid=(char *)base+mid*width;
  plefte=pmid-width;
  ptemp=(char *)temp;
  pright=(char *)base+right*width;

  if (pleft <= plefte && pmid <= pright) while(1) {
    if (fcmp((void *)pleft,(void *)pmid)<=0) {
      memcpy(ptemp,pleft,width);
      movect(1)
      ptemp+=width;
      pleft+=width;
      if (pleft>plefte) break;
    } else {
      memcpy(ptemp,pmid,width);
      movect(1)
      ptemp+=width;
      pmid+=width;
      if (pmid>pright) break;
    }
  }
  while(pleft<=plefte) {
      memcpy(ptemp,pleft,width);
      movect(1)
      ptemp+=width;
      pleft+=width;
  }
  while(pmid<=pright) {
      memcpy(ptemp,pmid,width);
      movect(1)
      ptemp+=width;
      pmid+=width;
  }
  memcpy((char *)base+left*width,temp,(char *)ptemp-(char *)temp);
  movect(((char *)ptemp-(char *)temp)/width)
}

void _mergesortsplitint(unsigned *base,unsigned *temp,size_t left,size_t right,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  unsigned mid;

  if (right > left)   {
    mid = (right + left) / 2;
    _mergesortsplitint(base,temp,left ,mid        ,fcmp);
    _mergesortsplitint(base,temp      ,mid+1,right,fcmp);
    _mergesortint(base,temp,left      ,mid+1,right,fcmp);
  }
}

void _mergesortsplit(void *base,void *temp,size_t left,size_t right,size_t width,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  unsigned mid;

  if (right > left)   {
    mid = (right + left) / 2;
    _mergesortsplit(base,temp,left ,mid        ,width,fcmp);
    _mergesortsplit(base,temp      ,mid+1,right,width,fcmp);
    _mergesort(base,temp,left      ,mid+1,right,width,fcmp);
  }
}

void qsortmerge(void *base,size_t nelem,size_t width,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  void *temp;
  if (nelem>1 && width>=1 && (temp=mallocsafe(nelem*width,"msort"))) {
    if (1 && width==sizeof(int)) _mergesortsplitint((unsigned *)base,(unsigned *)temp,0,nelem-1,fcmp);
    else _mergesortsplit(base,temp,0,nelem-1,width,fcmp);
    freesafe(temp,"msort");
  }
}

// http://en.wikipedia.org/wiki/Heapsort
// This unified version is much nicer than the one with the sift() function
void _heapsortint(unsigned *base,unsigned *pivottemp,size_t nelem,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  unsigned int n = nelem, i = n/2, parent, child;

  while(1) {
    if (i) {
      i--;
      *pivottemp = base[i];
    } else {
      n--;
      if (n == 0) return;
      *pivottemp = base[n];
      base[n] = base[0];
    }
    movect(1)

    parent = i;
    child = parent*2 + 1;

    while(child < n) {
      if (child+1<n && fcmp(&base[child + 1],&base[child])>0) {
        child++;
      }
      if (fcmp(&base[child],pivottemp)<=0) break;
        base[parent] = base[child];
        movect(1)
        parent = child;
        child = parent*2 + 1;
    }
    base[parent] = *pivottemp;
    movect(1)
  }
}

void _heapsort(void *base,void *pivottemp,size_t nelem,size_t width,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  unsigned int n = nelem, i = n/2, parent, child;

  while(1) {
    if (i) {
      i--;
      memcpy(pivottemp,(char *)base+i*width,width); //*pivottemp = base[i];
    } else {
      n--;
      if (n == 0) return;
      memcpy(pivottemp,(char *)base+n*width,width); //*pivottemp = base[n];
      memcpy((char *)base+n*width,base,width); //base[n] = base[0];
    }
    movect(1)

    parent = i;
    child = parent*2 + 1;

    while(child < n) {
      if (child+1<n && fcmp((char *)base+(child+1)*width,(char *)base+child*width)>0) {
        child++;
      }
      if (fcmp((char *)base+child*width,pivottemp)<=0) break;
        memcpy((char *)base+parent*width,(char *)base+child*width,width);
        movect(1)
        parent = child;
        child = parent*2 + 1;
    }
    memcpy((char *)base+parent*width,pivottemp,width); // base[parent] = *pivottemp;
    movect(1)
  }
}

void qsortheap(void *base,size_t nelem,size_t width,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  if (nelem>1 && width>=1) {
    if (1 && width==sizeof(int)) {
      unsigned temp;
      _heapsortint((unsigned *)base,&temp,nelem,fcmp);
    } else {
      void *temp;
      if (temp=mallocsafe(width,"qsortheap")) {
        _heapsort(base,temp,nelem,width,fcmp);
        freesafe(temp,"qsortheap");
      } else qsort(base,nelem,width,fcmp);
    }
  }
}

// http://linux.wku.edu/~lamonml/kb.html (GNU GPL version 2)
void _introsortint(unsigned depth,unsigned *base,unsigned *pivottemp,unsigned nelem,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  unsigned *leftpiv=base,*right=base+(--nelem),prn;

  // median of 3: we want the center value in left, the highest value in right, and the lowest value in the center
  // http://www.java2s.com/ExampleCode/Collections-Data-Structure/Quicksortwithmedianofthreepartitioning.htm
  if (nelem>=16) { // less than 16 produces too many extra compares
    unsigned *center=base+nelem/2;
    if      (fcmp(center ,right )>0) {*pivottemp=*right  ;*right  =*center;*center=*pivottemp;movect(3)}
    if      (fcmp(leftpiv,right )>0) {*pivottemp=*leftpiv;*leftpiv=*right ; *right=*pivottemp;movect(3)}
    else if (fcmp(leftpiv,center)<0) {*pivottemp=*leftpiv;*leftpiv=*center;*center=*pivottemp;movect(3)}
    //if (*leftpiv<*center || *leftpiv>*right) printf("F"); // failed to get the lowest value
  }
  *pivottemp = *leftpiv;
  movect(1)
  while(leftpiv<right) {
    if (1) while (1) {
      if (fcmp(right,pivottemp)>=0) {
        if (leftpiv >= --right) break;
      } else {
        *leftpiv = *right;
        movect(1)
        leftpiv++;
        break;
      }
    }
    if (leftpiv<right) while (1) {
      if (fcmp(leftpiv,pivottemp)<=0) {
        if (++leftpiv >= right) break;
      } else {
        *right = *leftpiv;
        movect(1)
        right--;
        break;
      }
    }
  }
  *leftpiv = *pivottemp;
  movect(1)
  prn=leftpiv-base;
  if (prn    > 1) {
    if (INTROSORTENABLE && (depth>INTROSTACKLEVEL/*|| nelem-prn<=2*/)) // if the stack has grown too large, pass off to heap sort
      _heapsortint(base,pivottemp,prn ,fcmp);
    else
      _introsortint(depth+1,base,pivottemp,prn ,fcmp);
  }
  if (nelem > prn+1  ) {
    if (INTROSORTENABLE && (depth>INTROSTACKLEVEL/*|| nelem-prn<=2*/)) // all evidence shows that there is no minimum value where heapsort performs better
      _heapsortint (base+prn+1,pivottemp,nelem-prn,fcmp);
    else
      _introsortint(depth+1,base+prn+1,pivottemp,nelem-prn,fcmp);
  }
}

void _introsort(unsigned depth,void *base,void *pivottemp,unsigned nelem,size_t width,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  char *leftpiv=(char *)base,*right=(char *)base+(--nelem)*width;
  unsigned prn;

  // median of 3: we want the center value in left, the highest value in right, and the lowest value in the center
  // http://www.java2s.com/ExampleCode/Collections-Data-Structure/Quicksortwithmedianofthreepartitioning.htm
  if (nelem>=16) { // less than 16 produces too many extra compares
    char *center=(char *)base+(nelem/2)*width;
    if      (fcmp(center ,right )>0) {memcpy(pivottemp,right,width)  ;memcpy(right,center,width);memcpy(center,pivottemp,width);movect(3)}
    if      (fcmp(leftpiv,right )>0) {memcpy(pivottemp,leftpiv,width);memcpy(leftpiv,right,width);memcpy(right,pivottemp,width);movect(3)}
    else if (fcmp(leftpiv,center)<0) {memcpy(pivottemp,leftpiv,width);memcpy(leftpiv,center,width);memcpy(center,pivottemp,width);movect(3)}
    //if (*leftpiv<*center || *leftpiv>*right) printf("F"); // failed to get the lowest value
  }
  memcpy(pivottemp,leftpiv,width);
  movect(1)
  while(leftpiv<right) {
    if (1) while (1) {
      if (fcmp(right,pivottemp)>=0) {
        right -= width;
        if (leftpiv >= right) break;
      } else {
        memcpy(leftpiv,right,width);
        movect(1)
        leftpiv+=width;
        break;
      }
    }
    if (leftpiv<right) while (1) {
      if (fcmp(leftpiv,pivottemp)<=0) {
        leftpiv += width;
        if (leftpiv >= right) break;
      } else {
        memcpy(right,leftpiv,width);
        movect(1)
        right-=width;
        break;
      }
    }
  }
  memcpy(leftpiv,pivottemp,width);
  movect(1)
  prn=(leftpiv-(char *)base)/width;
  if (prn    > 1) {
    if (INTROSORTENABLE && (depth>INTROSTACKLEVEL/*|| nelem-prn<=2*/)) // if the stack has grown too large, pass off to heap sort
      _heapsort(base,pivottemp,prn ,width,fcmp);
    else
      _introsort(depth+1,base,pivottemp,prn ,width,fcmp); // all evidence shows that there is no minimum value where heapsort performs better
  }
  if (nelem > prn+1  ) {
    if (INTROSORTENABLE && (depth>INTROSTACKLEVEL/* || nelem-prn<=2*/))
      _heapsort ((void *)((char *)base+(prn+1)*width),pivottemp,nelem-prn,width,fcmp);
    else
      _introsort(depth+1,(void *)((char *)base+(prn+1)*width),pivottemp,nelem-prn,width,fcmp);
  }
}

// introsort starts with quicksort and bails to heapsort when the stack gets too large
void qsortintro(void *base,size_t nelem,size_t width,int (QSORT_FCMP *fcmp)(const void *,const void *)) {
  if (nelem>1 && width>=1) {
    if (1 && width==sizeof(int)) {
      unsigned temp;
      _introsortint(0,(unsigned *)base,&temp,nelem,fcmp);
    } else {
      void *temp;
      if (temp=mallocsafe(width,"qsortqsort")) {
        _introsort(0,base,temp,nelem,width,fcmp);
        freesafe(temp,"qsortqsort");
      } else qsort(base,nelem,width,fcmp);
    }
  }
}
//#undef movect
//#undef setmoves
//#undef getmoves

// Windows File I/O routines are a hassle. Here's some wrappers that make them no harder to use than MS-DOS
#define O_DENYWRITE FILE_SHARE_WRITE
#define O_DENYREAD FILE_SHARE_READ
#define O_DENYALL (FILE_SHARE_READ|FILE_SHARE_WRITE)
#define O_DENYNONE 0
#define O_RDONLY GENERIC_READ
#define O_WRONLY GENERIC_WRITE
#define O_RDWR (GENERIC_READ|GENERIC_WRITE)
#define O_BINARY 0
#define _openX(filename,oflags,shflags) CreateFile(filename,oflags,shflags,NULL,OPEN_EXISTING,0,NULL)

#define _creatX(filename,attrib) CreateFile(filename,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,attrib,NULL)

DWORD _writeX(HANDLE handle,const void *buf,DWORD len) {
  DWORD nr;

  WriteFile(handle,buf,len,&nr,NULL);
  return(nr);
}

DWORD _readX(HANDLE handle,void *buf,DWORD len) {
  DWORD nr;

  nr=0;
  ReadFile(handle,buf,len,&nr,NULL);
  return(nr);
}

BOOL _closeX(HANDLE hnd) {
  return(CloseHandle(hnd));
}

// SUBSECTION End
// SECTION: END
// SECTION: Beginning of Transforms

// ****************************************************************

/*Many of the following functions have common parameters. We'll describe them here.
  Each of these functions receives a c-string text buffer, performs a transform,
  then returns the number of changes made. convertall()'s job is to allow
  these text transforms to operate on editor text without knowing any of the details for
  accessing it within the Scintilla editor component. Considering that I have debugged all
  of the text transforms in Borland C++ 3.1 for DOS, you should be able to reuse them in
  any project.
dest:
  points to the start of the buffer. If the transform may need to expand or pass back
  a totally different buffer is a pointer to the start of the buffer.
destsz:
  This is the total size of the buf when it was malloc()'d. The transform may need to
  expand or replace dest and must return the size of any alteration in destsz.
destlen:
  destlen is the C-string length of the text in dest. Transforms that expand or contract
  the C-string length must adjust destlen.
stopeol
  Some functions can be told to perform only on the current line. Pass in the
  start of the line to be processed in *stopeol to transform until a \r or \n is hit.
  *stopeol return the the point stopped at, typically the \r\n or \0.
  . *stopeol must point
  between buf and buf+*strsln or it will be forcibly reset. While *strsln is not required
  for processing a single line, usually the caller asking for a line to be processed
  has the length available and will be needing length returned.
  Pass NULL as stopeol if you want the entire buffer to be processed.

Some writers claim that Java string handling is so good that it can be faster C. What
they don't mention because they don't know is why C performs so poorly. Tracing through a few
common C string functions shows they are fine for small strings but for multi-megabyte buffers they are extremely
inefficient. Some libraries implement the strcpy(dest,source) function as
memcpy(dest,source,strlen(source)+1) which causes a double scan of the string. A function like
strchr(str,ch) implemented as memchr(str,ch,strlen(str)+ch?0:1) repeated many times through
the string to repeatedly find a particular character
would generate a huge amount of wasted string scans to find the length. Using the mem
function instead we only need to calculate the length of the string a single time which may
already be available
and every mem function can use it. Switching the code from str to mem functions not only allowed
TextFX to support embedded \0 in strings, it made every compiler perform equally well.
Because of certain string library design choices which aren't wrong, MinGW was a
terrible performer with str functions. With mem functions MinGW is a superb performer.

  I don't use function declarations that need to be updated each time a function call's
  parameters are changed. Instead I order the functions so they are created before their first use.
  Less code == less trouble!
*/

unsigned meminvertcase(char *str,unsigned destlen) {
  char *end;
  unsigned n=0;

  if (str) for(end=str+destlen; str<end; str++) {
    if (islower(*str)) {*str = toupper(*str); n++; }
    else if (isupper(*str)) {*str = tolower(*str); n++; }
  }
  return(n);
}

//returns number of characters altered
unsigned memsentencecase(char *str,unsigned destlen) {
  char *end;
  unsigned n=0;
  int waspunct=TRUE;

  if (str) for(end=str+destlen; str<end; str++) {
    if (islower(*str)) {if (waspunct) {*str = toupper(*str); n++;}}
    else if (isupper(*str)) { if (!waspunct) {*str = tolower(*str); n++;}}
    else if (*str=='.' || *str=='!' || *str=='?') {if (isspace(str[1])) waspunct=TRUE;}
    if (isalnum(*str)) waspunct=FALSE;
  }
  return(n);
}

//returns number of characters altered
unsigned mempropercase(char *str,unsigned destlen) {
  char *end;
  unsigned n=0;
  int wasspace=TRUE;

  if (str) for(end=str+destlen; str<end; str++) {
    if (wasspace && islower(*str)) {*str = toupper(*str); n++; }
    else if (!wasspace && isupper(*str)) {*str = tolower(*str); n++; }
    wasspace=!isalnum(*str);
  }
  return(n);
}

// Single character CHRT(), returns number replaced
unsigned memchrtran1(char *str,unsigned destlen,char cfind,char creplace) {
  char *end;
  unsigned n=0;

  if (str && cfind != creplace) for(end=str+destlen; str<end && (str=(char *)memchr(str,cfind,end-str)); ) {
    *str=creplace;
    n++;
  }
  return(n);
}

// a full implementation of Foxpro's CHRTRAN() function
// Case Sensitive Replace any character found in
// *find with the character at the coresponding position in *repl
// if strlen(repl)<strlen(find) extra chars will be deleted
unsigned memchrtran(char **dest,unsigned *destsz,unsigned *destlen,const char *find,unsigned lfind,const char *repl,unsigned lrepl) {
#define lold 1
#define lnew 0
  char *d,*end;
  unsigned n=0,fpn,quick[256/(sizeof(unsigned)*8)];

  if ((d=*dest) && lfind && (lfind!=lrepl || memcmp(find,repl,lfind))) for(memcqspnstart(find,lfind,quick),end=*dest+*destlen,lrepl=strlen(repl); (d=memcqspn(d,end,quick))<end; ) {
    fpn=(const char *)memchr(find,*d,lfind)-find;
    if (fpn>=lrepl) {
      d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
      end=*dest+*destlen;
    } else {
      *d = repl[fpn];
      d++;
    }
    n++;
  }
failbreak:
  return(n);
#undef lold
#undef lnew
}

//a full implementation of Foxpro's STRTRAN()
// a non regex search and replace for c-strings
unsigned memstrtran(char **dest,unsigned *destsz,unsigned *destlen,char **stopeol,const char *oldst,unsigned lold,const char *newst,unsigned lnew) {
  unsigned n=0;
  char *d,*end;
  int dx;

  //DWORD tk=GetTickCount();
  if ((d=*dest) && lold && (lold!=lnew || memcmp(oldst,newst,lold))) {
    end=d+*destlen;
    if (stopeol) end=memcspn(d=*stopeol,end,"\r\n",2);
    while((d=memstr(d,end,oldst,lold))<end) {
      if (lnew != lold) {
        dx=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak; // can't test this. using stopeol always has an intentional \0 in the string.
        d+=dx;
        end+=dx+lnew-lold;
      }
      if (lnew) {
        memcpy(d,newst,lnew);
        d+=lnew;
      }
      n++;
    }
    if (stopeol) *stopeol=d;
  }
failbreak:
  //MessageBoxFree(nppData._nppHandle,smprintf("Tick Count: %u",GetTickCount()-tk),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
  return(n);
}

//strchrstrans: Replaces any in a set of single characters with corresponding strings
//adapted from memstrtran() && memchrtran()
//chrs: the set of chars to search for, and all chrs must be found in repls
//repls: deli + setof[char+string+deli] the first character of repls is the delimiter
//which must be chosen such that it doesn't occur in the data of repls
//stopeol: process only a single line, provide a pointer to the line beginning and it will return a pointer to the line ending \r\n
//  NULL to process entire str, set to NULL if starting at beginning,
// if using stopeol mode, slndx is required and returns the amount that the string was reduced by moving forwards minus alterations so the caller can update their counters
unsigned strchrstrans(char **dest,unsigned *destsz,unsigned *destlen,char **stopeol,const char *chrs,unsigned chrsl,const char *repls) {
#define lold 1
  unsigned n=0,lnew;
  char *d,*newst,*dnew,ckst[3],*end;
  unsigned quick[256/(sizeof(unsigned)*8)];

  if ((d=*dest) && lold && chrsl) {
    end=d+*destlen;
    if (stopeol) end=memcspn(d=*stopeol,end,"\r\n",2);
    ckst[0]=*repls;
    ckst[2]='\0';
    memcqspnstart(chrs,chrsl,quick);
    while ( (d=memcqspn(d,end,quick))<end) {
      ckst[1]=*d;
      if (!(newst=(char *)strstr(repls,ckst))) {
#if NPPDEBUG /* { */
        MessageBoxFree(nppData._nppHandle,smprintf("strchrstrans: Improper String Format (#%d)\r\n%s",1,repls),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
#endif /* } */
        goto failbreak;
      }
      newst+=2;
      if (!(dnew=strchr(newst,*repls))) { // find the termination character (typically ;)
#if NPPDEBUG /* { */
        MessageBoxFree(nppData._nppHandle,smprintf("strchrstrans: Improper String Format (#%d)\r\n%s",2,repls),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
#endif /* } */
        goto failbreak;
      }
      lnew=dnew-newst;
      if (lnew != lold) {
        d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
      }
      if (lnew) {
        memcpy(d,newst,lnew);
        d+=lnew;
      }
      n++;
    }
    if (stopeol) *stopeol = d;
  }
failbreak:
  return(n);
#undef lold
}

// For each line:
// adds begin to the start, replaces strings in the middle, then adds end on the end
// the spaces in the first line are counted to produce an indent for remaining lines
// Some transforms are more complicated than others. func=0:memstrtran, func=1:strchrtrans
unsigned prepostpendlines(char **dest,unsigned *destsz,unsigned *destlen,int func,const char *begin,unsigned lbegin,const char *end,unsigned lend,const char *oldst,unsigned loldst,const char *repl,unsigned lrepl) {
#define lold 0
  unsigned n=0;
  char *d,*detent;
  int indent,indent2;

  if ((d=*dest) && loldst) for (indent=-1; *d; ) {
    while(*d=='\r' || *d=='\n') d++;
    if (indent<0) for(indent=0; isspace(*d); d++, indent++); // calc first indent
    else for(indent2=indent; indent2 && isspace(*d); d++, indent2--);  // all future indents are at the same place as the first
    if (!*d) break;
#define newst begin
#define lnew lbegin
    if (lnew != lold) {
      d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
    }
    if (lnew) {
      memcpy(d,newst,lnew);
      d+=lnew;
    }
#undef lnew
#undef newst
    detent=d; // this ensures the optimizer can assign a register to d, &variables are always placed on the stack
    n+=(!func?memstrtran(dest,destsz,destlen,&detent,oldst,loldst,repl,lrepl):
         strchrstrans(dest,destsz,destlen,&detent,oldst,loldst,repl));
    d=detent;
    if (!*dest) goto failbreak;
#define newst end
#define lnew lend
    if (lnew != lold) {
      d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
    }
    if (lnew) {
      memcpy(d,newst,lnew);
      d+=lnew;
    }
#undef lnew
#undef newst
    n++;
  }
failbreak:
  return(n);
#undef lold
}

#if NPPDEBUG /* { */
unsigned pass; // this is only used when rewraptest is being tested by the test function below
#endif /* } */

// textwid=0 to unwrap completely, 1 or other invalid width for default of 72
// All removed: multiple whitespace, non space whitespace including tabs,
//   newlines not conforming to eoltype, extra newlines, more than double lines
// unwrapping will break to another paragraph at a double line, \r\n\r\n, \r\r, or \n\n
// (It may look easy now but this function took 3 straight days to get it working right.)
// eoltype is retrieved from N++ and is used with eoltypes[]
// new feature: wrap text looks at the spacing of the first line to space other lines
// new feature: wrap avoids line comments
// new feature: indent properly for bullets
// Instead of implementing all of the above and more, there are full featured open source text formatters
// available which could be inserted.
// new feature: this no longer wraps correctly, probably becuause of **problem** below

const char eoltypes[][4]={"\r\n","\r","\n"}; // could be 3 but 4 makes the compiler use SHL,2 instead of IMUL*3
unsigned rewraptext(char **dest,unsigned *destsz,unsigned *destlen,unsigned tw,unsigned eoltype) {
  unsigned lold,lnew;
  char *d,*end,*dp,*lastspace,*newst,temp[6]; // largest=" \r\n\r\n"
  unsigned flags; // The optimizer handles bit flags much better than separate variables
#define FLAG_QUIT 1
#define FLAG_WASSPACE 2
#define FLAG_MODIFY 4

  if (*dest) {
    if (eoltype>=NELEM(eoltypes)) eoltype=NELEM(eoltypes)-1;
    if (tw && (tw<8 || tw>2048)) tw=72; // values picked to keep within reasonable editor limits
    for(d=*dest,end=d+*destlen; d<end; ) {
      dp=d;
      lastspace=NULL;
      flags=0;
      do {
        switch(*d) {
        case '\r':
        case '\n':
          if (d[0]==d[1] || d[0]=='\r' && d[1]=='\n' && d[2]=='\r' && d[3]=='\n') {
            flags |= FLAG_QUIT;
            break;
          }
        case '\t':
          flags |= FLAG_MODIFY; // modify anything other than a space
        case ' ':
          if (!(flags&FLAG_WASSPACE)) lastspace=d;
          flags |= FLAG_WASSPACE;
          if (d==dp || d>lastspace || (flags&FLAG_MODIFY)) {
            flags |= FLAG_MODIFY; // modify anything more than 1 whitespace
            d++;
            continue; // colasce whitespace
          }
          break;
        case '\0':
          flags |= FLAG_QUIT;
          break;
        default:
          flags &= ~FLAG_WASSPACE;
          break;
        }
        if ((flags&FLAG_QUIT) || (tw && !(flags&FLAG_WASSPACE) && (unsigned)(d-dp)>=(end-dp<tw?end-dp:tw))) { // **problem** was sln
          if (flags&FLAG_QUIT) {
            if (lastspace && (flags&FLAG_WASSPACE)) d=lastspace;
            else lastspace=d;
          }
          if (lastspace) {
            d=lastspace;
            lold=strspn(d,"\r\n\t ");
          } else lold=0;
          newst=temp;
          memcpy(temp," ",2);
          memcpy(newst+1,"\r\n" /*eoltypes[eoltype]*/,3);
          if (flags&FLAG_QUIT) strcat(newst,"\r\n" /*eoltypes[eoltype]*/);
          lnew=strlen(newst);
          flags |= FLAG_QUIT; //=FLAG_QUIT seems to work too
        } else if (flags & FLAG_MODIFY) {
          lold=d-lastspace;
          d=lastspace;
          newst=" ";
          lnew=1;
          flags &= ~FLAG_MODIFY; // =0 seems to work too
        } else {
          d++;
          continue;
        }
        if (lnew != lold) {
          d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
          end = *dest + *destlen;
        }
        if (lnew) {
          memcpy(d,newst,lnew);
          d+=lnew;
        }
      } while(!(flags&FLAG_QUIT));
    }
    return(1); // it is impractical to calculate the actual number of changes
  } else {
failbreak:
    return(0);
  }
#undef FLAG_QUIT
#undef FLAG_MODIFY
#undef FLAG_WASSPACE
}

#if NPPDEBUG /* { */
#if !(NPPWINDOWS) /* { */
// Under DOS, this prints out the literal characters so space, \r, and \n are visible
void printout(char *st,char *nl) {
  for(;*st; st++) switch(*st) {
  case '\r':
    fputs("\\r",stdout);
    break;
  case '\n':
    printf("\\n%s",nl);
    break;
  case ' ':
    fputs(" ",stdout);
    break;
  default:
    putchar(*st);
    break;
  }
  puts("");
}
#endif /* } */

// to test rewraptest, we wrap the text then wrap it again. The result should be the same both times.
// Extended testing can vary the text width from lowest to highest to catch the edge cases.
// the pass variable can be used to insert dummy code for breakpoints in case the second pass is
// where the problem is
unsigned rewraptexttest(char **dest,unsigned *destsz,unsigned *destlen,unsigned textwid,unsigned eoltype) {
  char *res;
  unsigned rv=0;

  pass=1; // add code to check breakpoints for the pass number
  rv+=rewraptext(dest,destsz,destlen,textwid,eoltype);
  if (*dest) {
    if (res=strdupsafe(*dest,"rewraptexttest")) {
      pass=2; // wrap again the wrapped text and we should get the same result
      rv+=rewraptext(dest,destsz,destlen,textwid,eoltype);
      if (strcmp(res,*dest)) {
#if NPPWINDOWS /* { */
        MessageBoxFree(nppData._nppHandle,smprintf("Improper Second Conversion at wrap %d",textwid),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
#else /* } else { */
        printf("Improper second conversion at wrap %d\n",textwid);
        printout(res,"");
        printf("To:\n");
        printout(*dest,"");
#endif /* } */
        freesafe(*dest,"rewraptexttest");
        *dest=NULL;
      }
      freesafe(res,"rewraptexttest");
    } else MessageBox(nppData._nppHandle,"Out of memory at rewraptexttest",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
  } else MessageBox(nppData._nppHandle,"Wrap memory failed at rewraptexttest",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
  return(rv);
}
#else /* } else { */
#define rewraptexttest rewraptext
#endif /* } */

const char ENCODEURISTR[]="!'()*-.0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz~";
// Same function as JavaScript's encodeURIcomponent()
// component refers to the fact that this is not expected to encode a complete URL,
// only a component of it
unsigned encodeURIcomponent(char **dest,unsigned *destsz,unsigned *destlen) {
  unsigned n=0;
  char *d,*end,newst[4];
  unsigned quick[256/(sizeof(unsigned)*8)];
#define lold 1
#define lnew (sizeof(newst)-1)

  if ((d=*dest) && lold) {
    char *newdest=NULL,*newlast=*dest; unsigned newsz=*destsz*2,newlen=0; // only used for HighPerformance
  	for(end=d+*destlen,memcqspnstart(ENCODEURISTR,sizeof(ENCODEURISTR)-1,quick); (d=memqspn(d,end,quick))<end;) {
      snprintfX(newst,sizeof(newst),"%%%2.2X",*(unsigned char *)d);
      if (HighPerformanceon || HighPerformance) {
        if (newlast<d) {memcpyarmsafe(&newdest,&newsz,&newlen,newlast,d-newlast,"encodeURIcomponent"); if (!newdest) goto freebreak;}
        newlast=(d+=lold);
        armreallocsafe(&newdest,&newsz,newlen+lnew+1 /* 1 provides space for \0 below*/,ARMSTRATEGY_INCREASE,0,"encodeURIcomponent"); if (!newdest) goto freebreak;
        memcpy(newdest+newlen,newst,lnew);
        newlen+=lnew;
        //end=*dest+*destlen;
      } else {
        if (lnew != lold) {
          d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
          end=*dest+*destlen;
        }
        if (lnew) {
          memcpy(d,newst,lnew);
          d+=lnew;
        }
      }
      n++;
    }
    if (HighPerformanceon || HighPerformance) {
    	if (n) {
        if (newlast<d) strncpyarmsafe(&newdest,&newsz,&newlen,newlast,d-newlast,"encodeURIcomponent");
        else newdest[newlen]='\0'; // space provided for up above
freebreak:
        freesafe(*dest,"encodeURIcomponent");
        if (!(*dest=newdest)) n=0;
        *destsz=newsz;
        *destlen=newlen;
      } else if (newdest) freesafe(newdest,"encodeURIcomponent");
    }
  }
failbreak:
  return(n);
#undef lnew
#undef lold
}

// These work for Notepad++ now, later they will be modified to make the code compatible with Code::Blocks by calling currentEdit->WndProc()
#define SENDMSGTOCED(which,mesg,wpm,lpm) SendMessage(((which)?nppData._scintillaSecondHandle:nppData._scintillaMainHandle),mesg,(WPARAM)(wpm),(LPARAM)(lpm))
#define INT_CURRENTEDIT int currentEdit
#define GET_CURRENTEDIT SendMessage(nppData._nppHandle, WM_GETCURRENTSCINTILLA, 0, (LPARAM)&currentEdit)
#define INVALID_CURRENT_EDIT (-1)

// dest = space(n)+source
// like sprintf(dest,"%-*s%s",n,"",source) returns number of characters written
// eventually this will be replaced with smprintfpath() when it gets space padding features
unsigned strcatspacel(char *dest,const char *source,size_t n) {
  memset(dest,' ',n);
  strcpy(dest+n,source);
  return(n+strlen(source));
}

// This function inserts text or an incrementing counter at a specified line position in text lines
// or a specified +-offset1 from the end of the line
// search format [\][$[+-offset.from.end$]][#[startval[+-adder]]/text
// preceed with \ to ensure that $ and # and future symbols are interpreted as text and not a
// preceed with \\ to start your text with a \ (backslash)
// #counter or $offset.from.end. text is not used if a #counter is provided.
// lines that are too short are skipped
// Examples:
// "text" insert text
// "#" insert 0..1..2..3...
// "#5" insert 5..6..7..8..9... (the default increment is +1)
// "#5+2" insert 5..7..9..11...
// "$#5-2" insert 5..3..1..-1... at end of line
// "$$M$soft" insert M$soft at end of line
// "$4$text" insert text 4 characters beyond end of line. line will be ammended as necessary
// "$-3$#-5-2" insert -5..-7..-9..-11... 4 characters before end of line. lines that are too short will be skipped
// new feature: insert text after comma number
unsigned insertclipboardcolumn(char **dest,unsigned *destsz,unsigned *destlen,const char *search,int offset1) {
  unsigned n=0,lnew,lsearch;
  char *d,*end,*cmd,nbuf[32],buf[32],*newst;
  const char *stmp;
#define lold 0
  unsigned flags;
#define FLAG_COUNTING 1
#define FLAG_HEX 2
#define FLAG_END 4
  int offset,cstart,cdx;

  if (*dest && *search) {
    flags=0;
    do {
      unsigned chn2;
      switch(*search) {
      case '$':
        flags |= FLAG_END;
        search++;
        if (stmp=strchr(search,'$')) {
          offset1=atoi(search);
          search=stmp+1;
        } else offset1=0;
        continue;
      case '\\': search++; break;
      case '#':
        flags |= FLAG_COUNTING;
        search++;
        if (*search=='x') {flags |= FLAG_HEX; search++;}
        if (*search=='-') {cstart=-1; search++;} else cstart=1;
        cdx=0;
        if (cmd=strdupsafe(search,"insertclipboardcolumn")) {
          chn2=strcspn(cmd,"+-");
          if (cmd[chn2]=='+') cdx=1;
          else if (cmd[chn2]=='-') cdx=-1;
          cstart *= atoi(strtok(cmd,"+-"));
          cdx    *= atoi(strtok(NULL,"+-"));
          memcpy(buf,"0",2);
          search=buf;
          freesafe(cmd,"insertclipboardcolumn");
        }
        if (!cdx) cdx=1;
        break;
      }
      break;
    } while(1);
    if (lsearch=strlen(search)) {
      unsigned chn;
      for(d=*dest,end=d+*destlen; d<end; ) {
        chn=memcspn(d,end,"\r\n",2)-d;
        offset=offset1;
        if (flags & FLAG_COUNTING) {
          lnew=snprintfX(nbuf,sizeof(nbuf),(flags&FLAG_HEX)?"%X":"%d",cstart); // this violates const rules but we aren't modifiying the original
          newst=nbuf;
          cstart += cdx;
        } else {
          newst=(char *)search; // we promise not to modify this
          lnew=lsearch;
        }
        if (flags&FLAG_END) {
          if (offset>0) {
            lnew=strcatspacel(buf,newst,offset);
            newst=buf;
            offset=chn;
          } else if (offset>=0 || chn>= (unsigned)(-offset)) {
            offset+=chn;
          } else {
            lnew=0;
          }
        } else if (offset<0 || (unsigned)offset>chn) {
          lnew=0; // lold=0;
        }
        d+=offset;
        chn-=offset;
        if (lnew != lold) {
          d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
          end=*dest+*destlen;
          chn += lnew-lold;
        }
        if (lnew) {
          memcpy(d,newst,lnew);
          n++;
        }
        d += chn;
        while(d<end && (*d=='\r' || *d=='\n')) d++;
      }
    }
  }
failbreak:
  return(n);
#undef lold
}

unsigned numberconvert(char **dest,unsigned *destsz,unsigned *destlen,char from,char to) {
  unsigned rv=0;
  char *prefix,temp[128];
  if (dest) {
    long num;
    switch(from) {
    case 'c': num=strtol(*dest,NULL,0); break;
    case 'd': num=strtol(*dest,NULL,10); break;
    case 'h': num=strtol(*dest,NULL,16); break;
    case 'b': num=strtol(*dest,NULL,2); break;
    case 'o': num=strtol(*dest,NULL,8); break;
    default: goto failbreak;
    }
    switch(to) {
    case 'd': ltoa(num,temp,10); prefix=""; break;
    case 'h': ltoa(num,temp,16); prefix="0x"; break;
    case 'b': ltoa(num,temp,2); prefix="$"; break;
    case 'o': ltoa(num,temp,8); prefix="0"; break;
    default: goto failbreak;
    }
    *destlen=0;
    sarmprintf(dest,destsz,destlen,"%s%s",prefix,temp);
    if (*dest) rv=1;
  }
failbreak:
  return(rv);
}

/* case insenstive strstr() */
char *memistr(char *str,char *end,char *search,unsigned lold) {
  char *d,fch[3];

  if (lold) {
    for(d=search+lold-1; d>=search; d--) if (tolower(*d)!=toupper(*d)) goto stristryes;
    //return(lold==1?strchr(str,*search):strstr(str,search));
    return(memstr(str,end,search,lold));
stristryes:
    fch[0]=tolower(*search);
    fch[1]=toupper(*search);
    fch[2]='\0';
    d=str;
    goto bottest; while(d<end) {
      if (!memicmp(d,search,lold)) return(d);
      d++;
bottest:
      if (fch[0]==fch[1]) {
        if (!(d=(char *)memchr(d,*search,end-d))) break;
      } else {
        d=memcspn(d,end,fch,lold);
      }
    }
  }
  return(NULL);
}

// the search will be faster if like first characters are placed together
// the search will be faster if sizeof(_MSTR) is a power of two
struct _MSTR {
// eventually, mstr[0] will be eliminated completely.
// eventually, string will be a pointer and not a buffer
  char string[27]; // nocase=0: 1+the number of different search first characters
                   // nocase=1: 1+the number of different search first characters with an extra for characters who's case differs
  unsigned char nocase; // Each search is individually case sensitive. The entire table is handled case insensitive until we are iterating through possible matches then pick it out case sensitive if requested.
  char rv;
  unsigned char len; // filled in by the initializer
  // proposed features for faster string searching
  unsigned char lennext; // how many characters are common between the current string and the next string
  unsigned char lenskip; // We can skip lenskip+1 forward if string[lennext-1] does not match. currentpos+lenskip+1 is permitted to exceed NELEM. If !lenskip perform a memcmp()
  //unsigned char minlength; // the smallest length in the lenskip+1 to be skipped. This enables bsearch() to locate the nearest starting location using characters up to the smallest length
} mstring[]={
{""                  ,0,0 ,0,0,0}, // mstring[0] is where the search string is stored from call to call
{"\t"                ,0,7 ,0,0,0}, // set case insensitive strings to 0 for better efficiency
{"\n"                ,0,8 ,0,0,0},
{"\r"                ,0,8 ,0,0,0},
{" "                 ,0,7 ,0,0,0},
{"&#"                ,0,0 ,0,0,0},
{"&AMP;\0&"          ,1,1 ,0,0,0},
{"&COPY;\0(C)"       ,1,1 ,0,0,0},
{"&GT;\0>"           ,1,1 ,0,0,0},
{"&lT;\0<"           ,1,1 ,0,0,0},
{"&NBSP;\0 "         ,1,1 ,0,0,0},
{"&quot;\0\""        ,1,1 ,0,0,0},
{"<!--\0-->"         ,0,2 ,0,0,0},
{"</CAPTION"         ,1,3 ,0,0,0},
{"</H"               ,1,3 ,0,0,0},
{"</LI"              ,1,3 ,0,0,0},
{"</PRE"             ,1,6 ,0,0,0},
{"</P"               ,1,3 ,0,0,0},
{"</TEXTAREA"        ,1,6 ,0,0,0},
{"</TITLE"           ,1,3 ,0,0,0},
{"</TR"              ,1,3 ,0,0,0},
{"<BR"               ,1,3 ,0,0,0},
{"<HR"               ,1,3 ,0,0,0},
{"<PRE"              ,1,5 ,0,0,0},
{"<P"                ,1,3 ,0,0,0},
{"<STYLE\0</STYLE>"  ,1,2 ,0,0,0},
{"<SCRIPT\0</SCRIPT>",1,2 ,0,0,0},
{"<TD"               ,1,4 ,0,0,0},
{"<TEXTAREA"         ,1,5 ,0,0,0},
{"<TH"               ,1,4 ,0,0,0},
{"<\0>"              ,0,2 ,0,0,0},
};

//int mstrnocase=0;
//char *mstrxst=NULL; // if ax or bx is NULL, compare the non null with mstrxst

int QSORT_FCMP fcmpmstrsort(const void *ax,const void *bx) {
  int acu,bcu;
#if 0 /* { Each time I try to use bsearch() I fail because of differing string lengths */
  struct _MSTR *am=ax;
  struct _MSTR *bm=bx;
  //unsigned const char *ac,*bc;
  if (mstrnocase) {
    dif=toupper(am->string[0])-toupper(bm->string[0]);
  } else {
    dif=am->string[0]-bm->string[0];
  }
  if (dif) return(dif);
  dif=bm->len-am->len;
  if (dif) return(dif);
  return(stricmp(am->string,bm->string));
  //        return(stricmp(((struct _MSTR *)ax)->string,((struct _MSTR *)bx)->string));
#else /* } else { */
  char *ac,*bc;
  for(ac=((struct _MSTR *)ax)->string,bc=((struct _MSTR *)bx)->string; 1; ac++, bc++) {
    //if (mstrnocase) {
      acu=toupper(*ac);
      bcu=toupper(*bc);
    /*} else {
      acu=*ac;
      bcu=*bc;
    }*/
    if (acu == bcu) {
      if (!acu) return(0);
    } else if (!bcu) return(-1); // the longer string is always sorted first
    else if (!acu) return(1); // the shorter string is always sorted last
    else return(acu - bcu);
  }
#endif /* } */
}

#if 0 /* { */
int QSORT_FCMP fcmpmstrbsearch(const void *ax,const void *bx) {
//struct _MSTR *axm=ax;struct _MSTR *bxm=bx;
  char *ac,*bc;
  int acu,bcu;
  if (ax && bx) { // in Borland-C, ax is always the key
    MessageBox(nppData._nppHandle,"Impossible",PLUGIN_NAME, MB_OK|MB_ICONWARNING);
    exit(1);
  }
  ac=mstrxst;
  bc=ax?(((struct _MSTR *)ax)->string):(((struct _MSTR *)bx)->string);
  while(1) {
    if (mstrnocase) {
      acu=toupper(*ac);
      bcu=toupper(*bc);
    } else {
      acu=*ac;
      bcu=*bc;
    }
    if (acu == bcu) {
      if (!acu) return(0);
    } else if (!bcu) return(0); // they are considered equal if the right string ends
    //else if (!acu) return(-1);
    //#error If we are at the largest of a run of equals, forward, else subtract
    else return(bx?(acu-bcu):(bcu-acu));
    ac++;
    bc++;
  }
}
#endif /* } */

// returns the number of initial characters that are equal
size_t streqct(char *t1,char *t2) {
  size_t rv=0;
  while(*t1 && *t1==*t2) {rv++; t1++; t2++; }
  return(rv);
}

// compute the skip values in the mstring array
void strmstrinit(struct _MSTR *msearch,unsigned nsearch,unsigned *cs/*,int nocase*/) {
  unsigned ch,chn;
  unsigned i,dn,up,lsearch;
#if NPPDEBUG /* { */
  if (!powcheck(sizeof(struct _MSTR)))
    MessageBoxFree(nppData._nppHandle,smprintf("sizeof(struct _MSTR)==%u is not a power of two",sizeof(struct _MSTR)),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
#endif /* } */
  //mstrnocase=nocase;
  for(i=1; i<nsearch; i++) {msearch[i].len=strlen(msearch[i].string); msearch[i].lenskip=0;}
  qsort(msearch+1,nsearch-1,sizeof(*msearch),fcmpmstrsort);
  for(i=2; i<nsearch; i++) msearch[i-1].lennext=streqct(msearch[i-1].string,msearch[i].string);
  for(i=1; i<nsearch; i++) {
    if ((chn=msearch[i].lennext) && !(msearch[i].lenskip)) {
      for(dn=1; dn<256 && dn+i<nsearch && msearch[dn+i].lennext>=chn; dn++);
      for(up=dn; up!=(unsigned)-1; up--) if (msearch[i+up].lennext==chn) msearch[i+up].lenskip=dn-up;
    }
  }
  //for(i=1; i<nsearch; i++) printf("{\"%24s\",%3d,%3d,%3u,%3u,%3u},\n",msearch[i].string,msearch[i].nocase,msearch[i].rv,msearch[i].rv,msearch[i].len,msearch[i].lennext,msearch[i].lenskip);
  for(i=0; i<256; i++) {cs[i]=(unsigned)-1; cs[i+256]=0; }
  memset(msearch,0,sizeof(msearch[0]));
  for(lsearch=0, i=1; i<nsearch && lsearch<sizeof(msearch[0].string); i++) {
    ch=(unsigned)msearch[i].string[0];
    if (/*nocase && */(tolower(ch)!=toupper(ch))) {
      if (cs[tolower(ch)]>i) cs[tolower(ch)]=i;
      if (cs[tolower(ch)+256]<i) cs[tolower(ch)+256]=i;
      if (cs[toupper(ch)]>i) cs[toupper(ch)]=i;
      if (cs[toupper(ch)+256]<i) cs[toupper(ch)+256]=i;
    } else {
      if (cs[ch]>i) cs[ch]=i;
      if (cs[ch+256]<i) cs[ch+256]=i;
    }
    if (msearch[i].len && !strchr(msearch[0].string,ch)) {
      msearch[0].string[lsearch++]=toupper(ch);
      if (/*nocase &&*/ tolower(ch)!=toupper(ch)) msearch[0].string[lsearch++]=tolower(ch);
    }
  }
  msearch[0].len=lsearch;
}

// When mstring search gets more advanced the close function will need to clean some stuff up.
#define strmstrclose(mstr)

// This multiple substring match evolved from humble beginnings:
// Try #1: if (stricmp()) {} else if (strcmp()) {} else if (stricmp())...
// Try #2: strcmp() through table to find a match
// Try #3: Iterate through sorted table from quick[ch] to quick[ch+256] to find a match
// Try #4: "Boyer-Moore Prefix Vector Skip Table" because smaller prefixes have greater skip potential
// Purpose: Search through a long non delimited string for the
// largest possible (greedy) of multiple case sensitive or insensitive substrings.
//
// There is a lot of good theory on multiple substring search which someone better than I may
// be able to implement.
// Returns: 0,p=NULL if not found, or mstring position 1...nsearch-1,p=location if found
//
// We could eliminate sln by padding the string with as many \0 as the longest substring
// This would eliminate the necessity of string scan to calculte strlen().
// The trouble is that the caller has to manage and possibly move the padded \0's and this may
// be more trouble than the REPNZ SCASB that calculates strlen(). Often times
// the caller already needs strlen() for something else so has it readily available.

// new feature: Implement bsearch() and minlength through lenskip to reduce iteration
// new feature: Allow for active use with deletestring, deletestringmultiple, insertstring and insertstringmultiple with autorecalc using insert sorts or recalc entire array
// new feature: When strings are pointers, shiftstrings will change the pointer values in case the caller has to realloc() the memory and the base pointer changes
char *strmstr(char *str,char *end,unsigned *mno,struct _MSTR *msearch/*,unsigned nsearch*/,unsigned *cs/*,int nocase*/) {
  unsigned i;
  char *d;
  struct _MSTR *im;

  if (str && str<end) for(d=str; (d=memcspn(d,end,msearch[0].string,msearch[0].len))<end; d++ ) {
      for(i=cs[*(unsigned char *)d],im=msearch+i; i<=cs[*(unsigned char *)d+256]; /*i++,im+=sizeof(*msearch)*/) {
          if (d>=end-im->len) {
            i++; im++; // no space left for string
#if 1 /* { Disables the Prefix Skip system if it doesn't work */
        } else if (im->lennext && toupper(im->string[im->lennext-1])!=toupper(d[im->lennext-1])) { // non match at largest position, skip all possible
          i += im->lenskip+1;
          im = msearch+i; // this is the easiest way to do the multiplication
#endif /* } */
        } else if (!(im->nocase?memicmp(d,im->string,im->len):memcmp(d,im->string,im->len))) { // iterate through the possible matches checking case if requested
          *mno=i; // someday when the iteration counts get too long, we will use bsearch() with the newly created minlength to find the iteration starting point
          return(d);
        } else {
          i++; im++;
        }
      }
  }
  *mno=0;
  return(end);
}

// bug: the editor can double certain line breaks during a paste, this code can't fix it.
// when > is used as data like ">", this code doesn't catch it, and I may not fix it because it is improper HTML
unsigned stripHTMLtags(char **dest,unsigned *destsz,unsigned *destlen,const char *snotabs) {
//#define ncase 1
  unsigned n=0,lold,lnew,mno;
  char *d,*dp,*newst,*lastspace,*lastnewline,tempst[2],space[]=" ",tab[]="\t",*end;
  unsigned intextarea;
  int notabs=(*snotabs=='n');
  unsigned quick[512];

  if (*dest) {
    strmstrinit(mstring,NELEM(mstring),quick);
    for(lastnewline=lastspace=d=*dest,end=d+*destlen,intextarea=0; (d=strmstr(d,end,&mno,mstring,quick))<end; ) {
      switch(mstring[mno].rv) {
      case 0: // & general
        dp=(char *)memchr(d,';',end-d);
        if (!dp || dp-d>8 || !isdigit(d[2]) || (int)(lnew=strtol(d+2,NULL,0))<0 || lnew>255) {
          d++;
          continue;
        } else {
          lold=dp-d+1;
          newst=tempst;
          newst[0]=lnew;
          newst[1]='\0';
          lnew=1;
        }
        break;
      case 1: // &quot;
        lold=mstring[mno].len;
        newst=mstring[mno].string+lold+1;
        lnew=strlen(newst);
        break;
      case 2: // <!-- stuff to delete -->
        lold=strlen(mstring[mno].string+mstring[mno].len+1);
        newst=mstring[mno].string+mstring[mno].len+1; // dummy variable
        dp=memistr(d,end,newst,strlen(newst));
        lnew=0;
        goto delcom;
      case 3: // <tag that generates newline
        newst="\r\n"; lnew=2;
        lastspace=d+lnew;
        lastnewline=d+lnew;
        goto bktcom;
      case 4: // <tag that generates tab
        newst=notabs?space:tab;
        lnew=1;
        goto bktcom;
      case 5: // <tag that indicates we are entering a preformatted section
        intextarea++;
        lnew=0;
        goto bktcom;
      case 6: // <tag that indicates we are leaving a preformatted section
        if (intextarea) intextarea--;
        lnew=0;
bktcom:
        lold=1;
        if (!(dp=(char *)memchr(d,'>',end-d))) continue;

delcom:
        if (dp==end) {
          d=end;
          continue;
        } else {
          lold+=dp-d;
        }
        break;
      case 7: // space without return
      case 8: // return
        lold=memspn(d,end,"\r\n\t ",5)-d; // 5 includes an occasional \0
        if (intextarea) {
          d+=lold;
          continue;
        }
        if (d<=lastspace || d<=lastnewline/* || mstring[mno].rv==8*/) {
          lnew=0;
        } else {
          newst=" ";
          lnew=1;
          lastspace=d+1;
        }
        if (mstring[mno].rv==8) lastnewline=d;
        break;
      }
      if (lnew != lold) {
        d += memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
        end=*dest+*destlen;
      }
      if (lnew) {
        memcpy(d,newst,lnew);
        d+=lnew;
      }
      n++;
    }
failbreak:
    strmstrclose(mstring);
  }
  return(n);
}

// returns the number of changes made
unsigned mymemset(char *s,char c,unsigned num){
  unsigned n=0;
  for(; num; num--, s++) {
    if (*s != c) {
      *s=c;
      n++;
    }
  }
  return(n);
}

// Search through a string for quote symbols
// If starting at a non quote, will return the postion of the next quote, possibly end
// If starting at a quote, will return the position after the close quote, possibly end
// If starting at end, returns NULL
// style=0, detect c-string escaping \" or \' or \\ or \anycharacter
// style=1, detect vb-string escaping "" or sql escaping ''
// style=2, Foxpro-string: no escaping
// some routines use 0 for no string detection and style-1 for the styles
char *findnextquote(char *str,char *end,unsigned style) {
  unsigned dx;
  char cq[3];

  if (str>=end) return(NULL);
  if (*str=='"' || *str=='\'') {
    strcpy(cq,style?"'":"'\\");
    cq[0]=str[0];
    str++;
    dx=1;
  } else {
    strcpy(cq,"'\"");
    dx=0;
  }
  while(1) {
    str=memcspn(str,end,cq,strlen(cq));
    if (str>=end) return(end);
    if (!style) { // C
      if (*str!='\\') return(str+dx);
      str+=2;
    } else { // VB & Foxpro
      if (style==1 && str[0]==str[1]) str+=2;
      else return(str+dx);
    }
  }
}

// This splits all lines after splitch
// style is defined by findnextquote()
// http://www.ultraedit.com/index.php?name=Forums&file=viewtopic&t=1415
unsigned splitlinesatch(char **dest,unsigned *destsz,unsigned *destlen,char splitch,unsigned style,unsigned eoltype) {
#define lold 0
  unsigned n=0,lnew;
  char *d,*end;
  char splitst[4]="?'\"";

  if (*dest) {
    if (eoltype>=NELEM(eoltypes)) eoltype=NELEM(eoltypes)-1;
    lnew=strlen(eoltypes[eoltype]);
    splitst[0]=splitch;
    for(d=*dest,end=d+*destlen; (d=memcspn(d,end,splitst,sizeof(splitst)-1))<end; ) {
    	if (*d=='"' || *d=='\'') {
        d=findnextquote(d,end,style);
    	} else if (*d==splitch) {
    		d++;
        if (lnew != lold) {
          d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
          end = *dest+*destlen;
        }
        if (lnew) {
          memcpy(d,&eoltypes[eoltype],lnew);
          d+=lnew;
        }
        n++;
      }
    }
  }
failbreak:
  return(n);
#undef lold
}

// This removes all non quoted text from str. The quoted text segments that remain are separated by newst
// style is defined by findnextquote()
unsigned findqtstrings(char **dest,unsigned *destsz,unsigned *destlen,const char *insertst,unsigned style) {
  unsigned n=0,lold,lnew;
  char *d,*dp,*end;
  const char *newst;

  if (*dest) {
    newst=strlen(insertst)>20?",":insertst;
    for(dp=*dest,end=dp+*destlen; d=findnextquote(dp,end,style); dp=d) {
      switch(*dp) {
      case '"' :
      case '\'':
        lold=0;
        lnew=strlen(newst);
        break;
      default:
        lold=d-dp;
        lnew=0;
        d=dp;
        break;
      }
      if (lnew != lold) {
        d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
        end = *dest+*destlen;
      }
      if (lnew) {
        memcpy(d,newst,lnew);
        d+=lnew;
      }
      n++;
    }
  }
failbreak:
  return(n);
}

// usetabs=1: convert leading spaces to tabs
// usetabs=0: convert leading tabs to spaces
// preserveodd=TRUE: non multiples of tabwidth spacing will be preserved, otherwise truncated (amending might overflow the buffer if we amend a lot of spaces)
unsigned space2tabs(char **dest,unsigned *destsz,unsigned *destlen,int usetabs,unsigned tabwidth,int preserveodd) {
  unsigned n=0,lnew,lold,indent;
  char *d,*dp,*end;

  if (*dest) {
    for(d=*dest,end=d+*destlen; d<end; ) {
      for(indent=0,dp=d; 1; dp++) {
        if (*dp==' ') indent++;
        else if (*dp=='\t') indent+=tabwidth;
        else break;
      }
      lold=(dp-d);
      if (usetabs) {
        lnew=indent/tabwidth;
        if (preserveodd && lnew*tabwidth!=indent) lnew=lold=0;
      } else lnew=indent;
          if (lnew != lold) {
            d += memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
            end = *dest+*destlen;
            n++;
          }
          if (lnew) {
            if (mymemset(d,usetabs?'\t':' ',lnew) && lnew==lold) n++;
            d+=lnew;
          }
      d=memcspn(d,end,"\r\n",2);
      while (d<end && *d=='\r' || *d=='\n') d++; //d+=strspn(d,"\r\n");
    }
  }
failbreak:
  return(n);
}

struct _MSTR cstring[]={
{""                  ,0,0 ,0,0,0}, // mstring[0] is where the search string is stored from call to call
{"//"                ,0,1 ,0,0,0}, // set case insensitive strings to 0 for better efficiency
{"/*\0*/"            ,0,2 ,0,0,0},
{"\""                ,0,3 ,0,0,0},
{"'"                 ,0,3 ,0,0,0},
{"if ("              ,0,4 ,0,0,0},
{"if("               ,0,4 ,0,0,0},
{"\r"                ,0,5 ,0,0,0},
{"\n"                ,0,5 ,0,0,0},
{"("                 ,0,6 ,0,0,0},
{")"                 ,0,7 ,0,0,0},
{"{"                 ,0,8 ,0,0,0},
{"}"                 ,0,9 ,0,0,0},
{" "                 ,0,10 ,0,0,0},
{"\t"                ,0,10 ,0,0,0},
};

/* The MinGW C libraries are more space efficient than any other compiler. The MinGW C++ libraries are
   more wasteful than any other. C++ is unnecessary, inappropriate, and wasteful for projects that can easily
   be done in C. Including any C++ project would ruin the efficiency and reliability of this project.
   AStyle was poor C code in an even worse C++ wrapper, nothing I would want to reuse.
   AStyle also doesn't indent the way I like it and was not planned with an editor in mind.
   This does not handle certain if-else layouts because it is too much work to detect a single statement:
  if (cond)
     stmt;
  else
     stmt;*/
// http://sourceforge.net/project/showfiles.php?group_id=126649 http://sourceforge.net/projects/gcgreatcode/
unsigned reindentcode(char **dest,unsigned *destsz,unsigned *destlen,int usetabs,unsigned tabwidth) {
  unsigned n=0,bktct,lold,lnew,parenct,tempindent,mno;
  char *d,*dp,*label,*end;
  unsigned quick[512];

  n+=space2tabs(dest,destsz,destlen,1,tabwidth,0);
  if (*dest) {
    strmstrinit(cstring,NELEM(cstring),quick);
    for(tempindent=parenct=0,bktct=strspn(dp=d=*dest,"\t"),end=d+*destlen; (dp=strmstr(dp,end,&mno,cstring,quick))<end; ) switch(cstring[mno].rv) {
    case 1: // C++ comment
      dp=memcspn(dp,end,"\r\n",2); // this does not handle \ comment continuations which shouldn't be used anways
      break;
    case 2: /* C comment */
      dp=memstr(dp+2,end,cstring[mno].string+cstring[mno].len+1,strlen(cstring[mno].string+cstring[mno].len+1));
      if (dp<end) dp+=2;
      //else dp= *dest + *destlen;
      break;
    case 3: // quoted string
      dp = findnextquote(dp,end,0);
      break;
    case 4: // if (
      dp += cstring[mno].len;
      parenct=1;
      break;
    case 5: // end of line
      if (*(dp-1)=='\\' && (*d=='#' || tempindent)) tempindent=2;
      dp+=(*dp=='\r' && dp[1]=='\n')?2:1;
      for(d=dp; *dp=='\t'; dp++); // d=bol, dp=curpos
      lold=dp-d;
      lnew=bktct+parenct+(parenct?1:0)+(tempindent>1?1:0); // indent conditionals separated by lines
      if (*dp!=':') for (label=dp; *label; label++) {
        if (*label==':') {
          if (label[1]==':') label=NULL; // C++ class::function
          break;
        }
        if (!isalnum(*label)) {
          label=NULL;
          break;
        }// label will be non NULL if we hit the end of file with nothing but alpha: but this will be so rare that it's not worth catching
      } else label=NULL;
      if (/*{*/*dp=='}' || !memcmp(dp,"case ",5) || !memcmp(dp,"default:",8) || !memcmp(dp,"public:",7) || !memcmp(dp,"private:",8) || !memcmp(dp,"protected:",10)) {
        if (lnew) lnew--;
      } else if (!*dp || *d=='#' || *d=='\r' || *d=='\n' || label) lnew=0;
      if (lnew != lold) {
        d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
        end=*dest+*destlen;
        n++;
      }
      if (lnew) {
        if (mymemset(d,'\t',lnew) && lnew==lold) n++;
        d+=lnew;
      }
      // new feature: if *d=='#' skip to eol
      dp=d;
      if (tempindent) tempindent--;
      break;
    case 6: // (
      if (parenct) parenct++;
      goto dodef;
    case 7: // )
      if (parenct) parenct--;
      goto dodef;
    case 8: // {
      bktct++;
      goto dodef;
    case 9: // {
      bktct--; // fall through to default
    default:
dodef:  dp++;
      break;
    }
failbreak:
    strmstrclose(mstring);
  }
  if (!usetabs) n+=space2tabs(dest,destsz,destlen,usetabs,tabwidth,0);
  return(n);
}

// This removes all non quoted text from str. The quoted text segments that remain are separated by newst
// style is defined by findnextquote()
unsigned killwhitenonqt(char *dest,unsigned destsz,unsigned *destlen,unsigned style) {
#define lnew 0
  unsigned n=0,lold,mno,quick[512];
  char *d,*end;

  if (*dest) {
    strmstrinit(cstring,NELEM(cstring),quick);
    d=dest+strspn(dest,"\r\n\t ");
    end=dest+*destlen;
    while((d=strmstr(d,end,&mno,cstring,quick))<end) switch(cstring[mno].rv) {
    case 1: // C++ comment
      d=memcspn(d,end,"\r\n",2); // this does not handle \ comment continuations which shouldn't be used anways
      break;
    case 2: /* C comment */
      d=memstr(d+2,end,cstring[mno].string+cstring[mno].len+1,strlen(cstring[mno].string+cstring[mno].len+1));
      if (d<end) d+=2;
      //else d= dest + *destlen;
      //d+=2;
      break;
    case 3: // quoted string
      d = findnextquote(d,end,style);
      break;
    case 5: // end of line
      d=memspn(d,end,"\r\n\t ",4);
      break;
    case 10: // space or tab
      lold=memspn(d,end,"\t ",2)-d;
      if (lnew != lold) {
        memmovetest(d+lnew,d+lold,*destlen-(d-dest)-lold+1);
        *destlen += lnew-lold;
        end += lnew-lold;
        n++;
      }
      break;
    default:
      d++;
      break;
    }
    strmstrclose(mstring);
  }
  return(n);
#undef lnew
}

// lines up text by adding or subtracting spaces to make a selected character
// line up. , and = are common characters. Can be used to line up
// comma separated tables or / marks to make comments even
// style=0, do not detect strings, otherwise style-1 is as findnextquote()
// new feature: bug: one of the NPP Menu structs does not line up properly
unsigned lineup(char **dest,unsigned *destsz,unsigned *destlen,char lch,int usetabs,unsigned tabwidth,unsigned style) {
  unsigned n=0;
  char *d,*dp,*end;
  unsigned tabno,pass,tabnol;
  unsigned mxpos,lold,lnew,flags;
#define FLAG_INDENT 1
#define FLAG_QUIT 2

  if (*dest) {
    if (lch && lch != ' ' && lch != '\t') {
    	end=*dest+*destlen;
      for(tabno=0; tabno<999; tabno++) for(pass=0,mxpos=(unsigned)-1; pass<=1; pass++) {
          for(dp=*dest; dp<end; ) {
            for(flags=FLAG_INDENT,tabnol=0,d=dp; !(flags&FLAG_QUIT) && d<end && *d!='\r' && *d!='\n'; d++) {
            lold=lnew=0;
            if (style && (*d=='"' || *d=='\'')) {
              d=findnextquote(d,end,style-1)-1;
            } else if (*d == ' ') { // nothing
            } else if (*d == '\t') {
              lold=1;
              lnew=(flags&FLAG_INDENT)?tabwidth:1; // Scintilla only permits tabs at the beginning of the line without any interspersed spaces
            } else {
              flags &= (~FLAG_INDENT);
              if (*d == lch) {
                tabnol++;
                if (tabnol>tabno) {
                  flags |= FLAG_QUIT;
                  if (!pass) { // calculate the maximum tab
                    do {
                      d--;
                    } while(isspace(*d) && d>=dp);
                    d++;
                    if (mxpos<(unsigned)(d-dp) || mxpos==(unsigned)-1) mxpos=d-dp;
                  } else {   //apply the minimum tab, shrinking extra spaces if necessary
                    if ((unsigned)(d-dp)>mxpos) {
                      lold=(d-dp)-mxpos;
                      lnew=0;
                      d=dp+mxpos;
                    } else {
                      lold=0;
                      lnew=mxpos-(d-dp);
                    }
                  }
                }
              }
            }
            if (lnew != lold) {
              d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
              end = *dest + *destlen;
              n++;
            }
            if (lnew) {
              memset(d,' ',lnew);
              if (lnew==lold) n++;
            }
          }
          d=memcspn(d,end,"\r\n",2); //while(*d && *d!='\r' && *d!='\n') d++;
          while(d<end && (*d =='\r' || *d =='\n')) d++;
          dp=d;
        }
        if (!pass) {
          if (mxpos==(unsigned)-1) break;
          //printf("Tab=%u Max=%u\n%s",tabno,mxpos,str);
        }
      }
    }
failbreak:
    n+=space2tabs(dest,destsz,destlen,usetabs,tabwidth,0);
  }
  return(n);
#undef FLAG_INDENT
#undef FLAG_STOP
}

// new feature: allow overwrite to insert part of newst if there is some space
unsigned filldown(char **dest,unsigned *destsz,unsigned *destlen,int insert) {
  unsigned n=0,chn,lold,lnew/*,detent*/;
  char *d,*end;
#define newst *dest

  if (*dest && *destlen && (d=memcspn(*dest,end=*dest+*destlen,"\r\n",2))>*dest) {
    lnew=d-*dest;
    goto bottest; do {
      chn=memcspn(d,end,"\r\n",2)-d;
      if (lnew<=chn || insert) {
        lold=insert?0:lnew;
        if (lnew != lold) {
          d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
          end=*dest+*destlen;
          chn += lnew-lold;
        }
        memcpy(d,newst,lnew);
        n++;
      }
      d += chn;
bottest:
      while(d<end && (*d=='\r' || *d=='\n')) d++; // more efficient than strspn();
    } while(d<end);
  }
failbreak:
  return(n);
#undef newst
}

// adds up all the numbers separated by any non number characters
unsigned addup(char *str) {
  double sum=0;

  if (str) do {
    switch(*str) {
    case '\0':
      MessageBoxFree(0,smprintf("Sum=%#lg",sum),PLUGIN_NAME,MB_OK|MB_ICONINFORMATION);
      break;
    case '+':
    case '-':
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
      sum+=strtod(str,&str);
    default:
      str++;
      continue;
    }
    break;
  } while(1);
  return(0);
}

unsigned zapchar(char *str,unsigned destlen,char ch,int nonprint) {
  char *end;
  int n=0;
  if (str) for(end=str+destlen;str<end;str++) if (*str != '\r' && *str!='\n' && *str!='\t' && *str!=ch && (!nonprint || !isprint(*str))) {*str=ch; n++;}
  return(n);
}

unsigned submitW3C(char **dest,unsigned *destsz,unsigned *destlen,const char *fn,const char *fno) {
  if (*dest) {
    char *path;
    do {
      char buf1[512];
      FILE *fp;
      if (!(path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,fn)) || !(fp=fopen(path,"rt"))) break;
      freesafe(path,"submitW3C");
      memset(buf1,0,sizeof(buf1));
      fread(buf1,1,sizeof(buf1)-1,fp);
      fclose(fp);
      if (!(path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,fno))) break;
      DeleteFile(path);
      //MessageBox(nppData._nppHandle,path,"Before",MB_OK|MB_ICONINFORMATION);
      if (!(fp=fopen(path,"wt"))) break;
      //MessageBox(nppData._nppHandle,path,"After",MB_OK|MB_ICONINFORMATION);
      memstrtran(dest,destsz,destlen,NULL,"\r\n",2,"\n",1);
      strchrstrans(dest,destsz,destlen,NULL,"&<>\"",4,",&&amp;,<&lt;,>&gt;,\"&quot;,");
      if (*dest) {
        fprintf(fp,buf1,*dest);
        fclose(fp);
        ShellExecute(nppData._nppHandle,"open",path,NULL,".",0);
      } else MessageBoxFree(nppData._nppHandle,smprintf("Unable to execute:\r\n%s",path),PLUGIN_NAME,MB_OK|MB_ICONINFORMATION);
    } while(0);
    if (path) freesafe(path,"submitW3C");
  }
  return(0);
}

int fcmpnocase=0;
unsigned fcmpcolumn=0;
int fcmpascending=1;
BOOL SortLinesUnique=FALSE;

// compares up to the end of a line or end of string starting at column
int QSORT_FCMP fcmpline(const void *ax,const void *bx) {
  char *a=*((char **)ax);char *b=*((char **)bx);
  int p,q,dif=0;
  unsigned tempcolumn;

  for(tempcolumn=fcmpcolumn; tempcolumn; tempcolumn--,a++,b++) {
    if (!*a || !*b || *a=='\r' || *a=='\n' || *b=='\r' || *b=='\n') return(0); //if either line is not long enough, pretend equal
  }
  while(*a && *b) {
    dif=fcmpnocase?(toupper(*a)-toupper(*b)):(*a-*b);
    q=(*b=='\r' || *b=='\n'); // here's a case where short-circuit logic makes us separate this out
    if ((p=(*a=='\r' || *a=='\n')) || q) {
      if (p&&q) dif=0;
      break;
    }
    if (dif) break;
    a++;
    b++;
  }
  return(fcmpascending?dif:-dif);
}

// copies only a single line, returns \0 end of dest
char *strcpyline(char *dest,char *src) {
  do {
    *dest=*src++;
    if (!*dest) break;
    //if (*dest=='\n' || (*dest=='\r' && *src!='\n') ) {dest++; *dest='\0'; break;}
    if (*dest=='\n' || *dest=='\r') {*dest='\0'; break;}
    dest++;
  } while(1);
  return(dest);
}

// new feature: numeric sort
// new feature: ignore leading spaces,tabs
unsigned strqsortlines(char **dest,unsigned *destsz,unsigned *destlen,int nocase,int ascending,unsigned column) {
  unsigned n=0/*,chn*/,lineno,lineno2,linessz;
  char *d,*dp,*nstr,**lines=NULL;
  if (*dest) {
    fcmpcolumn=column;
    fcmpascending=ascending;
    for(d=*dest+strspn(*dest,"\r\n"), lineno=0,linessz=0; *d ; lineno++) {
      armreallocsafe((char **)&lines,&linessz,(lineno+1)*sizeof(*lines),ARMSTRATEGY_INCREASE,0,"strqsortlines"); if (!lines) goto failbreak;
      lines[lineno]=d;
      d+=strcspn(d,"\r\n");
      while(*d && (*d=='\r' || *d=='\n')) d++;
    }
    if (!(nstr=(char *)mallocsafe(*destlen+1,"strqsortlines"))) goto failbreak;
    //memset(nstr,0,*destlen+1);
    fcmpnocase=nocase;
    qsortintro(lines,lineno,sizeof(lines[0]),fcmpline);
    for(dp=*dest,d=nstr,lineno2=0; lineno2<lineno; lineno2++) if (lineno2==0 || !SortLinesUnique || fcmpline(lines+lineno2,lines+lineno2-1)) {
    	if (!SortLinesUnique) {
        while(*dp=='\r' || *dp=='\n') {*d=*dp; d++; dp++;} // import the line endings from the unsorted string
    	} else {
    		if (*dp=='\r' || *dp=='\n') {
    			*d=*dp; d++; dp++;
      		if (*(dp-1)=='\r' && *dp=='\n') {*d=*dp; d++; dp++; }
    		}
    	}
      dp+=strcspn(dp,"\r\n");
    	d=strcpyline(d,lines[lineno2]);
    }
    if (!SortLinesUnique) {strcpy(d,dp); d += strlen(dp);} // grab extra lines at the end
#if NPPDEBUG /* { */
    if (!SortLinesUnique && *destlen!=d-nstr)
      MessageBox(nppData._nppHandle,"The size of the sort buffer should not have changed",PLUGIN_NAME, MB_OK|MB_ICONWARNING);
#endif /* } */
    freesafe(*dest,"strqsortlines");
    *dest=nstr;
    *destsz=*destlen+1;
    *destlen=d-nstr;
    n=1;
  }
failbreak:
  if (lines) freesafe(lines,"strqsortlines");
  return(n);
}

unsigned indentlines(char **dest,unsigned *destsz,unsigned *destlen,int usetabs,unsigned tabwidth) {
#if NPPDEBUG
    //DWORD tkc=GetTickCount();
#endif
#define lold 0
  unsigned n=0,lnew;
  int movezero;
  char *d,*end;
  if (*dest) {
    char *newdest=NULL,*newlast=*dest; unsigned newsz=*destsz,newlen=0; // only used for HighPerformance
    for(d=memspn(*dest,end=*dest+*destlen,"\r\n",2),movezero=!isspace(*d) ; d<end; ) {
      if (movezero || isspace(*d)) {
        lnew=usetabs?1:tabwidth;
        if (HighPerformance) {
          if (newlast<d) {memcpyarmsafe(&newdest,&newsz,&newlen,newlast,d-newlast,"indentlines"); if (!newdest) goto freebreak;}
          newlast=(d+=lold);
          armreallocsafe(&newdest,&newsz,newlen+lnew+1 /* 1 reserved for \0 below */,ARMSTRATEGY_INCREASE,0,"indentlines"); if (!newdest) goto freebreak;
          memset(newdest+newlen,usetabs?'\t':' ',lnew);
          newlen+=lnew;
          //end=*dest+*destlen;
        } else {
          if (lnew != lold) {
            d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
            end=*dest+*destlen;
          }
          memset(d,usetabs?'\t':' ',lnew);
        }
        n++;
      }
      d=memcspn(d,end,"\r\n",2);
      while(d<end && (*d=='\r' || *d=='\n')) d++; //d+=strspn(d,"\r\n");
    }
    if (HighPerformance) {
    	if (n) {
        if (newlast<d) memcpyarmsafe(&newdest,&newsz,&newlen,newlast,d-newlast,"indentlines");
        else newdest[newlen]='\0'; // space reserved up above
freebreak:
        freesafe(*dest,"indentlines");
        if (!(*dest=newdest)) n=0;
        *destsz=newsz;
        *destlen=newlen;
      } else if (newdest) freesafe(newdest,"indentlines");
    }
  }
failbreak:
#if NPPDEBUG
  //MessageBoxFree(nppData._nppHandle,smprintf("Ticks used %u",GetTickCount()-tkc),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
#endif
  return(n);
#undef lold
}

unsigned indentlinessurround(char **dest,unsigned *destsz,unsigned *destlen,int usetabs,unsigned tabwidth,unsigned eoltype) {
#define lold 0
  char *e,*ind=NULL; unsigned indsz=0,indln;
  unsigned n=0,/*lnew,*/eollen;
  if (*dest) {
    if (eoltype>=NELEM(eoltypes)) eoltype=NELEM(eoltypes)-1;
    eollen=strlen(eoltypes[eoltype]);
    e=memspn(*dest,*dest+*destlen,"\r\n",2);
    memcpyarmsafe(&ind,&indsz,&indln,*dest,memspn(e,*dest+*destlen," \t",2)-e,"indentlinessurround"); if (!ind) goto failbreak;
    n+=indentlines(dest,destsz,destlen,usetabs,tabwidth); if (!*dest) goto failbreak;
    memmovearmtest(dest,destsz,destlen,*dest+indln+1+eollen,*dest+lold,1); if (!*dest) goto failbreak;
    memcpy(*dest,ind,indln);
    *(*dest+indln)='{';
    memcpy(*dest+indln+1,&eoltypes[eoltype],eollen);
    n++;
    memcpyarmsafe(dest,destsz,destlen,*dest,indln+eollen,"indentlinessurround"); if (!*dest) goto failbreak;
    //memcpy(*dest+*destlen-eollen-1-indln,ind,indln);
    *(*dest+*destlen-eollen-1)='}';
    memcpy(*dest+*destlen-eollen,&eoltypes[eoltype],eollen);
    //(*destlen)-=2;
    //memcpyarmsafe(dest,destsz,destlen,"",1,"indentlinessurround"); if (!*dest) goto failbreak;
    //(*destlen)--;
    freesafe(ind,"indentlinessurround");
  }
failbreak:
  return(n);
#undef lold
}

// http://forum.pspad.com/read.php?f=2&i=1404&t=1404
// new feature: hex editor in words
// new feature: printable hex only (assuming that Notepad++ would try to print characters<32)
unsigned tohex(char **dest,unsigned *destsz,unsigned *destlen,unsigned offset,unsigned width) {
  unsigned n=0,bufl;
  char *d,*end,buf[128]; //,hex[3];
  unsigned counter8;
  if (*dest && width >= 16 && width<=sizeof(buf)) {
    char *newdest=NULL; unsigned newsz=*destsz*5,newlen=0;
    d=*dest;
    end=*dest+*destlen;
    bufl=0;
    counter8=7;
    while(d<end) {
      if (!bufl) sarmprintf(&newdest,&newsz,&newlen,"\"%9.9X  ",offset+n);
      sarmprintf(&newdest,&newsz,&newlen,(counter8 || bufl==width-1)?"%2.2X ":"%2.2X-",*(unsigned char *)d);
      if (counter8) counter8--; else counter8=7;
      buf[bufl++]=(*d=='\r' || *d=='\n' || *d=='\t' || *d=='"')?'.':*d;
      d++;
      n++;
      if (bufl == width || d>=end) {
        memsetarmsafe(&newdest,&newsz,&newlen,' ',(width-bufl)*3+2,"tohex");
        memsetarmsafe(&newdest,&newsz,&newlen,'|',1,"tohex");
        memcpyarmsafe(&newdest,&newsz,&newlen,buf,bufl,"tohex");
        if (bufl<width) memsetarmsafe(&newdest,&newsz,&newlen,' ',width-bufl,"tohex");
        bufl=0;
        strcpyarmsafe(&newdest,&newsz,&newlen,"|\"\r\n","tohex");
      }
    }
    if (n) {
      memsetarmsafe(&newdest,&newsz,&newlen,'\0',1,"tohex"); newlen--;
freebreak:
      freesafe(*dest,"tohex");
      if (!(*dest=newdest)) n=0;
      *destsz=newsz;
      *destlen=newlen;
    } else if (newdest) freesafe(newdest,"tohex");
  }
failbreak:
  return(n);
}

unsigned fromhex(char **dest,unsigned *destsz,unsigned *destlen,unsigned offset) {
  unsigned n=0,val;
  int badoffset;
  char *d,*dp,*end;
  if (*dest) {
    char *newdest=NULL; unsigned newsz=*destsz/5,newlen=0;
    d=*dest;
    end=*dest+*destlen;
    badoffset=FALSE;
    while(d<end) {
      if (d[0]==' ' && d[1]==' ' && d[2]==' ' || *d=='|') d = memcspn(d,end,"\r\n",2);
      else if (*d>='0' && *d<='9' || *d>='A' && *d<='F' || *d>='a' && *d<='f') {
        val=strtol(d,&dp,16);
        if (dp-d==9) {
        	if (offset+n != val) badoffset=offset+n-val;
        } else if (dp-d==2) {
          memsetarmsafe(&newdest,&newsz,&newlen,val,1,"fromhex");
          n++;
        }
        d=dp;
      } else d++;
    }
    if (n) {
      memsetarmsafe(&newdest,&newsz,&newlen,'\0',1,"fromhex"); newlen--;
freebreak:
      freesafe(*dest,"fromhex");
      if (!(*dest=newdest)) n=0;
      *destsz=newsz;
      *destlen=newlen;
      if (badoffset) MessageBoxFree(nppData._nppHandle,smprintf("Offset has changed by %d",badoffset),PLUGIN_NAME, MB_OK|MB_ICONWARNING);

    } else if (newdest) freesafe(newdest,"fromhex");
  }
failbreak:
  return(n);
}

unsigned hexbyterunstolittlendian(char **dest,unsigned *destsz,unsigned *destlen,unsigned bytewidth) {
  unsigned n=0,lold,lnew;
  char *d,*dp,*end,buf[9]="??..??..";
  unsigned bufl,dp2;
  if (*dest && bytewidth>=2 && bytewidth<=4 && powcheck(bytewidth)) {
    for(d=*dest,end=*dest+*destlen ; d<end; d++) if (*d==' ' && d[1]==' ' && d[2]==' ' || *d=='|') {
      d=memcspn(d,end,"\r\n",2);
    } else {
      for(dp2=2,bufl=2*bytewidth,dp=d; dp<end && dp-d<3*bytewidth && bufl; dp++,dp2=dp2?(dp2-1):2) if (!dp2) {
        if (!(*dp==' ' || *dp=='-')) break;
      } else {
        if (!(*dp>='0' && *dp<='9' || *dp>='A' && *dp<='F' || *dp>='a' && *dp<='f')) break;
        buf[bufl-2*(1-(bufl&1))]=*dp;
        bufl--;
      }
      if ((dp==end || *dp==' ' || *dp=='-') && (lold=dp-d)==3*bytewidth-1) {
      	lnew=2*bytewidth;
        if (lnew != lold) {
          d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
          end=*dest+*destlen;
        }
        memcpy(d,buf,lnew);
        d += lnew;
        n++;
      }
    }
  }
failbreak:
  return(n);
}

unsigned littlendiantohexbyteruns(char **dest,unsigned *destsz,unsigned *destlen) {
#define lnew buflen
  unsigned n=0;
  char *d,*dp,*dpe,*end;
  char *buf=NULL; unsigned bufsz=0,buflen,lold;
  if (*dest && *destlen) {
    for(dp=d=*dest,end=*dest+*destlen ; 1; d++) {
        if (d>=end || !(*d>='0' && *d<='9' || *d>='A' && *d<='F' || *d>='a' && *d<='f')) {
          if ( !((d-dp)&1)) { // even number of hex characters
            for(buflen=0,dpe=d-2; dpe>=dp; dpe-=2) {
              memcpyarmsafe(&buf,&bufsz,&buflen,dpe,2,"littlendiantohexbyteruns"); if (!buf) goto failbreak;
              if (dpe!=dp) memsetarmsafe(&buf,&bufsz,&buflen,' ',1,"littlendiantohexbyteruns"); if (!buf) goto failbreak;
            }
            lold=d-dp;
            d=dp;
            if (lnew != lold) {
              d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
              end=*dest+*destlen;
            }
            memcpy(d,buf,lnew);
            d += lnew;
            n++;
          }
          dp=d+1;
        }
      if (d<end && (*d==' ' && d[1]==' ' && d[2]==' ' || *d=='|')) {
        dp=d=memcspn(d,end,"\r\n",2);
      }
      if (dp>=end) break;
    }
  }
failbreak:
  if (buf) freesafe(buf,"littlendiantohexbyteruns");
  return(n);
#undef lnew
}

// http://forum.pspad.com/read.php?f=2&i=4073&t=4073
unsigned uudecode(char **dest,unsigned *destsz,unsigned *destlen) {
  unsigned n=0; //,val;
  unsigned char *d,*end;
  unsigned char collector[3],*cp;
  unsigned linelen;
  if (*dest) {
    char *newdest=NULL; unsigned newsz=*destsz/5,newlen=0;
    d=(unsigned char *)(*dest);
    end=(unsigned char *)(*dest)+*destlen;
    while(d<end) {
      if (*d>=33 && *d<='M') {
        for(linelen=*d-32,d++; linelen;  ) {
          cp=collector;
          *cp=((*d-32)&0x3F)<<2; // 6 bits
          d++;
          *cp |= ((*d-32)>>4)&0x3; // +2 bits leaving 4 bits
          cp++;
          *cp=((*d-32)&0xF)<<4; // 4 bits
          d++;
          *cp |= ((*d-32)>>2)&0xF; // +4 bits leaving 2
          cp++;
          *cp=((*d-32)&0x3)<<6; // 2 bits
          d++;
          *cp |= ((*d-32)&0x3F); // +6 bits
          d++;
          memcpyarmsafe(&newdest,&newsz,&newlen,(char *)collector,(linelen>3?3:linelen),"uudecode");
          n++;
          linelen=linelen>3?linelen-3:0;
        }
      }
      d=(unsigned char *)memcspn((char *)d,(char *)end,"\r\n",2);
      while(d<end && (*d=='\r' || *d=='\n')) d++;
    }
    if (n) {
freebreak:
      freesafe(*dest,"uudecode");
      if (!(*dest=newdest)) n=0;
      *destsz=newsz;
      *destlen=newlen;
    } else if (newdest) freesafe(newdest,"uudecode");
  }
failbreak:
  return(n);
}

unsigned char base64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

// http://forum.pspad.com/read.php?f=2&i=4073&t=4073
unsigned base64decode(char **dest,unsigned *destsz,unsigned *destlen) {
  unsigned n=0; //,val;
  unsigned char *d,*end;
  unsigned char collector[64],*cp,*t,bits6;
  unsigned seq;
  if (*dest) {
    char *newdest=NULL; unsigned newsz=*destsz/5,newlen=0;
    d=(unsigned char *)(*dest);
    end=(unsigned char *)(*dest)+*destlen;
    for(seq=3,cp=collector; d<end; d++) if (t=(unsigned char *)memchr(base64,*d,sizeof(base64)-1)) {
          bits6=t-base64;
          switch(seq) {
          case 3:
            *cp=bits6<<2; // set 6 bits, 2 not set
            seq=2;
            break;
          case 2:
            *cp|=(bits6>>4)&0x3;  // set 2 remaining bits
            cp++;
            *cp=(bits6&0xF)<<4; // set 4 bits, 4 not set
            seq=1;
            break;
          case 1:
            *cp|=(bits6>>2)&0xF;  // set 4 remaining bits
            cp++;
            *cp=(bits6&0x3)<<6; // set 2 bits, 6 not set
            seq=0;
            break;
          case 0:
            *cp|=bits6&0x3F; // set 6 remaining bits
            cp++;
            seq=3;
            break;
          }
          n++;
          if (cp-collector>=sizeof(collector)-1) {
            memcpyarmsafe(&newdest,&newsz,&newlen,(char *)collector,cp-collector,"base64decode");
            cp=collector;
          }
    }
    if (n) {
      if (cp>collector) memcpyarmsafe(&newdest,&newsz,&newlen,(char *)collector,cp-collector,"base64decode");
      //memsetarmsafe(&newdest,&newsz,&newlen,'\0',1,"base64decode");
freebreak:
      freesafe(*dest,"base64decode");
      if (!(*dest=newdest)) n=0;
      *destsz=newsz;
      *destlen=newlen;
    } else if (newdest) freesafe(newdest,"base64decode");
  }
failbreak:
  return(n);
}

unsigned converteol(char **dest,unsigned *destsz,unsigned *destlen,unsigned eoltype) {
  unsigned n=0,lold,lnew;
  char *d,*end;
  if (*dest) {
    if (eoltype>=NELEM(eoltypes)) eoltype=NELEM(eoltypes)-1;
    lnew=strlen(eoltypes[eoltype]);
    d=*dest; end=*dest+*destlen;
    goto bottest; do {
    	lold=(*d=='\r' && d[1]=='\n')?2:1;
      if (lnew != lold) {
        d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
        end=*dest+*destlen;
      }
      memcpy(d,&eoltypes[eoltype],lnew);
      d+=lnew;
      n++;
bottest:
    	d=memcspn(d,end,"\r\n",2);
    } while(d<end);
  }
failbreak:
  return(n);
}

unsigned insertlinenumbers(char **dest,unsigned *destsz,unsigned *destlen,unsigned lno) {
#define lold 0
#define lnew sizeof(buf)
  unsigned n=0;
  char *d,*end,buf[9];
  if (*dest) {
    for(d=*dest,end=*dest+*destlen ; d<end; lno++) {
      snprintfX(buf,sizeof(buf),"%*.*u",sizeof(buf)-1,sizeof(buf)-1,lno);
      buf[lnew-1]=' ';
      if (lnew != lold) {
        d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
        end=*dest+*destlen;
      }
      memcpy(d,buf,lnew);
      n++;
      d=memcspn(d,end,"\r\n",2);
      if (*d=='\r' && d[1]=='\n') d+=2;
      else if (*d=='\r' || *d=='\n') d++;
    }
  }
failbreak:
  return(n);
#undef lold
#undef lnew
}

// also used to delete line numbers
unsigned deletefirstword(char **dest,unsigned *destsz,unsigned *destlen) {
#define lnew 0
  unsigned n=0,lold;
  char *d,*end;
  if (*dest) {
    for(d=*dest,end=*dest+*destlen ; d<end; ) {
    	lold=memcspn(d,end,"\r\n\t ",4)-d;
    	if (d[lold]==' ' || d[lold]=='\t') {
    		lold++;
        if (lnew != lold) {
          d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
          end=*dest+*destlen;
          n++;
        }
        d=memcspn(d,end,"\r\n",2);
    	} else d+=lold;
      while(d<end && (*d=='\r' || *d=='\n')) d++;
    }
  }
failbreak:
  return(n);
#undef lnew
}

// also used to delete line numbers
unsigned cleanemailquoting(char **dest,unsigned *destsz,unsigned *destlen) {
#define lnew 0
  unsigned n=0,lold;
  char *d,*end;
  if (*dest) {
    for(d=*dest,end=*dest+*destlen ; d<end; ) {
    	lold=memspn(d,end,"> \t",3)-d;
        if (lnew != lold) {
          d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
          end=*dest+*destlen;
          n++;
        }
        d=memcspn(d,end,"\r\n",2);
      while(d<end && (*d=='\r' || *d=='\n')) d++;
    }
  }
failbreak:
  return(n);
#undef lnew
}

// http://www.textpad.info/forum/viewtopic.php?t=6645
unsigned extendblockspaces(char **dest,unsigned *destsz,unsigned *destlen) {
#define lold 0
  unsigned n=0,lnew,maxlength,pass;
  char *d,*end;
  if (*dest) {
    for(maxlength=0,pass=0; pass<2; pass++) for(d=*dest,end=*dest+*destlen ; d<end; ) {
    	lnew=memcspn(d,end,"\r\n",2)-d;
        d += lnew;
    	if (!pass) {
    		if (maxlength<lnew) maxlength=lnew;
    	} else {
    		if (lnew<maxlength) {
          lnew=maxlength-lnew;
          if (lold != lnew) {
            d+=memmovearmtest(dest,destsz,destlen,d+lnew,d+lold,1); if (!*dest) goto failbreak;
            end=*dest+*destlen;
          }
          memset(d,' ',lnew);
          d+=lnew;
          n++;
    		}
    	}
      if (*d=='\r' && d[1]=='\n') d+=2;
      else if (*d=='\r' || *d=='\n') d++;
    }
  }
failbreak:
  return(n);
#undef lold
}

unsigned trimtrailingspace(char *dest,unsigned destsz,unsigned *destlen) {
#define lnew 0
  unsigned n=0,lold;
  char *d,*dp,*end;
  if (dest) {
    for(d=dest,end=dest+*destlen;d<end; ) {
      dp=d;
      d=memcspn(d,end,"\r\n",2);
      for(d--,lold=0; d>=dp && *d==' '; d--, lold++);
      d++;
      if (lnew != lold) {
        memmovetest(d+lnew,d+lold,*destlen-(d-dest)-lold+1);
        *destlen += lnew-lold;
        end += lnew-lold;
        n++;
      }
      d=memspn(d,end,"\r\n",2);
    }
  }
  return(n);
#undef lnew
}

// http://www.crimsoneditor.com/english/board/CrazyWWWBoard.cgi?db=forum&mode=read&num=2339&page=161&ftype=6&fval=&backdepth=1
unsigned deleteblanklines(char *dest,unsigned destsz,unsigned *destlen,unsigned surplus) {
#define lnew 0
  unsigned n=0,lold;
  char *d,*end;
  unsigned surplus1;
  if (dest && surplus>=1) {
    for(d=dest,end=dest+*destlen;d<end; ) {
      d=memcspn(d,end,"\r\n",2);
      for(surplus1=surplus; d<end && surplus1; surplus1--) {
        if (*d=='\r' && d[1]=='\n') d+=2;
        else d++;
      }
      lold=memspn(d,end,"\r\n",2)-d;
      if (lnew != lold) {
        memmovetest(d+lnew,d+lold,*destlen-(d-dest)-lold+1);
        *destlen += lnew-lold;
        end += lnew-lold;
        n++;
      }
    }
  }
  return(n);
#undef lnew
}

#if 0
unsigned deleteeverynthline(char *dest,unsigned destsz,unsigned *destlen,unsigned nthline) {
#define lnew 0
  unsigned n=0,lold,nthct;
  char *d,*end;
  if (dest && nthline>1) { // deleting every line isn't the purpose here
    //  MessageBoxFree(nppData._nppHandle,smprintf("nthct:%u lold:%u",nthct,lold),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
    for(nthline--,nthct=0,d=dest,end=dest+*destlen;d<end; ) {
      lold=memcspn(d,end,"\r\n",2)-d;
      if (d<end-lold) lold+=(d[lold]=='\r' && d[lold+1]=='\n')?2:1;
      if (!nthct) {
        if (lnew != lold) {
          memmovetest(d+lnew,d+lold,*destlen-(d-dest)-lold+1);
          *destlen += lnew-lold;
          end += lnew-lold;
          n++;
        }
        nthct=nthline;
      } else {
        d+=lold;
        nthct--;
      }
    }
  }
  return(n);
#undef lnew
}
#endif

//http://www.room42.com/store/computer_center/c_translate.shtml
//http://support.microsoft.com/kb/q216399/
//http://www.usc.edu/isd/doc/statistics/help/multiuse/datasamples/
unsigned char ASCII_To_EBCDIC_Table []={ // EBCDIC->ASCII is second half of table
  0x00,0x01,0x02,0x03,0x37,0x2D,0x2E,0x2F,0x16,0x05,0x25,0x0B,0x0C,0x0D,0x0E,0x0F,
  0x10,0x11,0x12,0x13,0x3C,0x3D,0x32,0x26,0x18,0x19,0x3F,0x27,0x1C,0x1D,0x1E,0x1F,
  0x40,0x5A,0x7F,0x7B,0x5B,0x6C,0x50,0x7D,0x4D,0x5D,0x5C,0x4E,0x6B,0x60,0x4B,0x61,
  0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0x7A,0x5E,0x4C,0x7E,0x6E,0x6F,
  0x7C,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,
  0xD7,0xD8,0xD9,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xAD,0xE0,0xBD,0x5F,0x6D,
  0x79,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x91,0x92,0x93,0x94,0x95,0x96,
  0x97,0x98,0x99,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xC0,0x4F,0xD0,0xA1,0x07,
  0x20,0x21,0x22,0x23,0x24,0x15,0x06,0x17,0x28,0x29,0x2A,0x2B,0x2C,0x09,0x0A,0x1B,
  0x30,0x31,0x1A,0x33,0x34,0x35,0x36,0x08,0x38,0x39,0x3A,0x3B,0x04,0x14,0x3E,0xE1,
  0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x51,0x52,0x53,0x54,0x55,0x56,0x57,
  0x58,0x59,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x70,0x71,0x72,0x73,0x74,0x75,
  0x76,0x77,0x78,0x80,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F,0x90,0x9A,0x9B,0x9C,0x9D,0x9E,
  0x9F,0xA0,0xAA,0xAB,0xAC,0x4A,0xAE,0xAF,0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,
  0xB8,0xB9,0xBA,0xBB,0xBC,0x6A,0xBE,0xBF,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF,0xDA,0xdB,
  0xDC,0xDD,0xDE,0xDF,0xEA,0xEB,0xEC,0xED,0xEE,0xEF,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF,
  0x00,0x01,0x02,0x03,0x9C,0x09,0x86,0x7F,0x97,0x8D,0x8E,0x0B,0x0C,0x0D,0x0E,0x0F,
  0x10,0x11,0x12,0x13,0x9D,0x85,0x08,0x87,0x18,0x19,0x92,0x8F,0x1C,0x1D,0x1E,0x1F,
  0x80,0x81,0x82,0x83,0x84,0x0A,0x17,0x1B,0x88,0x89,0x8A,0x8B,0x8C,0x05,0x06,0x07,
  0x90,0x91,0x16,0x93,0x94,0x95,0x96,0x04,0x98,0x99,0x9A,0x9B,0x14,0x15,0x9E,0x1A,
  0x20,0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xD5,0x2E,0x3C,0x28,0x2B,0x7C,
  0x26,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,0xB0,0xB1,0x21,0x24,0x2A,0x29,0x3B,0x5E,
  0x2D,0x2F,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xE5,0x2C,0x25,0x5F,0x3E,0x3F,
  0xBA,0xBB,0xBC,0xBD,0xBE,0xBF,0xC0,0xC1,0xC2,0x60,0x3A,0x23,0x40,0x27,0x3D,0x22,
  0xC3,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,
  0xCA,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,0x70,0x71,0x72,0xCB,0xCC,0xCD,0xCE,0xCF,0xD0,
  0xD1,0x7E,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0xD2,0xD3,0xD4,0x5B,0xD6,0xD7,
  0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF,0xE0,0xE1,0xE2,0xE3,0xE4,0x5D,0xE6,0xE7,
  0x7B,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,
  0x7D,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,0x51,0x52,0xEE,0xEF,0xF0,0xF1,0xF2,0xF3,
  0x5C,0x9F,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,
  0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF};

unsigned asciiEBCDIC(char *str,unsigned destlen,int toEBCDIC) {
  char *end,temp[513],*path=NULL;
  HANDLE fp;
  int n=0;
  if (str) {
    memcpy(temp,ASCII_To_EBCDIC_Table,sizeof(temp)-1);
    if ((path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,"AsciiToEBCDIC.bin"))) {
      if (INVALID_HANDLE_VALUE!=(fp=_openX(path,O_RDONLY,O_DENYWRITE))) {
        if (_readX(fp,temp,sizeof(temp))!=sizeof(temp)-1) {
          MessageBoxFree(nppData._nppHandle,smprintf("%s must be exactly 512 bytes.\r\nThe first 256 bytes convert ASCII->EBCEDIC.\r\nThe remaining 256 bytes convert EBCDIC->ASCII\r\nUsing default table!",path),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
          memcpy(temp,ASCII_To_EBCDIC_Table,sizeof(temp)-1);
        }
        _closeX(fp);
      }
      freesafe(path,"asciiEBCDIC");
    }
    for(end=str+destlen;str<end;str++) {*str= temp[*(unsigned char *)str+(toEBCDIC?0:256)]; n++;}
  }
  return(n);
}

unsigned wordcount(char *str,unsigned destlen) {
  char *end;
  unsigned linelen;
  int wasspace;
  unsigned words=0,chrasutf8=0,chrnsp=0,chrsp=0,lines=0,linesnonblank=0,linelongest=0,lineshortest=(unsigned)-1;
  if (str) {
    for(linelen=0,wasspace=TRUE,end=str+destlen;str<end;str++) {
    	switch(*str) {
      case '\r':
        if (str[1]=='\n') str++;
      case '\n':
        if (linelen) linesnonblank++;
        if (linelen>0 && lineshortest>linelen) lineshortest=linelen;
        if (linelongest<linelen) linelongest=linelen;
        lines++;
        wasspace=TRUE;
        linelen=0;
        break;
      case '\t':
      case ' ':
        chrasutf8++;
        chrsp++;
        wasspace=TRUE;
        linelen++;
        break;
      default:
        if (*(unsigned char *)str<0x80 || *(unsigned char *)str>0xBF) chrasutf8++; //http://www.cl.cam.ac.uk/~mgk25/unicode.html#mod
        if (wasspace) words++;
        chrsp++;
        chrnsp++;
        wasspace=FALSE;
        linelen++;
        break;
    	}
    }
    if (linelen) {lines++; linesnonblank++;}
    MessageBoxFree(nppData._nppHandle,smprintf("Words:%u\r\nCharacters (no spaces):%u\r\nCharacters:%u\r\nUTF-8 Characters:%u\r\nUTF-8 Efficiency:%u%%\r\nLines:%u\r\nLongest Line:%u\r\nShortest nonblank line:%u\r\nNonblank lines:%u",
      words,chrnsp,chrsp,chrasutf8,chrnsp?(100*chrasutf8)/chrsp:0,lines,linelongest,lineshortest,linesnonblank),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  }
  return(0);
}

// SECTION: END
// SECTION: Beginning of Imported Functions

// strncpy() is a dangerous function. Per spec, it is not required to place a \0 byte
// at the end of a string. This is an unfortunate definition because though this
// is a valuable use for strncpy(), such a feature is almost never used intentionally.
// strncpy() is used to prevent buffer overruns and ends up causing them through
// undesirable unintended behaviour.
// If the macro expansion causes n to be calculated twice, this is better implemented
// as a function.
// strncpyX() functions the way everyone wants strncpy() to
// #define strncpyX(dest,src,n) { strncpy(dest,src,n-1); dest[n-1]='\0'; }

#if ENABLE_TIDYDLL /* { */
// __fastcall is unacceptable as a calling sequence becuase it is not compatible from compiler to compiler http://support.microsoft.com/kb/100832
// I compiled libTidy.dll with __fastcall in MinGW32 and NPPTextFX with __fastcall in Borland-C and I got crash city!
#ifndef TIDY_CALL_CDECL
#define TIDY_CALL_CDECL 1 /* All the web sources for libTidy.dll that I can find (Sep 2005) are using __cdecl which is the best one! */
#endif
#ifndef TIDY_CALL_STDCALL
#define TIDY_CALL_STDCALL 0
#endif

// These #defines are set up for runtime linking. They are easily changed to produce static or dynamic linking.
#if TIDY_CALL_CDECL /* { */
#define TIDY_CALL __cdecl
#define TIDY_CALLQT "__cdecl"
#define TIDYGENDLLINDIRECT2(DLLFUNCTYPE,DLLFUNCNAME,FSTCALLSIZE,DLLFUNCARGS) if (!(TIDYDLLCALL(DLLFUNCNAME)=(TIDYTD_##DLLFUNCNAME *)GetProcAddress(HItidydll,#DLLFUNCNAME))) {errfunc=#DLLFUNCNAME; goto fail;}
#elif TIDY_CALL_STDCALL
#define TIDY_CALL __stdcall
#define TIDY_CALLQT "__stdcall"
#define TIDYGENDLLINDIRECT2(DLLFUNCTYPE,DLLFUNCNAME,FSTCALLSIZE,DLLFUNCARGS) if (!(TIDYDLLCALL(DLLFUNCNAME)=(TIDYTD_##DLLFUNCNAME *)GetProcAddress(HItidydll,"_" #DLLFUNCNAME FSTCALLSIZE))) {errfunc="_" #DLLFUNCNAME FSTCALLSIZE; goto fail;}
#else
#error You must pick TIDY_CALL_CDECL or TIDY_CALL_STDCALL
#endif /* } */
#include "../tidy/include/tidy.h"
#include "../tidy/include/buffio.h"

#define TIDYDLLCALL(DLLFUNCNAME) TIDYFC_##DLLFUNCNAME
#define TIDYGENDLLINDIRECT1(DLLFUNCTYPE,DLLFUNCNAME,FSTCALLSIZE,DLLFUNCARGS) typedef DLLFUNCTYPE (FAR TIDY_CALL TIDYTD_##DLLFUNCNAME)DLLFUNCARGS; TIDYTD_##DLLFUNCNAME *TIDYDLLCALL(DLLFUNCNAME)
#define TIDYGENDLLINDIRECT TIDYGENDLLINDIRECT1
#define TIDYDLLINIT \
  TIDYGENDLLINDIRECT(TidyDoc,tidyCreate,"@0",(void));\
  TIDYGENDLLINDIRECT(void,tidyRelease,"@4",(TidyDoc));\
  TIDYGENDLLINDIRECT(Bool,tidyFileExists,"@4",(ctmbstr));\
  TIDYGENDLLINDIRECT(int,tidyLoadConfig,"@8",(TidyDoc,ctmbstr));\
  TIDYGENDLLINDIRECT(Bool,tidyOptResetToDefault,"@8",(TidyDoc,TidyOptionId));\
  TIDYGENDLLINDIRECT(Bool,tidyOptGetBool,"@8",(TidyDoc,TidyOptionId));\
  TIDYGENDLLINDIRECT(Bool,tidyOptSetBool,"@12",(TidyDoc,TidyOptionId,Bool));\
  TIDYGENDLLINDIRECT(Bool,tidyOptSetInt,"@12",(TidyDoc,TidyOptionId,ulong));\
  TIDYGENDLLINDIRECT(ulong,tidyOptGetInt,"@8",(TidyDoc,TidyOptionId));\
  TIDYGENDLLINDIRECT(ctmbstr,tidyOptGetValue,"@8",(TidyDoc,TidyOptionId));\
  TIDYGENDLLINDIRECT(Bool,tidyOptParseValue,"@12",(TidyDoc,ctmbstr,ctmbstr));\
  TIDYGENDLLINDIRECT(Bool,tidyOptSetValue,"@12",(TidyDoc,TidyOptionId,ctmbstr));\
  TIDYGENDLLINDIRECT(int,tidySetCharEncoding,"@8",(TidyDoc,ctmbstr));\
  TIDYGENDLLINDIRECT(FILE*,tidySetErrorFile,"@8",(TidyDoc,ctmbstr));\
  TIDYGENDLLINDIRECT(int,tidySetErrorBuffer,"@8",(TidyDoc,TidyBuffer*));\
  TIDYGENDLLINDIRECT(uint,tidyErrorCount,"@4",(TidyDoc));\
  TIDYGENDLLINDIRECT(uint,tidyWarningCount,"@4",(TidyDoc));\
  TIDYGENDLLINDIRECT(uint,tidyAccessWarningCount,"@4",(TidyDoc));\
  TIDYGENDLLINDIRECT(int,tidyParseString,"@8",(TidyDoc,ctmbstr));\
  TIDYGENDLLINDIRECT(int,tidyCleanAndRepair,"@4",(TidyDoc));\
  TIDYGENDLLINDIRECT(int,tidyRunDiagnostics,"@4",(TidyDoc));\
  TIDYGENDLLINDIRECT(int,tidySaveBuffer,"@8",(TidyDoc,TidyBuffer*));\
  TIDYGENDLLINDIRECT(void,tidyBufFree,"@4",(TidyBuffer*));\
  TIDYGENDLLINDIRECT(ctmbstr,tidyReleaseDate,"@0",(void));\
  TIDYGENDLLINDIRECT(TidyOptionId,tidyOptGetIdForName,"@4",(ctmbstr));

TIDYDLLINIT /* Here TIDYDLLINIT creates all the typedefs and function pointers */
#undef TIDYGENDLLINDIRECT
#define TIDYGENDLLINDIRECT TIDYGENDLLINDIRECT2 /* Later TIDYDLLINIT is reused to perform the imports. */

#define NPPTIDY_CFG "HTMLTIDY.CFG"
unsigned ScintillaEOLtoTidy[3]={TidyCRLF,TidyCR,TidyLF};
// functions are extracted from tidy.c:main() and the sample code in tidy.h
unsigned tidyHTML(char **dest,unsigned *destsz,unsigned *destlen,unsigned eoltype,unsigned tabwidth) {
  unsigned sln,rv=0;
  if (*dest) {
    char *str=*dest;
    sln=*destlen;
    char *path;
    do {
      if (!(path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,NPPTIDY_CFG))) break;
      TidyBuffer output; memset(&output,0,sizeof(output));
      TidyBuffer errbuf; memset(&errbuf,0,sizeof(errbuf));
      int rc = -1;
      BOOL ok;

      TidyDoc tdoc = TIDYDLLCALL(tidyCreate)();                     // Initialize "document"
      if ( TIDYDLLCALL(tidyFileExists)(path) ) {
        int status = TIDYDLLCALL(tidyLoadConfig)( tdoc, path );
        if ( status != 0 ) {
          MessageBoxFree(nppData._nppHandle,smprintf("Loading config file \"%s\" failed\r\nErr = %d (bad option?)",path,status),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
          //ok=no;
          break;
        } else ok=yes;
      } else {
        if (eoltype>=NELEM(ScintillaEOLtoTidy)) eoltype=NELEM(ScintillaEOLtoTidy)-1;
        ok = TIDYDLLCALL(tidyOptSetBool)(tdoc, TidyXhtmlOut, yes ) && // Convert to XHTML
             TIDYDLLCALL(tidyOptSetInt)(tdoc, TidyIndentSpaces,tabwidth) &&
             TIDYDLLCALL(tidyOptSetInt)(tdoc, TidyTabSize,tabwidth) &&
             TIDYDLLCALL(tidyOptSetInt)(tdoc, TidyNewline,ScintillaEOLtoTidy[eoltype]);
        // there might be a document type we can set from from Scintilla/Notepad++
        MessageBoxFree(nppData._nppHandle,smprintf("Config file \"%s\"\r\nnot found! I'll pick whatever defaults I want to!",path),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
      }
      if (ok     ) rc = TIDYDLLCALL(tidySetErrorBuffer)( tdoc, &errbuf );      // Capture diagnostics
      if (rc >= 0) rc = TIDYDLLCALL(tidyParseString)( tdoc, str );           // Parse the input
      if (rc >= 0) rc = TIDYDLLCALL(tidyCleanAndRepair)( tdoc );               // Tidy it up!
      if (rc >= 0) rc = TIDYDLLCALL(tidyRunDiagnostics)( tdoc );               // Kvetch
      if (rc > 1 ) rc = (TIDYDLLCALL(tidyOptSetBool)( tdoc, TidyForceOutput, yes)?rc:-1); // If error, force output.
      if (rc >= 0) rc = TIDYDLLCALL(tidySaveBuffer)( tdoc, &output );          // Pretty Print
      if (rc >= 0) {
        char *str2; if (!output.size) {} else if (str2=strdupsafe((char *)output.bp,"tidyHTML")) {
          freesafe(str,"tidyHTML");
          *dest=str=str2;
          sln=output.size;
          *destsz=sln+1;
          rv=1;
        } else MessageBox(nppData._nppHandle,"Out of memory in "PLUGIN_NAME" (not a HTMLtidy error)",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
        freesafe(path,"tidyHTML");
        if (!(path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,"HTMLTIDY.ERR"))) goto tidyfree;
        DeleteFile(path);
        if (rc > 0) {
          MessageBoxFree(nppData._nppHandle,smprintf("Writing error summary to\r\n%s",path),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
          FILE *fo;
          if (fo=fopen(path,"wt")) {
            fputs((char *)errbuf.bp,fo);
            fclose(fo);
          }
        }
      } else {
        MessageBoxFree(nppData._nppHandle,smprintf("A severe error (%d) occurred.\n",rc),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
      }
tidyfree:
      TIDYDLLCALL(tidyBufFree)(&output);
      TIDYDLLCALL(tidyBufFree)(&errbuf);
      TIDYDLLCALL(tidyRelease)(tdoc);
    } while(0);
    if (path) freesafe(path,"tidyHTML");
    *destlen=sln;
  }
  return(rv);
}
#endif /* } */

// SECTION: END
// SECTION: Beginning of Windows Only Library Functions

#include "Scintilla\UniConversion.h"
// returns malloc() clipboard text in UTF-8 or ANSI which the caller is expected to free
// returns NULL if unable or the clipboard is empty
// writes size to *size, which may be NULL if the caller does not need to know the size
// binary==TRUE: size=size of entire buffer; You need to specify a size parameter.
// binary=FALSE: size=size of text not including \0. Buffer size is always +1.
// eoltype = SCI_GETEOLMODE; can be set to any value if SCDS_PASTETOEDITOREOL is not set.
// This unified set of flags is used by several routines. They are unified so the flags can
// be passed among them without having to filter or translate flag bits. Each routine only
// looks at the bits applicable to itself.
#define SCDS_COPY 0x1   /* To cut, specify SCDS_COPY|SCDS_DELETE */
#define SCDS_DELETE 0x2
#define SCDS_COPYRECTANGULAR 0x4 /* Mark the text as from a rectangular selection according to the Scintilla standard */
#define SCDS_COPYAPPEND 0x8 /* Reads the old clipboard (depends on flags) and appends the new text to it */
#define SCDS_COPYEOLALWAYSCRLF 0x10 /* EOL are converted CRLF before placing on the clipboard so the text will paste properly in apps that can't handle CR or LF termination. */
#define SCDS_COPYALSOUTF8 0x20 /* UNICODE is always copied in UNICODEMODE, this also copies UTF-8 into CF_TEXT in UNICODE */
#define SCDS_COPYCONVERTNULLSTOSPACE 0x40 /* Convert NUL to space for non binary safe editors */
#define SCDS_COPYNOTUNICODE 0x80 /* Do not convert UTF-8 to UNICODE for CF_UNICODETEXT. System will automatically produce wchar_t UTF-8 which will paste into any application as UTF-8. */
#define SCDS_UNICODEMODE 0x100 /* Paste: Paste CF_TEXT in ANSI mode, CF_UNICODETEXT in UTF-8 OR UCS-2 modes; Copy: produce CF_UNICODETEXT in UTF-8/UCS-2 modes, produce CF_TEXT in ANSI mode */
#define SCDS_COPYPASTEBINARY 0x200 /* Retrieve entire Clipboard with last nul character removed; otherwise paste strlen() */
#define SCDS_PASTEANSIUTF8 0x400 /* Always Paste from CF_TEXT */
#define SCDS_PASTETOEDITOREOL 0x800 /* Convert all EOL's to SCI_GETEOLMODE */
// new feature: Unlike SCI_PASTE, this does not do Unicode -> OEM conversion in ANSI mode
char *strdupClipboardText(unsigned *size,unsigned flags,unsigned eoltype,BOOL *isrectangular)  {
  char *rv=NULL;
  UINT cf;
  unsigned chsize;

  //wchar_t uc=0xFF11; unsigned u=uc; MessageBoxFree(nppData._nppHandle,smprintf("unsigned:%u wchar_t:%u",u,uc),PLUGIN_NAME, MB_OK); // this is a test to see if wchar_t is unsigned
  if (size) *size=0;
  if (isrectangular) *isrectangular=IsClipboardFormatAvailable(cfColumnSelect);
  if ((flags&SCDS_PASTEANSIUTF8) || !IsClipboardFormatAvailable(CF_UNICODETEXT)) flags &= ~SCDS_UNICODEMODE; // safe as long as we don't pass the flags to another function
  if (flags&SCDS_COPYPASTEBINARY) flags &= ~(SCDS_COPYEOLALWAYSCRLF|SCDS_PASTETOEDITOREOL|SCDS_COPYCONVERTNULLSTOSPACE);
  if (flags&SCDS_UNICODEMODE) {
    cf=CF_UNICODETEXT;
    chsize=2;
  } else {
    cf=CF_TEXT;
    chsize=1;
  }
  if (IsClipboardFormatAvailable(cf) && OpenClipboard(nppData._nppHandle)) {
    DWORD clipsz;
    HGLOBAL   hglb;
    char *clip;
    if ((hglb = GetClipboardData(cf)) && (clipsz=GlobalSize(hglb))>chsize && (clip = (char *)GlobalLock(hglb))) {
      //MessageBoxFree(nppData._nppHandle,smprintf("Binary %s clipsz:%u",(flags&SCDS_COPYPASTEBINARY)?"TRUE":"FALSE",clipsz),PLUGIN_NAME, MB_OK);
      if (flags&SCDS_UNICODEMODE) { // guarantee a \0 for misbehaving applications
        *((wchar_t *)clip+clipsz/2-1)=L'\0';
      } else {
        clip[clipsz-1]='\0';
      }
      unsigned clipszextraorig;
      if (flags&SCDS_COPYPASTEBINARY) {
        clipsz-=chsize;
        clipszextraorig=0;
      } else {
        clipsz=(flags&SCDS_UNICODEMODE)?(wcslen((wchar_t *)clip)*2):strlen(clip);
        clipszextraorig=chsize;
        //MessageBoxFree(nppData._nppHandle,smprintf("clipsize:%u clipszextraorig:%u",clipsz,clipszextraorig),PLUGIN_NAME, MB_OK);
      }
      if (clipsz) {
        unsigned rvlen=clipsz,rvsz=clipsz+clipszextraorig;
        if (flags&SCDS_UNICODEMODE) {
          rvsz=UTF8Length((wchar_t *)clip,rvsz/2);
          rvlen=rvsz-(clipszextraorig?1:0);
        }
        if (rv=(char *)mallocsafe(rvsz,"strdupClipboardText")) {
          if (flags&SCDS_UNICODEMODE) UTF8FromUCS2((wchar_t *)clip,(clipsz+clipszextraorig)/2,rv,rvsz);
          else memcpy(rv,clip,rvsz);
          if (flags&SCDS_PASTETOEDITOREOL) {
            //MessageBoxFree(nppData._nppHandle,smprintf("Converting EOL to %u",eoltype),PLUGIN_NAME, MB_OK);
            converteol(&rv,&rvsz,&rvlen,eoltype);
            if (!(flags&SCDS_COPYPASTEBINARY) && rv) {
              armreallocsafe(&rv,&rvsz,rvlen+1,ARMSTRATEGY_REDUCE,0,"strdupClipboardText");
              if (rv) rv[rvlen]='\0';
            }
          }
          if (rv && size) *size=rvlen;
         }
       }
       GlobalUnlock(hglb);
     }
    CloseClipboard();
  }
  return rv;
}

// DO NOT \0 NUL terminate the string, it will be added automatically; buflen should be the length of the actual data
// returns TRUE if copying succeeded
// depending on the flags, the original buffer may get modified
// http://www.ultraedit.com/index.php?name=Forums&file=viewtopic&t=664
BOOL SetClipboardDataSingle(char **buf,unsigned *bufsz,unsigned *buflen,unsigned flags,unsigned eoltype)  {
  BOOL rv=FALSE;

  if (*buf && *buflen) do {
    HGLOBAL hglbnewclipUTF8;
    char *pnewclipUTF8,*apbuf=NULL;
    unsigned apbuflen=0;
    if (flags&SCDS_COPYPASTEBINARY) flags &= ~(SCDS_COPYEOLALWAYSCRLF|SCDS_PASTETOEDITOREOL|SCDS_COPYCONVERTNULLSTOSPACE);
    if (flags&SCDS_COPYCONVERTNULLSTOSPACE) memchrtran1(*buf,*buflen,'\0',' ');
    if (flags&SCDS_COPYEOLALWAYSCRLF) {converteol(buf,bufsz,buflen,SC_EOL_CRLF); if (!*buf) break;}
    if (flags&SCDS_COPYNOTUNICODE) flags &= ~SCDS_UNICODEMODE; // I'm not sure whether before or after strdupClipboardText() is the best way; remember that SCDS_PASTEANSIUTF8 is not a [x]Setting!
    if (flags&SCDS_COPYAPPEND) apbuf=strdupClipboardText(&apbuflen,flags,eoltype,NULL);
    if (OpenClipboard(nppData._nppHandle)) {
      EmptyClipboard();
#define UTF8SIZE (apbuflen+*buflen+1) /* always \0 terminated */
      if (hglbnewclipUTF8 = GlobalAlloc(GMEM_MOVEABLE,UTF8SIZE)) {
        pnewclipUTF8 = (char *)GlobalLock(hglbnewclipUTF8);
        if (apbuf) {
          memcpy(pnewclipUTF8,apbuf,apbuflen); // apbuflen doesn't include the \0
          //pnewclipUTF8[apbuflen] = '\0'; MessageBoxFree(nppData._nppHandle,smprintf("Reuse %s",pnewclipUTF8),PLUGIN_NAME, MB_OK);
        }
        memcpy(pnewclipUTF8+apbuflen, *buf, *buflen); // buflen doesn't include the \0
        pnewclipUTF8[UTF8SIZE-1] = '\0';
        //MessageBoxFree(nppData._nppHandle,smprintf("Total %s len:%u",pnewclipUTF8,*buflen+apbuflen),PLUGIN_NAME, MB_OK);
        HGLOBAL hglbnewclipUNICODE=NULL;
        if (flags&SCDS_UNICODEMODE) {
          unsigned newclipUNICODEsz=UCS2Length(pnewclipUTF8,UTF8SIZE);
          //MessageBoxFree(nppData._nppHandle,smprintf("newclipUNICODEsz:%u (UNICODE)",newclipUNICODEsz),PLUGIN_NAME, MB_OK);
          if (hglbnewclipUNICODE=GlobalAlloc(GMEM_MOVEABLE,newclipUNICODEsz*2)) {
            wchar_t *pnewclipUNICODE=(wchar_t *)GlobalLock(hglbnewclipUNICODE);
            /*if (*/UCS2FromUTF8(pnewclipUTF8,UTF8SIZE,pnewclipUNICODE,newclipUNICODEsz); //!=newclipUNICODEsz) MessageBoxFree(nppData._nppHandle,smprintf("Wrong size:%u (UNICODE)",newclipUNICODEsz),PLUGIN_NAME, MB_OK);
          } else flags |= SCDS_COPYALSOUTF8; // get something copied
        }
        GlobalUnlock(hglbnewclipUTF8);
        if (!IsNT() || flags&SCDS_COPYALSOUTF8 || !(flags&SCDS_UNICODEMODE)) {
          SetClipboardData(CF_TEXT,hglbnewclipUTF8);
        } else {
          GlobalFree(hglbnewclipUTF8);
        }
        if (hglbnewclipUNICODE) {
          GlobalUnlock(hglbnewclipUNICODE);
          SetClipboardData(CF_UNICODETEXT,hglbnewclipUNICODE);
        }
#undef UTF8SIZE
        if (flags&SCDS_COPYRECTANGULAR) SetClipboardData(cfColumnSelect,0);
        rv=TRUE;
      }
      CloseClipboard();
    }
    if (apbuf) freesafe(apbuf,"SetClipboardDataSingle");
  } while(0);
  return(rv);
}

#if NPPDEBUG
// http://www.vbaccelerator.com/home/VB/Tips/Determine_All_Formats_On_Clipboard/article.asp
PFUNCPLUGINCMD pfinsertclipinfo() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  if (OpenClipboard(nppData._nppHandle)) {
    char *buf=NULL; unsigned bufsz=0,buflen;
    sarmprintf(&buf,&bufsz,&buflen,"**CountClipboardFormats()=%u\r\n",CountClipboardFormats());
    for(UINT cf=0,cfsz=0,cfct=0; cf=EnumClipboardFormats(cf); ) {
      HGLOBAL hglb;
      sarmprintf(&buf,&bufsz,&buflen,"%5u:",cf);
      switch(cf) {
      case CF_TEXT        : strcpyarmsafe(&buf,&bufsz,&buflen,"Text (ANSI)","pfinsertclipinfo"); goto getdetails;
      case CF_BITMAP      : strcpyarmsafe(&buf,&bufsz,&buflen,"Bitmap","pfinsertclipinfo"); break;
      case CF_METAFILEPICT: strcpyarmsafe(&buf,&bufsz,&buflen,"MetaFile Picture","pfinsertclipinfo"); break;
      case CF_SYLK        : strcpyarmsafe(&buf,&bufsz,&buflen,"Symbolic Link (SYLK)","pfinsertclipinfo"); break;
      case CF_DIF         : strcpyarmsafe(&buf,&bufsz,&buflen,"Data Interchange Format","pfinsertclipinfo"); break;
      case CF_TIFF        : strcpyarmsafe(&buf,&bufsz,&buflen,"TIFF","pfinsertclipinfo"); break;
      case CF_OEMTEXT     : strcpyarmsafe(&buf,&bufsz,&buflen,"Text (OEM)","pfinsertclipinfo"); goto getdetails;
      case CF_DIB         : strcpyarmsafe(&buf,&bufsz,&buflen,"DIB Device Independant Bitmap","pfinsertclipinfo"); break;
      case CF_PALETTE     : strcpyarmsafe(&buf,&bufsz,&buflen,"Palette","pfinsertclipinfo"); break;
      case CF_PENDATA     : strcpyarmsafe(&buf,&bufsz,&buflen,"Pen Data","pfinsertclipinfo"); break;
      case CF_RIFF        : strcpyarmsafe(&buf,&bufsz,&buflen,"RIFF Audio","pfinsertclipinfo"); break;
      case CF_WAVE        : strcpyarmsafe(&buf,&bufsz,&buflen,"Wave Audio","pfinsertclipinfo"); break;
      case CF_UNICODETEXT :
        strcpyarmsafe(&buf,&bufsz,&buflen,"Text (Unicode)","pfinsertclipinfo");
getdetails:
        if ((hglb = GetClipboardData(cf))) {
          DWORD clipsz=GlobalSize(hglb);
          sarmprintf(&buf,&bufsz,&buflen,";len=%u;",clipsz);
          char *clip = (char *)GlobalLock(hglb);
          memcpyarmsafe(&buf,&bufsz,&buflen,clip,clipsz,"pfinsertclipinfo");
          GlobalUnlock(hglb);
        }
        break;
      case CF_ENHMETAFILE : strcpyarmsafe(&buf,&bufsz,&buflen,"Enhanced MetaFile","pfinsertclipinfo"); break;
      case CF_HDROP       : strcpyarmsafe(&buf,&bufsz,&buflen,"File List (HDROP)","pfinsertclipinfo"); break;
      case CF_LOCALE      : strcpyarmsafe(&buf,&bufsz,&buflen,"Text Locale Identifier","pfinsertclipinfo"); goto getdetails;
      default             :
        if (cf>=0xC000 && cf<=0xFFFF) {
          char fmt[128];
          GetClipboardFormatName(cf,fmt,sizeof(fmt));
          sarmprintf(&buf,&bufsz,&buflen,"Registered Format: %s",fmt); break;
        } else {
          strcpyarmsafe(&buf,&bufsz,&buflen,"(unknown)","pfinsertclipinfo"); break;
        }
      }
      strcpyarmsafe(&buf,&bufsz,&buflen,"\r\n","pfinsertclipinfo");
    }
failbreak:
    CloseClipboard();
    if (buf) {
      SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0,0);
      SENDMSGTOCED(currentEdit, SCI_ADDTEXT, buflen,buf);
      freesafe(buf,"pfinsertclipinfo");
    }
  }
}
#endif

// SECTION: END
// SECTION: Beginning of Scintilla Interface

//extends anchor and curpos out to bound entire entire lines
//returns TRUE if curpos or anchor was altered, meaning that either curpos or anchor was not originally at a line boundry
//grabextralines: FALSE=extend the minimal amount to enclose lines; TRUE=include additional blank lines after the lowest part of the boundry
BOOL lineextend(INT_CURRENTEDIT,unsigned *anchor,unsigned *curpos,BOOL grabextralines) {
  BOOL rv=FALSE;

  unsigned p1,p2,p2bol,lineno; if (*curpos>*anchor) {p1= *anchor;p2= *curpos;} else {p1= *curpos;p2= *anchor;}
  p1=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE,SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,p1,0),0);
  lineno=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,p2,0);
  p2bol=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE,lineno,0);
  if (p2 != p2bol) p2=p2bol+SENDMSGTOCED(currentEdit, SCI_LINELENGTH,lineno++,0);
  int tl; if (grabextralines) for(tl=SENDMSGTOCED(currentEdit, SCI_GETLINECOUNT,0,0); lineno<=tl && !(SENDMSGTOCED(currentEdit, SCI_GETLINEENDPOSITION,lineno,0)-p2); lineno++,p2=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE,lineno,0)) ;
  if (*curpos>*anchor) {
    if (*anchor != p1 || *curpos != p2) {rv=TRUE;}
    *anchor=p1;
    *curpos=p2;
  } else {
    if (*anchor != p2 || *curpos != p1) {rv=TRUE;}
    *anchor=p2;
    *curpos=p1;
  }
  return(rv);
}

// bracematch() looks to the left then to the right of curpos for a brace,
// then it locates the matching brace. SCI_BRACEMATCH is not sufficient because:
//  SCI_BRACEMATCH  only matches brace characters, not keywords as bracematch() eventually will.
//  During a scNotify(), SCI_BRACEMATCH always fails to locate a brace
//  SCI_BRACEMATCH does not look on both sides of curpos for a brace as the user expects
//  SCI_BRACEMATCH does not have flags to correct or adjust position for user needs
// returns TRUE if matches were found, returns curpos1 and matchpos1 of the match positions
// unless otherwise noted, brace refers to all of the characters {([<>])}
// new feature: find <tag></tag> or <tag/> pairs
// new feature: find #if #endif /* } */ pairs
// new feature: find if (){}else{} groups
// new feature: find /* (* *) */ pairs
#define FLAG_IGNOREBEGINBRACE 1 /* fail without searching if the match for an endbrace is requested; Never enable both begin&end options */
#define FLAG_IGNOREENDBRACE 2  /* fail without searching if the match for an beginbrace is requested */
#define CAFLAG_EXTENDTOLINES 4 /* TRUE: extends curposl,matchpos1 to entire lines; This one is reused in convertall() */
#define FLAG_INCLUDEBRACKETS 8 /* TRUE: include bracket characters between curpos1,matchpos1; FALSE: only include between the brackets */
BOOL bracematch(INT_CURRENTEDIT, unsigned *curpos1,unsigned *matchpos1,unsigned flags) {
  char BKTS[]="{}()[]<>";
  if (currentEdit==INVALID_CURRENT_EDIT) GET_CURRENTEDIT;
  unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  if (SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, 0)-1>0) {
    unsigned anchor; if ((anchor=SENDMSGTOCED(currentEdit, SCI_GETANCHOR, 0, 0))==curpos+1) curpos=anchor;
  }
  char bracebuf[3],*bkt; struct TextRange tr; tr.chrg.cpMin=curpos-1; tr.chrg.cpMax=curpos+1; tr.lpstrText=bracebuf;
  if (tr.chrg.cpMin<0) tr.chrg.cpMin=0;
  unsigned textlen=SENDMSGTOCED(currentEdit, SCI_GETLENGTH, 0, 0);
  if (tr.chrg.cpMax>textlen) tr.chrg.cpMax=textlen;
  unsigned sln; if ((sln=SENDMSGTOCED(currentEdit, SCI_GETTEXTRANGE, 0, &tr))<1) return(FALSE);
  //MessageBoxFree(nppData._nppHandle,smprintf("sln:%d curpos:%d cpmin:%d cpmax:%d buf\r\n%s",sln,curpos,tr.chrg.cpMin,tr.chrg.cpMax,tr.lpstrText),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  if ((bkt=strchr(BKTS,bracebuf[0]))) curpos=tr.chrg.cpMin;
  else if (sln<2 || !(bkt=strchr(BKTS,bracebuf[1]))) return(FALSE);
  if ((flags&FLAG_IGNOREBEGINBRACE) && !((bkt-BKTS)&1) ||
     ((flags&FLAG_IGNOREENDBRACE) && ((bkt-BKTS)&1)) ) return(FALSE);
  unsigned bktpos=/*-1;// */SENDMSGTOCED(currentEdit, SCI_BRACEMATCH, curpos, 0);
  if (bktpos<0) { // in some cases, SCI_BRACEMATCH fails even though we are on a valid brace with a findable opposing brace, such as during a SCNotification
    int dir=((bkt-BKTS)&1)?-1:1;
    if (dir>0) {
      tr.chrg.cpMin=curpos+1;
      tr.chrg.cpMax=SENDMSGTOCED(currentEdit, SCI_GETLENGTH, curpos, 0);
    } else {
      tr.chrg.cpMin=0;
      tr.chrg.cpMax=curpos;
    }
    if (!(tr.lpstrText=(char *)mallocsafe(tr.chrg.cpMax-tr.chrg.cpMin+2,"bracematch"))) return(FALSE);
    unsigned sln=SENDMSGTOCED(currentEdit, SCI_GETTEXTRANGE, 0, &tr);
    unsigned level=1;
    bktpos=curpos;
    char *d; for(d=tr.lpstrText+(dir<0?(sln-1):0); level && ((dir<0)?(d>=tr.lpstrText):(d<tr.lpstrText+sln)); d += dir, bktpos+=dir) {
      if (*d==bkt[0]) level++;
      else if (*d==bkt[dir]) level--;
    }
    if (level) bktpos=-1;
    //MessageBoxFree(nppData._nppHandle,smprintf("cpmin:%d cpmax:%d sln:%d dir:%d level:%u ch:%c curpos:%d bktpos:%d *d:%c",tr.chrg.cpMin,tr.chrg.cpMax,sln,dir,level,bkt[0],curpos,bktpos,*d),PLUGIN_NAME, MB_OK); //MessageBox(nppData._nppHandle,tr.lpstrText,PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    freesafe(tr.lpstrText,"bracematch");
  }
  if (bktpos>=0) {
    //MessageBoxFree(nppData._nppHandle,smprintf("curpos:%d bktpos:%d includebkts:%d",curpos,bktpos,includebkts),PLUGIN_NAME, MB_OK); //MessageBox(nppData._nppHandle,tr.lpstrText,PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    if (curpos>bktpos && (flags & FLAG_INCLUDEBRACKETS)) curpos++;
    if (!(flags & FLAG_INCLUDEBRACKETS) || curpos<bktpos) bktpos++;
    if ((flags & CAFLAG_EXTENDTOLINES)) lineextend(currentEdit,&bktpos,&curpos,TRUE);
    *curpos1=curpos;
    *matchpos1=bktpos;
    return(TRUE);
  } else {
    return(FALSE);
  }
}

// convertall is here to do all the hard work getting text from and back to Scintilla which
// is big now but it all started with this simple goal:
// 1) Extract the selected text from Scintilla into a c-string buffer with spare space
// 2) Call a text transform on the buffer. The transform will halt if spare space runs out.
// 3) Put the buffer back into the document if the transform indicates that the user will want the result.
// 4) Reselect the entire text, or the failed text if a failure occured
// Rectangular selections are handled without any special programming in the transforms.
// Transforms do not need to handle restrictions. Transforms only work with a simple c-string.
#define CAFLAG_DENYBLOCK 1 /* We typically deny rectangular selection if the tranform has no use for partial lines or transformed text would fit poorly in the rectangular area. */
#define CAFLAG_REQUIREBLOCK 2 /* Setting both DENY and REQUIRE block would be a bad idea */
/* CAFLAG_EXTENDTOLINES is defined above */
#define CAFLAG_REQUIREMULTILINES 8 /* Two or more lines must be selected (doesn't always calculate right) */
#define CAFLAG_GETCLIPBOARDTEXT 0x10 /* Get and submit the clipboard but it is not required */
#define CAFLAG_REQUIRECLIPBOARDTEXT 0x20 /* Get ... and the clipboard cannot be empty */
#define CAFLAG_REQUIRECLIPBOARDTEXT1CHAR 0x40 /* Get ... and the clipboard must contain exactly 1 character */
#define CAFLAG_GETALLWHENNOSELECTION 0x80 /* Get the entire document if the user hasn't selected anything */
//#define CAFLAG_GETCURLINEWHENNOSELECTION 0x100 /* Get the current line the user hasn't selected anything */
#define CAFLAG_DENYBINARY 0x200 /* User is not permitted to select any text with \0 in it */
void convertall(char cmd,unsigned flags,const char *s1,const char *s2,const char *s3,const char *s4) {
#define CAFLAG_BLOCKMODE 0x1000 /* internal: true if block mode text was selected */
#define CAFLAG_HASBINARY 0x2000 /* internal: true if the text contained a \0 */
#define origexpand 16384
  char *tx=NULL,*clip=NULL;
  unsigned *lps=NULL,*lpe=NULL;

  do {
    INT_CURRENTEDIT; GET_CURRENTEDIT;
    //struct TextRange tr; tr.chrg.cpMin=p1; tr.chrg.cpMax=p2; //tr.lpstrText=buf;
    unsigned curpos; curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
    unsigned anchor; anchor=SENDMSGTOCED(currentEdit, SCI_GETANCHOR, 0, 0);
    unsigned eoltype=SENDMSGTOCED(currentEdit, SCI_GETEOLMODE, 0, 0);

    unsigned p1,p2;
    if (anchor<curpos) {p1=anchor; p2=curpos; } else {p1=curpos; p2=anchor;}
    unsigned p1line=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION, p1, 0);
    unsigned sellen=SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, 0)-1;
    if (sellen<1) {
      flags &= ~(CAFLAG_EXTENDTOLINES);
      if (flags & CAFLAG_GETALLWHENNOSELECTION) {
        sellen=SENDMSGTOCED(currentEdit, SCI_GETLENGTH, 0, 0);
        p1line=p1=0;
        p2=sellen-1;
        /*flags &= ~ CAFLAG_GETCURLINEWHENNOSELECTION;
      } else if (flags & CAFLAG_GETCURLINEWHENNOSELECTION) {
        p1=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE, p1line, 0); // p1==p2 here
        p2=SENDMSGTOCED(currentEdit, SCI_GETLINEENDPOSITION, p1line, 0); // does not include the EOL
        sellen=p2-p1+1;
        //MessageBoxFree(nppData._nppHandle,smprintf("p1:%u p2:%u sellen:%u",p1,p2,sellen),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
        flags &= ~ CAFLAG_GETALLWHENNOSELECTION;*/
      }
      if (sellen<1) {
        MessageBox(nppData._nppHandle,"No text selected or document is empty",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
        break;
      }
    } else flags &= ~(/*CAFLAG_GETCURLINEWHENNOSELECTION|*/CAFLAG_GETALLWHENNOSELECTION);
    if (sellen<1) {
      MessageBox(nppData._nppHandle,"No text selected",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
      break;
    }
    //if (curpos<anchor) {int less=anchor;anchor=curpos;curpos=less; flags |= CAFLAG_LESS;}
    if ((flags & (CAFLAG_GETCLIPBOARDTEXT|CAFLAG_REQUIRECLIPBOARDTEXT|CAFLAG_REQUIRECLIPBOARDTEXT1CHAR)) && !(clip=strdupClipboardText(NULL,(SENDMSGTOCED(currentEdit, SCI_GETCODEPAGE, 0, 0)==SC_CP_UTF8?SCDS_UNICODEMODE:0)|SCDS_PASTETOEDITOREOL,eoltype,NULL))) {
      if (flags & CAFLAG_REQUIRECLIPBOARDTEXT) {
        MessageBox(nppData._nppHandle,"This tool requires some text in the clipboard",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
        break;
      }
    }
    if ((flags & CAFLAG_REQUIRECLIPBOARDTEXT1CHAR) && (!clip || clip[1])) { // this isn't UNICODE ready since one character can be more than one
      MessageBox(nppData._nppHandle,"This tool requires exactly one character in the clipboard",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
      break;
    }
    unsigned p1diff=0;
    if (flags&CAFLAG_EXTENDTOLINES) {
      unsigned p1o=p1;
      if (lineextend(currentEdit,&p1,&p2,FALSE)) {
        p1line=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION, p1, 0); // I'm not sure if this is necessary
        p1diff=p1o-p1;
      }
      //flags &= ~CAFLAG_BLOCKMODE;
      SENDMSGTOCED(currentEdit, SCI_SETSEL, p1, p2);
      sellen=SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, 0)-1;
    }
    if (SENDMSGTOCED(currentEdit, SCI_SELECTIONISRECTANGLE, 0, 0)) flags |= CAFLAG_BLOCKMODE; else flags &= ~CAFLAG_BLOCKMODE;
    if (flags&CAFLAG_BLOCKMODE) {
      if (flags&CAFLAG_DENYBLOCK) {
        MessageBox(nppData._nppHandle,"This tool cannot process a retangular text selection",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
        break;
      }
    } else if (flags&CAFLAG_REQUIREBLOCK) {
      MessageBox(nppData._nppHandle,"This tool only works with a retangular text selection.\r\nHold down Alt and select some text.",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
      break;
    }
    unsigned p2line=(p1==p2)?p1line:SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION, p2, 0);
    unsigned blocklines=p2line-p1line+1-((SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE, p2line, 0) == p2)?1:0);
    //MessageBoxFree(nppData._nppHandle,smprintf("p1line:%u p2line:%u blocklines:%u p2:%u SCI_POS:%u",p1line,p2line,blocklines,p2,SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE, p2line, 0)),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    if (blocklines<2 /*((flags&CAFLAG_EXTENDTOLINES)?3:2)*/) { // 3:2 won't catch everything but it will catch most
      flags &= ~CAFLAG_BLOCKMODE; // you're not really in block mode if you've only marked one line
      if (flags&CAFLAG_REQUIREMULTILINES) {
        MessageBox(nppData._nppHandle,"This tool requires more than one line to be selected",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
        break;
      }
    }
    if (flags&CAFLAG_BLOCKMODE /* || hidden lines */) {
      lps=(unsigned *)mallocsafe(blocklines*sizeof(*lps),"convertall-bloclines"); if (!lps) break;
      lpe=(unsigned *)mallocsafe(blocklines*sizeof(*lpe),"convertall-bloclines"); if (!lpe) break;
      unsigned ln; for(ln=0; ln<blocklines; ln++) {
        unsigned lbof=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE  , (p1line+ln),0);
        lps[ln]= SENDMSGTOCED(currentEdit, SCI_GETLINESELSTARTPOSITION, (p1line+ln),0)-lbof;
        lpe[ln]= SENDMSGTOCED(currentEdit, SCI_GETLINESELENDPOSITION  , (p1line+ln),0)-lbof;
      }
    }
    // this could be recoded to convert a line at a time but issues such as long line length
    // and possible performance issues in calling into Scintilla so much make me leave it this way.
    // slow: calc line length, select line text, malloc, copy selected text into buffer, convert buffer, replace selection
    // Each select line text may cause a screen jump so it may be best to do it as one big buf
    // also, line by line processing wouldn't permit transforms to operate on \r\n characters
    unsigned txsz=roundtonextpower(sellen+origexpand+1);
    if (!(tx=(char *)mallocsafe(txsz,"convertall-txsz"))) {
      MessageBox(nppData._nppHandle,"Not enough memory",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
      break;
    }
#if NPPDEBUG
    char *txorig=tx; // functions that free and re-malloc() tx are not required to provide spare space
#endif
    unsigned rv=0,sln;
    if      (flags&CAFLAG_GETALLWHENNOSELECTION)     sln=SENDMSGTOCED(currentEdit, SCI_GETTEXT   , (sellen+1), tx);
    //else if (flags&CAFLAG_GETCURLINEWHENNOSELECTION){SENDMSGTOCED(currentEdit, SCI_GETCURLINE, (sellen+1), tx); sln=sellen; }
    else                                             sln=SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0         , tx)-1;
#if NPPDEBUG
    if (sln != sellen) MessageBoxFree(nppData._nppHandle,smprintf("Mismatch sellen:%u sln:%u",sellen,sln),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
#endif
    if (memchr(tx,0,sln)) {
    	flags |= CAFLAG_HASBINARY;
    	if (flags&CAFLAG_DENYBINARY) {
        MessageBox(nppData._nppHandle,"This tool is not compatible with binary text. Please select text without [NUL] characters.",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
        break;
    	}
    } else flags &= ~CAFLAG_HASBINARY;
    SENDMSGTOCED(currentEdit, SCI_SETCURSOR, SC_CURSORWAIT, 0);
    switch(cmd) {
    case 'P': rv=mempropercase(tx,sln); break;
    case 'p': rv=meminvertcase(tx,sln); break;
    case 's': rv=memsentencecase(tx,sln); break;
    case 'c': rv=memchrtran1(tx,sln,*s1,*s2); break;
    case 'e': rv=asciiEBCDIC(tx,sln,*s1=='a'?0:1); break;
    case 'z': rv=zapchar(tx,sln,*s1,*s2=='n'?1:0); break;
    case '+': rv=addup(tx); break;
    case '-': rv=wordcount(tx,sln); break;

    case 'S': rv=memstrtran(&tx,&txsz,&sln,NULL,s1,strlen(s1),s2,strlen(s2)); if (s3 && s4) rv+=memstrtran(&tx,&txsz,&sln,NULL,s3,strlen(s3),s4,strlen(s4)); break;
    case 'Z': rv=strchrstrans(&tx,&txsz,&sln,NULL,s1,strlen(s1),s2); break;
    case '&': rv=prepostpendlines(&tx,&txsz,&sln,*s3=='s'?0:1,s1,strlen(s1),s2,strlen(s2),s3+1,strlen(s3+1),s4,strlen(s4)); break;
    case ',': rv=lineup(&tx,&txsz,&sln,clip?*clip:*s1,SENDMSGTOCED(currentEdit, SCI_GETUSETABS, 0, 0),SENDMSGTOCED(currentEdit, SCI_GETTABWIDTH, 0, 0),1 /* C-STRING */); break;
    case '\t': rv=space2tabs(&tx,&txsz,&sln,SENDMSGTOCED(currentEdit, SCI_GETUSETABS, 0, 0),s1?atoi(s1):SENDMSGTOCED(currentEdit, SCI_GETTABWIDTH, 0, 0),0); break;
    case '|': rv=insertclipboardcolumn(&tx,&txsz,&sln,clip,p1diff); break;
    case 'W':
    case 'w': rv=rewraptexttest(&tx,&txsz,&sln,(cmd=='W'?0:((clip&&isdigit(*clip))?atoi(clip):1)),eoltype); break;
    case 'h': rv=stripHTMLtags(&tx,&txsz,&sln,s1); break;
    case 'i': rv=indentlines(&tx,&txsz,&sln,SENDMSGTOCED(currentEdit, SCI_GETUSETABS, 0, 0),SENDMSGTOCED(currentEdit, SCI_GETTABWIDTH, 0, 0)); break;
    case 'I': rv=indentlinessurround(&tx,&txsz,&sln,SENDMSGTOCED(currentEdit, SCI_GETUSETABS, 0, 0),SENDMSGTOCED(currentEdit, SCI_GETTABWIDTH, 0, 0),eoltype); break;
    case 'q': rv=strqsortlines(&tx,&txsz,&sln,*s1=='n'?1:0,*s2=='a'?1:0,p1diff); break;
    case ' ': rv=killwhitenonqt(tx,txsz,&sln,s1[0]=='v'?1:0); break;
    case '"': rv=findqtstrings(&tx,&txsz,&sln,clip?clip:s1,s2[0]=='v'?1:0); break;
    case 't': rv=trimtrailingspace(tx,txsz,&sln); break;
    case 'd': rv=deleteblanklines(tx,txsz,&sln,atol(s1)); break;
    //case 'N': rv=deleteeverynthline(tx,txsz,&sln,atoi(s1)); break;
    case 'A': rv=reindentcode(&tx,&txsz,&sln,SENDMSGTOCED(currentEdit, SCI_GETUSETABS, 0, 0),SENDMSGTOCED(currentEdit, SCI_GETTABWIDTH, 0, 0)); break;
    case '3': rv=submitW3C(&tx,&txsz,&sln,s1,s2); break;
    case 'n': rv=numberconvert(&tx,&txsz,&sln,*s1,*s2); break;
    case 'H': rv=encodeURIcomponent(&tx,&txsz,&sln); break;
    case 'f': rv=filldown(&tx,&txsz,&sln,*s1=='i'?1:0); break;
    case 'C': rv=memchrtran(&tx,&txsz,&sln,s1,strlen(s1),s2,strlen(s2)); break;
    case 'X': rv=tohex(&tx,&txsz,&sln,anchor,atol(s1)); break;
    case 'x': rv=fromhex(&tx,&txsz,&sln,anchor); flags |= CAFLAG_HASBINARY; break;
    case 'l': rv=insertlinenumbers(&tx,&txsz,&sln,p1line+1); break;
    case 'L': rv=deletefirstword(&tx,&txsz,&sln); break;
    case 'E': rv=extendblockspaces(&tx,&txsz,&sln); break;
    case 'Q': rv=cleanemailquoting(&tx,&txsz,&sln); break;
    case 'u': rv=uudecode(&tx,&txsz,&sln); flags |= CAFLAG_HASBINARY; break;
    case 'U': rv=base64decode(&tx,&txsz,&sln); flags |= CAFLAG_HASBINARY; break;
    case '4': rv=hexbyterunstolittlendian(&tx,&txsz,&sln,atol(s1)); break;
    case '5': rv=littlendiantohexbyteruns(&tx,&txsz,&sln); break;
    case 'O': rv=converteol(&tx,&txsz,&sln,eoltype); break;
    case '~': rv=splitlinesatch(&tx,&txsz,&sln,(clip && !clip[1])?*clip:*s1,s2[0]=='v'?1:0,eoltype); break;
#if ENABLE_TIDYDLL
    case 'T': rv=tidyHTML(&tx,&txsz,&sln,eoltype,SENDMSGTOCED(currentEdit, SCI_GETTABWIDTH, 0, 0)); break;
#endif
    }
    if (tx) {
#if NPPDEBUG
      if (!(flags&CAFLAG_HASBINARY) && strlen(tx) != sln) MessageBoxFree(nppData._nppHandle,smprintf("sellen (sln) not calculated properly:\nstrlen(tx):%u != sln:%u",strlen(tx),sln),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
#endif
      if (rv) {
        SENDMSGTOCED(currentEdit, SCI_BEGINUNDOACTION, 0, 0);
        if (flags & CAFLAG_BLOCKMODE) {
          unsigned ln;
          char *d,*end;
          for(ln=0,d=tx,end=d+sln; ln<blocklines && d<end; ln++) {
            unsigned chn=memcspn(d,tx+sln,"\r\n",2)-d;
            if (lpe[ln]>lps[ln]) {
              int lbof= SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE, (p1line+ln),0);
              //SENDMSGTOCED(currentEdit, SCI_SETSEL, (lps[ln]+lbof), (lpe[ln]+lbof));
              //if (rv) SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, "");
              //if (rv) SENDMSGTOCED(currentEdit, SCI_ADDTEXT, chn, d);
              SENDMSGTOCED(currentEdit, SCI_SETTARGETSTART, lps[ln]+lbof, 0);
              SENDMSGTOCED(currentEdit, SCI_SETTARGETEND, lpe[ln]+lbof,0);
              SENDMSGTOCED(currentEdit, SCI_REPLACETARGET, chn, d);
            }
            d += chn;
            if (d[0]=='\r' && d[1]=='\n' || d[0]=='\n' && d[1]=='\r') d+=2;
            else if (d[0]=='\r' || d[0]=='\n') d++;
          }
          /*curpos = lpe[blocklines-1]+SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE, p1line+blocklines-1, 0);
          if (curpos<anchor) curpos=anchor+1; // carefully constructed cases can push anchor ahead of curpos
          SENDMSGTOCED(currentEdit, SCI_SETANCHOR,(flags&CAFLAG_LESS)?curpos:anchor, 0);
          SENDMSGTOCED(currentEdit, SCI_SETSELECTIONMODE,SC_SEL_RECTANGLE, 0);
          SENDMSGTOCED(currentEdit, SCI_SETCURRENTPOS,(flags&CAFLAG_LESS)?anchor:curpos, 0);*/
        } else {
          p2 += sln-sellen;
          if (rv) {
            if (flags&CAFLAG_GETALLWHENNOSELECTION) {
              SENDMSGTOCED(currentEdit, SCI_SETTEXT, 0, "");
              SENDMSGTOCED(currentEdit, SCI_ADDTEXT, sln, tx);
            } /*else if (flags&CAFLAG_GETCURLINEWHENNOSELECTION) {
              //MessageBoxFree(nppData._nppHandle,smprintf("p1:%u p2:%u sln:%u",p1,p2,sln),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
              SENDMSGTOCED(currentEdit, SCI_SETTARGETSTART, p1, 0);
              SENDMSGTOCED(currentEdit, SCI_SETTARGETEND, p2-1,0);
              SENDMSGTOCED(currentEdit, SCI_REPLACETARGET, sln, tx);
              sln=0;
            } */else {
              SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, "");
              SENDMSGTOCED(currentEdit, SCI_ADDTEXT, sln, tx);
            }
          }
          //if (sln) {
            if (curpos<anchor) SENDMSGTOCED(currentEdit, SCI_SETSEL, p2, p1);
            else               SENDMSGTOCED(currentEdit, SCI_SETSEL, p1, p2);
          //}
        }
        SENDMSGTOCED(currentEdit, SCI_ENDUNDOACTION, 0, 0);
      }
    } else MessageBoxFree(nppData._nppHandle,smprintf("Out of memory%s",HighPerformance?". Try turning off High Performance.":""),PLUGIN_NAME, MB_OK|MB_ICONSTOP);
    SENDMSGTOCED(currentEdit, SCI_SETCURSOR, SC_CURSORNORMAL, 0);
  } while(0);
  if (tx) freesafe(tx,"convertall-end");
  if (lps) freesafe(lps,"convertall-end");
  if (lpe) freesafe(lpe,"convertall-end");
  if (clip) freesafe(clip,"convertall-end");
#undef origexpand
#undef CAFLAG_HASBINARY
#undef CAFLAG_BLOCKMODE
#undef CAFLAG_LESS
#undef MAXLINES
}
// SECTION: END
// SECTION: Beginning of Notepad++ interface

// SUBSECTION: Single line menu functions
PFUNCPLUGINCMD pfdummy() { } // does absolutely nothing at all
PFUNCPLUGINCMD pfconvert1q2q()         { convertall('c',0,"'","\"",NULL,NULL);}
PFUNCPLUGINCMD pfconvert2q1q()         { convertall('c',0,"\"","'",NULL,NULL); }
PFUNCPLUGINCMD pfconvertqtswap()       { convertall('C',0,"\"'","'\"",NULL,NULL);}
PFUNCPLUGINCMD pfconvertqtdrop()       { convertall('C',0,"\"'","",NULL,NULL);}
PFUNCPLUGINCMD pfconvertescapesq()     { convertall('S',0,"\"","\\\"",NULL,NULL);}
PFUNCPLUGINCMD pfconvertescape1qsq()   { convertall('S',0,"'","\\\"",NULL,NULL);}
PFUNCPLUGINCMD pfconvertescape1qs1q()  { convertall('S',0,"'","\\'",NULL,NULL);}
PFUNCPLUGINCMD pfconvertescapeboth()   { convertall('S',0,"'","\\'","\"","\\\"");}
PFUNCPLUGINCMD pfconvertunescapesq()   { convertall('S',0,"\\\"","\"",NULL,NULL); }
PFUNCPLUGINCMD pfconvertunescapesq1q() { convertall('S',0,"\\\"","'",NULL,NULL); }
PFUNCPLUGINCMD pfconvertunescapes1q1q(){ convertall('S',0,"\\'","'",NULL,NULL); }
PFUNCPLUGINCMD pfconvertunescapeboth() { convertall('S',0,"\\'","'","\\\"","\""); }
PFUNCPLUGINCMD pfconvertescape2q22q()  { convertall('S',0,"\"","\"\"",NULL,NULL); }
PFUNCPLUGINCMD pfconvertescape1q22q()  { convertall('S',0,"'","\"\"",NULL,NULL); }
PFUNCPLUGINCMD pfconvertunescape22q2q(){ convertall('S',0,"\"\"","\"",NULL,NULL); }
PFUNCPLUGINCMD pfconvertunescape22q1q(){ convertall('S',0,"\"\"","'",NULL,NULL); }
PFUNCPLUGINCMD pfconvertpropercase()   { convertall('P',0,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfconvertsentencecase() { convertall('s',0,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfconvertinvertcase()   { convertall('p',0,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfencodeHTML()          { convertall('Z',CAFLAG_DENYBLOCK,"&<>\"",",&&amp;,<&lt;,>&gt;,\"&quot;,",NULL,NULL); }
PFUNCPLUGINCMD pfencodeURIcomponent()  { convertall('H',CAFLAG_DENYBLOCK,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfrot13()               { convertall('C',0,"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz","NOPQRSTUVWXYZABCDEFGHIJKLMnopqrstuvwxyzabcdefghijklm",NULL,NULL); }
PFUNCPLUGINCMD pfunwraptext()          { convertall('W',CAFLAG_DENYBLOCK,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfrewraptext()          { convertall('w',CAFLAG_DENYBLOCK|CAFLAG_GETCLIPBOARDTEXT,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pflineupcomma()         { convertall(',',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_REQUIREMULTILINES,",",NULL,NULL,NULL);}
PFUNCPLUGINCMD pflineupequals()        { convertall(',',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_REQUIREMULTILINES,"=",NULL,NULL,NULL);}
PFUNCPLUGINCMD pflineupclipboard()     { convertall(',',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_REQUIREMULTILINES|CAFLAG_REQUIRECLIPBOARDTEXT1CHAR,NULL,NULL,NULL,NULL);}
PFUNCPLUGINCMD pfinsertclipboardcolumn(){convertall('|',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_REQUIREMULTILINES|CAFLAG_REQUIRECLIPBOARDTEXT,NULL,NULL,NULL,NULL);}
PFUNCPLUGINCMD pfemptyundobuffer(){ INT_CURRENTEDIT; GET_CURRENTEDIT; SENDMSGTOCED(currentEdit, SCI_EMPTYUNDOBUFFER, 0, 0);}
PFUNCPLUGINCMD pfsubmitHTML()     { convertall('3',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY|CAFLAG_GETALLWHENNOSELECTION,"W3C-HTMLValidator.htm\0Extra unused space for l33t haxors to specify their own filenames\0","temp.htm\0Extra unused space for l33t haxors to specify their own filenames\0",NULL,NULL); }
PFUNCPLUGINCMD pfsubmitCSS()      { convertall('3',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY|CAFLAG_GETALLWHENNOSELECTION,"W3C-CSSValidator.htm\0Extra unused space for l33t haxors to specify their own filenames\0" ,"temp.htm\0Extra unused space for l33t haxors to specify their own filenames\0",NULL,NULL); }
PFUNCPLUGINCMD pfdecimal2binary() { convertall('n',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"d","b",NULL,NULL); }
PFUNCPLUGINCMD pfdecimal2octal()  { convertall('n',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"d","o",NULL,NULL); }
PFUNCPLUGINCMD pfdecimal2hex()    { convertall('n',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"d","h",NULL,NULL); }
PFUNCPLUGINCMD pfhex2decimal()    { convertall('n',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"h","d",NULL,NULL); }
PFUNCPLUGINCMD pfoctal2decimal()  { convertall('n',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"o","d",NULL,NULL); }
PFUNCPLUGINCMD pfbinary2decimal() { convertall('n',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"b","d",NULL,NULL); }
PFUNCPLUGINCMD pfcnum2decimal()   { convertall('n',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"c","d",NULL,NULL); }
PFUNCPLUGINCMD pfstripHTMLtags()  { convertall('h',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"t",NULL,NULL,NULL); }
PFUNCPLUGINCMD pfstripHTMLnotabs(){ convertall('h',CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,"n",NULL,NULL,NULL); }
PFUNCPLUGINCMD pfkillwhitenonqtvb(){convertall(' ',0,"v",NULL,NULL,NULL); }
PFUNCPLUGINCMD pfkillwhitenonqtc(){ convertall(' ',0,"c",NULL,NULL,NULL); }
PFUNCPLUGINCMD pffindqtstringvb() { convertall('"',CAFLAG_DENYBLOCK|CAFLAG_GETCLIPBOARDTEXT,",","v",NULL,NULL); }
PFUNCPLUGINCMD pffindqtstringc()  { convertall('"',CAFLAG_DENYBLOCK|CAFLAG_GETCLIPBOARDTEXT,",","c",NULL,NULL); }
PFUNCPLUGINCMD pffilldownover()   { convertall('f',CAFLAG_REQUIREBLOCK,"o",NULL,NULL,NULL); }
PFUNCPLUGINCMD pffilldownins()    { convertall('f',CAFLAG_REQUIREBLOCK,"i",NULL,NULL,NULL); }
PFUNCPLUGINCMD pfaddup()          { convertall('+',CAFLAG_DENYBINARY ,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfzapspace()       { convertall('z',0                 ," ","",NULL,NULL); }
PFUNCPLUGINCMD pfzapnonprint()    { convertall('z',0                 ,"#","n",NULL,NULL); }
PFUNCPLUGINCMD pfspace2tabs()     { convertall('\t',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfspace2tabs8()     { convertall('\t',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES,"8",NULL,NULL,NULL); } //http://forum.pspad.com/read.php?f=2&i=2799&t=2799
PFUNCPLUGINCMD pfindentlinessurround()    { convertall('I',CAFLAG_DENYBLOCK,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pftrimtrailingspace(){ convertall('t',0,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfdeleteblanklines(){ convertall('d',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_REQUIREMULTILINES,"1",NULL,NULL,NULL); }
PFUNCPLUGINCMD pfdeleteblanklines2(){ convertall('d',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_REQUIREMULTILINES,"2",NULL,NULL,NULL); }
//PFUNCPLUGINCMD pfdeleteevery2lines(){ convertall('N',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_REQUIREMULTILINES,"2",NULL,NULL,NULL); }
PFUNCPLUGINCMD pfreindentcode() { convertall('A',CAFLAG_EXTENDTOLINES|CAFLAG_GETALLWHENNOSELECTION|CAFLAG_DENYBLOCK|CAFLAG_REQUIREMULTILINES,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfascii2EBCDIC()   { convertall('e',0,"e",NULL,NULL,NULL); }
PFUNCPLUGINCMD pfEBCDIC2ascii()   { convertall('e',0,"a",NULL,NULL,NULL); }
PFUNCPLUGINCMD pftohex16()        { convertall('X',0,"16",NULL,NULL,NULL); }
PFUNCPLUGINCMD pftohex32()        { convertall('X',0,"32",NULL,NULL,NULL); }
PFUNCPLUGINCMD pftohex64()        { convertall('X',0,"64",NULL,NULL,NULL); }
PFUNCPLUGINCMD pftohex128()       { convertall('X',0,"128",NULL,NULL,NULL); }
PFUNCPLUGINCMD pffromhex()        { convertall('x',CAFLAG_DENYBINARY,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfwordcount()      { convertall('-',0,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfinsertlinenumbers(){ convertall('l',0,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfdeletefirstword(){ convertall('L',0,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfextendblockspaces(){ convertall('E',CAFLAG_REQUIREBLOCK|CAFLAG_REQUIREMULTILINES,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfcleanemailquoting(){ convertall('Q',0,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfuudecode()       { convertall('u',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_DENYBINARY,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfbase64decode()   { convertall('U',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES|CAFLAG_DENYBINARY,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfhexbyterunstolittlendian2()   { convertall('4',CAFLAG_DENYBINARY,"2",NULL,NULL,NULL); }
PFUNCPLUGINCMD pfhexbyterunstolittlendian4()   { convertall('4',CAFLAG_DENYBINARY,"4",NULL,NULL,NULL); }
PFUNCPLUGINCMD pflittlendiantohexbyteruns()   { convertall('5',CAFLAG_DENYBINARY,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfconverteol()     { convertall('O',CAFLAG_DENYBLOCK,NULL,NULL,NULL,NULL); }
PFUNCPLUGINCMD pfsplitlinesatchvb() { convertall('~',CAFLAG_DENYBLOCK|CAFLAG_GETCLIPBOARDTEXT,",","v",NULL,NULL); }
PFUNCPLUGINCMD pfsplitlinesatchc()  { convertall('~',CAFLAG_DENYBLOCK|CAFLAG_GETCLIPBOARDTEXT,",","c",NULL,NULL); }
// SUBSECTION: END
// SUBSECTION: Multiline menu functions
PFUNCPLUGINCMD pfindentlines()    {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  if (SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, 0)>1 /*&& !SENDMSGTOCED(currentEdit, SCI_SELECTIONISRECTANGLE, 0, 0)*/) {
    convertall('i',CAFLAG_EXTENDTOLINES|CAFLAG_DENYBLOCK/*|CAFLAG_GETCURLINEWHENNOSELECTION*/,NULL,NULL,NULL,NULL);
  } else SENDMSGTOCED(currentEdit, SCI_TAB,0,0);
}

PFUNCPLUGINCMD pffindmatchchar() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos,mtpos; if (bracematch(currentEdit,&curpos,&mtpos,0)) SENDMSGTOCED(currentEdit, SCI_GOTOPOS, (mtpos), 0);
}

PFUNCPLUGINCMD pfmarkmatchchar() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos,mtpos; if (bracematch(currentEdit,&curpos,&mtpos,FLAG_INCLUDEBRACKETS)) SENDMSGTOCED(currentEdit, SCI_SETSEL, curpos, mtpos);
}
PFUNCPLUGINCMD pfdeletebracepair() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos,mtpos; if (bracematch(currentEdit,&curpos,&mtpos,FLAG_INCLUDEBRACKETS)) {
    unsigned p1,p2;
    if (curpos<mtpos) {p1=curpos;p2=mtpos;} else {p1=mtpos;p2=curpos;}
    //MessageBoxFree(nppData._nppHandle,smprintf("Deleting p1:%u p2:%d",p1,p2),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    SENDMSGTOCED(currentEdit, SCI_BEGINUNDOACTION, 0, 0);
    SENDMSGTOCED(currentEdit, SCI_SETTARGETSTART, p2-1, 0);
    SENDMSGTOCED(currentEdit, SCI_SETTARGETEND, p2,0);
    SENDMSGTOCED(currentEdit, SCI_REPLACETARGET, 0,0);
    SENDMSGTOCED(currentEdit, SCI_SETTARGETSTART, p1, 0);
    SENDMSGTOCED(currentEdit, SCI_SETTARGETEND, p1+1,0);
    SENDMSGTOCED(currentEdit, SCI_REPLACETARGET, 0,0);
    SENDMSGTOCED(currentEdit, SCI_ENDUNDOACTION, 0, 0);
  }
}
#if 0
PFUNCPLUGINCMD pfdeletebracepaircontents() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos,mtpos; if (bracematch(currentEdit,&curpos,&mtpos,FLAG_INCLUDEBRACKETS)) {
    unsigned p1,p2;
    if (curpos<mtpos) {p1=curpos;p2=mtpos;} else {p1=mtpos;p2=curpos;}
    //MessageBoxFree(nppData._nppHandle,smprintf("Deleting p1:%u p2:%d",p1,p2),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    SENDMSGTOCED(currentEdit, SCI_SETTARGETSTART, p1, 0);
    SENDMSGTOCED(currentEdit, SCI_SETTARGETEND, p2,0);
    SENDMSGTOCED(currentEdit, SCI_REPLACETARGET, 0,0);
  }
}
#endif

PFUNCPLUGINCMD pfmarkmatchline() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos,mtpos; if (bracematch(currentEdit,&curpos,&mtpos,FLAG_INCLUDEBRACKETS | CAFLAG_EXTENDTOLINES)) SENDMSGTOCED(currentEdit, SCI_SETSEL, curpos, mtpos);
}

PFUNCPLUGINCMD pfshowmatchline() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos,mtpos; if (bracematch(currentEdit,&curpos,&mtpos,FLAG_IGNOREBEGINBRACE)) {
    int lineno=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,mtpos,0);
    int linelen=SENDMSGTOCED(currentEdit, SCI_GETLINE,lineno,0);
    char *buf;
    if (buf=(char *)mallocsafe(linelen,"pfshowmatchline")) {
      SENDMSGTOCED(currentEdit, SCI_GETLINE,lineno,buf);
      SetWindowText(nppData._nppHandle,buf);
      freesafe(buf,"pfshowmatchline");
    }
  }
}
// converts text lines to code lines, escaping quote and slash symbols where necessary.
// Sample Javascript result: Response.write("text \" lines");
// new feature: convert code lines back to text lines, unescaping where necessary. Only will be implemented if there is a need.
PFUNCPLUGINCMD pfprepostpendlines() { // if necessary, the inverse can be implemented as '-'
  enum LangType docType; SendMessage(nppData._nppHandle, WM_GETCURRENTLANGTYPE, 0, (LPARAM)&docType);
  switch(docType) {
  case L_PHP:
    convertall('&',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES,"echo \"","\\n\";","c\"\\",";\\\\\\;\"\\\";"); // start, end, search, replace
    break;
  case L_C:
  case L_CPP:
    convertall('&',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES,"\"","\\n\"","c\"\\",";\\\\\\;\"\\\";"); // start, end, search, replace
    break;
  case L_ASP: // used for ASP-VBScript
    convertall('&',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES,"Response.Write(\"","\"&vbCrLf)","s\"","\"\""); // start, end, search, replace
    break;
  case L_JS: // used for ASP-JScript
    convertall('&',CAFLAG_DENYBLOCK|CAFLAG_EXTENDTOLINES,"Response.Write(\"","\\n\");","c\"\\",";\\\\\\;\"\\\";"); // start, end, search, replace
    break;
  default:
    MessageBox(nppData._nppHandle,
      "At this time only PHP, ASP (VB & JSP), C, and C++ are supported.\r\n"
      "At this time ASP is ASP-VBScript and Javascript is ASP-JScript\r\n"
      "To get Javascript to display correctly, select Python then select Javascript\r\n"
      "Please select a supported language then select some text lines to convert.",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    break;
  }
}

PFUNCPLUGINCMD pfinsertasciichart() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned i=0,ie=255;
  unsigned chartsz=12000; // arm tools do not require us to get this number right
  if (SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, 0)-1==1) {
    unsigned char tx[2];
    SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, tx);
    i=ie=tx[0];
    chartsz=100;
  }
  char *chart=NULL;
  unsigned chartlen;
  for(; i<=ie; i++) {
    char str[6],lit[3],bin[9];
    strcpy(lit,"  ");
    str[0]=i; str[1]='\0';
    itoa(i,bin,2);
    switch(i) {
    case 0: strcpy(str,"%"); strcpy(lit,"\\0"); break; // this % will be replaced with a \0 later!
    case 7: strcpy(lit,"\\a"); break;
    case 8: strcpy(lit,"\\b"); break;
    case 9: strcpy(str,"TAB"); strcpy(lit,"\\t"); break;
    case 10: strcpy(str,"LF"); strcpy(lit,"\\n"); break;
    case 11: strcpy(lit,"\\v"); break;
    case 12: strcpy(lit,"\\f"); break;
    case 13: strcpy(str,"CR"); strcpy(lit,"\\r"); break;
    case 32: strcpy(str,"SPACE"); break;
    case '\"': strcpy(lit,"\\\""); break;
    case '\\': strcpy(lit,"\\\\"); break;
    case '\'': strcpy(lit,"\\'"); break;
    }
    sarmprintf(&chart,&chartsz,&chartlen,"%3u \\x%2.2X (0x%2.2X) \\%-3o (0%-3o) %8s %s %c%c %s\n",i,i,i,i,i,bin,lit,(i<32?'^':' '),(i<32?(i+64):' '),str);
    if (!chart) break;
  }
  if (chart) {
    char *p; if (p=(char *)memchr(chart,'%',chartlen)) *p='\0'; // sarmprintf doesn't handle \0 properly so we'll redact it
    SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, "");
    SENDMSGTOCED(currentEdit, SCI_ADDTEXT, chartlen, chart);
    SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, "");
    freesafe(chart,"pfinsertasciichart");
  }
}

PFUNCPLUGINCMD pfinsertruler() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned eoltype=SENDMSGTOCED(currentEdit, SCI_GETEOLMODE, 0, 0);
  if (eoltype>=NELEM(eoltypes)) eoltype=NELEM(eoltypes)-1;
  char *st1=NULL; unsigned sz1=103,sl1;
  char *st2=NULL; unsigned sz2=103,sl2;
  int i; for(i=0; i<10; i++) {
    sarmprintf(&st1,&sz1,&sl1,"---%3d---|",i*10);
    strcpyarmsafe(&st2,&sz2,&sl2,"123456789|","pfinsertruler");
    if (!st1 || !st2) goto fail;
  }
  sarmprintf(&st1,&sz1,&sl1,"%s%s%s",eoltypes[eoltype],st2,eoltypes[eoltype]);
  if (st1) SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, st1);
fail:
  if (st1) freesafe(st1,"pfinsertruler");
  if (st2) freesafe(st2,"pfinsertruler");
}

#if 0 /* This was fixed in N++ 3.3 */
PFUNCPLUGINCMD pfextendblock() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  int p1=SENDMSGTOCED(currentEdit, SCI_GETSELECTIONSTART, 0, 0);
  int p2=SENDMSGTOCED(currentEdit, SCI_GETSELECTIONEND, 0, 0);
  int p1l=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,p1,0);
  do {
    if (SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,p2,0)!=p1l) {
      MessageBox(nppData._nppHandle,"Only select text on a single line",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
      break;
    }
    p2-=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE,p1l,0);
    //MessageBoxFree(nppData._nppHandle,smprintf("p1:%d p1l:%d p2:%d",p1,p1l,p2),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);

    int i,boli; for(i=SENDMSGTOCED(currentEdit, SCI_GETLINECOUNT,0,0)-1; i>p1l; i--) {
      boli=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE,i,0);
      //MessageBoxFree(nppData._nppHandle,smprintf("line-i:%d boli:%d eoli:%d",i,boli,SENDMSGTOCED(currentEdit, SCI_GETLINEENDPOSITION,i,0)),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
      if (SENDMSGTOCED(currentEdit, SCI_GETLINEENDPOSITION,i,0)-boli>=p2) {
        SENDMSGTOCED(currentEdit, SCI_SETANCHOR,p1, 0);
        SENDMSGTOCED(currentEdit, SCI_SETSELECTIONMODE,SC_SEL_RECTANGLE, 0);
        SENDMSGTOCED(currentEdit, SCI_SETCURRENTPOS,p2+boli, 0);
        //MessageBoxFree(nppData._nppHandle,smprintf("p1:%d boli:%d p2+boli:%d",p1,boli,p2+boli),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
        break;
      }
    }
  } while(0);
}
#endif

#if 0
PFUNCPLUGINCMD pfshowstyle() {
	char buf[4];
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  if (curpos>0) {
    struct TextRange tr; tr.chrg.cpMin=curpos-1; tr.chrg.cpMax=curpos; tr.lpstrText=buf;
    SENDMSGTOCED(currentEdit, SCI_GETSTYLEDTEXT, 0, &tr);
    MessageBoxFree(nppData._nppHandle,smprintf("char:%c (%u,%2.2X) style:%u (%2.2X)",buf[0]?buf[0]:'.',buf[0],buf[0],buf[1],buf[1]),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  }
}
#endif

#if 0
PFUNCPLUGINCMD pffindinotherview() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  SENDMSGTOCED(!currentEdit, SCI_GOTOPOS, curpos, 0);
}
#endif

PFUNCPLUGINCMD pfswitchseltorectangular() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  unsigned anchor=SENDMSGTOCED(currentEdit, SCI_GETANCHOR, 0, 0);
  if (anchor!=curpos && !SENDMSGTOCED(currentEdit, SCI_SELECTIONISRECTANGLE, 0, 0)) {
    SENDMSGTOCED(currentEdit, SCI_SETANCHOR,anchor, 0);
    SENDMSGTOCED(currentEdit, SCI_SETSELECTIONMODE,SC_SEL_RECTANGLE, 0);
    SENDMSGTOCED(currentEdit, SCI_SETCURRENTPOS,curpos, 0);
  }
}

PFUNCPLUGINCMD pfhelp() {
  MessageBox(nppData._nppHandle,
    "A Demo File NPP"PLUGIN_NAME"Demo.TXT was included with your distribution.\r\n"
    "Load this file and find the feature you need help on. A description and\r\n"
    "sample text is provided so you can see how each tool works. Read the entire\r\n"
    "demo file to get an idea of what all the tools do.",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
}
// ensure this version number matches those in the Dev-C++ version resource
PFUNCPLUGINCMD pfabout() {
  MessageBox(nppData._nppHandle,
    PLUGIN_NAME " v0.2 ("
#if defined(__BORLANDC__)
  "Borland C++ 5.51"
#elif defined(_MSC_VER)
  "Microsoft Visual C++ Toolkit 2003"
#elif defined(__DMC__)
  "Digital Mars 845"
#elif defined(__WATCOMC__)
  "OpenWatcom32 1.3" // The current settings produce a Win98 incompatible DLL.
#elif defined(__MINGW32__)
  "MinGW/GCC 3.44"
#endif
    "), a GNU GPL Plugin for Notepad++ by Chris Severance,\r\n"// and other authors.\r\n"
    "performs a variety of common conversions on selected text.\r\n"
#if NPPDEBUG
    "\r\nThis DEBUG edition functions exactly like the non debug but the DLL is larger and you may\r\n"
    "see error boxes pop up if any internal structures are handled incorrectly. These are the programmer's\r\n"
    "fault, not yours, and they will be fixed if you report them. I prefer that feature requests be posted\r\n"
    "to the N++ plugins forum so everyone can add ideas. Bugs should be reported to me so I can look better!\r\n"
    "severach@users.sourceforge.net"
#else
    "\r\n" PLUGIN_NAME " is distributed with Notepad++ but is not a part\r\n"
    "of Notepad++. Please do not post " PLUGIN_NAME " bugs in the Notepad++\r\n"
    "bug tracker unless you believe the cause is Notepad++ and not "PLUGIN_NAME "."
#endif
    ,PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
}
PFUNCPLUGINCMD pfhNotepadweb() { LaunchURL("http://notepad-plus.sourceforge.net/"); }

// From PSPAD
BOOL MarkWordFindCaseSensitive=FALSE;
void MarkWordFind(int dir) {
  struct TextToFind tr;
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  unsigned p1=SENDMSGTOCED(currentEdit, SCI_GETSELECTIONSTART, 0, 0);
  unsigned p2=SENDMSGTOCED(currentEdit, SCI_GETSELECTIONEND, 0, 0);
  unsigned sellen; if ((sellen=SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, 0))<=1) {
    unsigned word1=SENDMSGTOCED(currentEdit, SCI_WORDSTARTPOSITION, curpos, true);
    unsigned word2=SENDMSGTOCED(currentEdit, SCI_WORDENDPOSITION, curpos, true);
    if (word1 != word2) SENDMSGTOCED(currentEdit, SCI_SETSEL, dir<0?word2:word1,dir<0?word1:word2);
    //MessageBoxFree(nppData._nppHandle,smprintf("sellen:%u word1:%u word2:%u",sellen,word1,word2),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  } else if ((p1>0 || dir>0) && (tr.lpstrText=(char *)mallocsafe(sellen,"MarkWordFind"))) {
    SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, tr.lpstrText);
    tr.chrg.cpMin=dir<0?p1:p2; tr.chrg.cpMax=dir<0?0:SENDMSGTOCED(currentEdit, SCI_GETLENGTH, 0, 0);
    //MessageBoxFree(nppData._nppHandle,smprintf("cpMin:%u cpMax:%u text:%s",tr.chrg.cpMin,tr.chrg.cpMax,tr.lpstrText),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    int findpos; if ((findpos=SENDMSGTOCED(currentEdit, SCI_FINDTEXT, MarkWordFindCaseSensitive?SCFIND_MATCHCASE:0, &tr))!=-1) {
      SENDMSGTOCED(currentEdit, SCI_GOTOPOS, dir<0?(findpos+sellen-1):findpos, 0);
      SENDMSGTOCED(currentEdit, SCI_SETSEL, dir<0?(findpos+sellen-1):findpos,dir<0?findpos:(findpos+sellen-1));
    } //else MessageBeep(MB_ICONHAND);
    freesafe(tr.lpstrText,"MarkWordFind");
  }
}
PFUNCPLUGINCMD pfMarkWordFindReverse() { MarkWordFind(-1); }
PFUNCPLUGINCMD pfMarkWordFindForward() { MarkWordFind(+1); }

// SUBSECTION: END
// SECTION: END

unsigned findmenuitem(PFUNCPLUGINCMD_MSC *_pFunc); // a couple of well placed forward references kills a ton of other ones
extern struct FuncItem funcItem[];

// SECTION: Large multi-routine menu functions
// SUBSECTION: SHOWHIDE
// http://sourceforge.net/forum/forum.php?thread_id=1112903&forum_id=195402
char *vizsequence=NULL;
unsigned vizsz=0,vizlen=0,vizpos=0; // for single stepping
BOOL VizCaseSensitive=FALSE;
BOOL VizWholeWords=FALSE;
BOOL VizRegex=FALSE;
#define VIZMENUFLAGS ((VizCaseSensitive?SCFIND_MATCHCASE:0)|(VizWholeWords?SCFIND_WHOLEWORD:0)|(VizRegex?SCFIND_REGEXP:0))
/* Commands: \r\n, \r, or \n terminate each command, all commands sequences start with an implicit SHOW ALL LINES
   +:r:searchtext show lines with found reg-exp text
   +:!:searchtext show lines without found text
   -:w:searchtext hide lines with found text whole words only
   -:!^:searchtext hide lines without found text case insensitive
   +[L1-L2]\r\n      Show lines between L1-L2
   -[L1-L2]\r\n      Hide lines between L1-L2
   ![L1-L2]\r\n      Invert lines from L1-L2
   +* Show all lines
   -* Hide all lines
   !* Invert all lines
*/
void hidelinerange(INT_CURRENTEDIT,int curline,int L1,int L2) {
	if (L1==0) L1++; // Scintilla does not allow us to hide the first line AKA Line#0.
  if (L1==curline) L1++;
  if (L2==curline) L2--;
  if (L2>=L1) {
    //MessageBoxFree(nppData._nppHandle,smprintf("Hiding from %d-%d",L1,L2),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    if (L1<curline && curline<L2) {
      SENDMSGTOCED(currentEdit, SCI_HIDELINES,L1,curline-1); // don't hide the current line
      SENDMSGTOCED(currentEdit, SCI_HIDELINES,curline+1,L2);
    } else {
      SENDMSGTOCED(currentEdit, SCI_HIDELINES,L1,L2);
    }
  }
}

// This robot routine expects the fake line numbers 1..SCI_GETLINECOUNT, not 0..SCI_GETLINECOUNT-1
// call with L1=0,L2=0 to do all lines
// reveal: '-'=hide; '+'=show, '!'=invert
void robotvizinvert(INT_CURRENTEDIT,char reveal,int curline,int L1,int L2) {
  if (L1>L2) {int temp=L1; L1=L2; L2=temp;}
	int L2x=SENDMSGTOCED(currentEdit, SCI_GETLINECOUNT,0,0);
	if (L1<1 || L1>L2x || L2<1 || L2>L2x) {L1=1; L2=L2x;}
	else {
    L2--; L1++;
    if (L1>=L2) {L1=1; L2=L2x;}
	}
  //MessageBoxFree(nppData._nppHandle,smprintf("%c from %d-%d",reveal,L1,L2),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
	if (reveal=='!') {
    for(; L2>=L1; L2--) if (L2!=curline) {
      if (SENDMSGTOCED(currentEdit, SCI_GETLINEVISIBLE,L2-1,0)) SENDMSGTOCED(currentEdit, SCI_HIDELINES,L2-1,L2-1);
      else SENDMSGTOCED(currentEdit, SCI_SHOWLINES,L2-1,L2-1);
    }
	} else {
		if (reveal=='+') SENDMSGTOCED(currentEdit, SCI_SHOWLINES,L1-1,L2-1);
    else hidelinerange(currentEdit,curline-1,L1-1,L2-1);
	}
}

void vizinvertselectedalllines(char reveal) {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  int curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  int curline=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,curpos,0);
  int L2=curline+1;
  int L1=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,SENDMSGTOCED(currentEdit, SCI_GETANCHOR,0,0),0)+1;
  if (L1>L2) {int temp=L1; L1=L2; L2=temp;}
  if (L1>=L2) {
  	L2=L1=0;
    if ((reveal=='+' || reveal=='-') && vizsequence) {freesafe(vizsequence,"vizinvertselectedalllines"); vizsequence=NULL; vizsz=0; }
    if (reveal!='+') sarmprintf(&vizsequence,&vizsz,&vizlen,"%c*\r\n",reveal);
  } else {
  	sarmprintf(&vizsequence,&vizsz,&vizlen,"%c[%d-%d]\r\n",reveal,L1,L2);
  }
  robotvizinvert(currentEdit,reveal,curline+1,L1,L2);
  SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos,0);
}
PFUNCPLUGINCMD pfvizhideselectedalllines() { vizinvertselectedalllines('-'); }
PFUNCPLUGINCMD pfvizshowselectedalllines() { vizinvertselectedalllines('+'); }
PFUNCPLUGINCMD pfvizinvertselectedalllines(){vizinvertselectedalllines('!'); }

// new feature: show container lines with text at column 0
// new feature: gutter width, how many lines around the found text get displayed?
BOOL robothidecliplines(INT_CURRENTEDIT,char *str,char reveal,BOOL complementary,unsigned searchflags) {
#define FLAG_CONTINUE 1
#define FLAG_FIRST 2
  BOOL rv=TRUE;

  if (str) {
    //MessageBoxFree(nppData._nppHandle,smprintf("%c%s%s Searching for %s",reveal,complementary?" complementary":"",casesensitive?" casesensitive":"",str),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    struct TextToFind tr;
    tr.chrg.cpMin=0; tr.chrg.cpMax=SENDMSGTOCED(currentEdit, SCI_GETLENGTH,0,0); tr.lpstrText=str;
    int curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
    int curline=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,curpos,0);
    unsigned flags=FLAG_FIRST; do {
      int L1=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,tr.chrg.cpMin,0);
      if (SENDMSGTOCED(currentEdit, SCI_FINDTEXT,searchflags,&tr)>=0) flags |= FLAG_CONTINUE; else flags &= ~FLAG_CONTINUE;
      if ((flags&(FLAG_FIRST|FLAG_CONTINUE)) == FLAG_FIRST /* halt when first search fails */) {MessageBox(nppData._nppHandle,"I'm not going to hide all the lines! Copy something to the Clipboard that can be found!",PLUGIN_NAME, MB_OK||MB_ICONSTOP); rv=FALSE; break;}
      int L2=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,((flags&FLAG_CONTINUE)?tr.chrgText.cpMin:tr.chrg.cpMax),0);
      if (reveal=='+') {
      	if (!complementary) L2=L1;
      	SENDMSGTOCED(currentEdit, SCI_SHOWLINES,L1,L2);
      } else {
      	if (complementary) hidelinerange(currentEdit,curline,L1+1 /* This shouldn't always be +1 but thanks to Scintilla not allowing us to hide line 0, it is! */,L2-((flags&FLAG_CONTINUE)?1:0));
      	else if (L1 != curline) SENDMSGTOCED(currentEdit, SCI_HIDELINES,L1,L1);
      }
      tr.chrg.cpMin=tr.chrgText.cpMax;
      flags &= ~FLAG_FIRST;
    } while(flags&FLAG_CONTINUE);
    SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos,0);
  }
  return(rv);
#undef FLAG_CONTINUE
#undef FLAG_FIRST
}

void vizcliplines(char reveal,BOOL complementary,unsigned searchflags) {
	char *va;

  INT_CURRENTEDIT; GET_CURRENTEDIT;
	if (va=strdupClipboardText(NULL,(SENDMSGTOCED(currentEdit, SCI_GETCODEPAGE, 0, 0)==SC_CP_UTF8?SCDS_UNICODEMODE:0),SENDMSGTOCED(currentEdit, SCI_GETEOLMODE, 0, 0),NULL)) {
		int pr=strcspn(va,"\r\n");
		if (va[pr]!='\0') MessageBox(nppData._nppHandle,"Search text cannot contain end of line characters",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
		else {
			if (robothidecliplines(currentEdit,va,reveal,complementary,searchflags)) {
				sarmprintf(&vizsequence,&vizsz,&vizlen,"%c:%s%s%s%s:%s\r\n",reveal,complementary?"!":"",
				  (searchflags&SCFIND_MATCHCASE)?"":"^",
				  (searchflags&SCFIND_WHOLEWORD)?"w":"",
				  (searchflags&SCFIND_REGEXP)   ?"r":"",
				  va);
			}
		}
		freesafe(va,"vizcliplines");
	}
}
PFUNCPLUGINCMD pfvizhidecliplines()  { vizcliplines('-',FALSE,VIZMENUFLAGS);}
PFUNCPLUGINCMD pfvizhideclipclines() { vizcliplines('-',TRUE ,VIZMENUFLAGS);}
PFUNCPLUGINCMD pfvizshowcliplines()  { vizcliplines('+',FALSE,VIZMENUFLAGS);}
PFUNCPLUGINCMD pfvizshowclipclines() { vizcliplines('+',TRUE ,VIZMENUFLAGS);}

PFUNCPLUGINCMD pfvizinsertsequence(){
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  if (vizsequence) SENDMSGTOCED(currentEdit, SCI_REPLACESEL,0,vizsequence);
  else MessageBox(nppData._nppHandle,"You have not performed any show hide operations since your last Show All-Reset Lines",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
}

PFUNCPLUGINCMD pfvizshowmorelines(){
#if 0
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  int curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  int curline=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,curpos,0);
  int L1=curline-3;
  int L2=curline+3;
  int maxline=SENDMSGTOCED(currentEdit, SCI_GETLINECOUNT,0,0)-1;
  if (L1<0) L1=0;
  if (L2>maxline) L2=maxline;
  //MessageBoxFree(nppData._nppHandle,smprintf("Showing from %d-%d",L1,L2),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  if (L2>L1) {
    SENDMSGTOCED(currentEdit, SCI_SHOWLINES,L1,L2);
    SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos,0);
  }
#else
  MessageBox(nppData._nppHandle,"Hold down Caps-Lock and move up or down against hidden text.\r\nLine by line the hidden text will be exposed.",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
#endif
}

PFUNCPLUGINCMD pfvizsequencenext(){
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  int curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  int curline=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,curpos,0);
  unsigned eol=vizpos+strcspn(vizsequence+vizpos,"\r\n");
	char chtemp=vizsequence[eol];
	vizsequence[eol]='\0'; // allows strdup()
	int L1=0,L2=0; char *p,*end,reveal; BOOL complementary=FALSE;
	unsigned searchflags;
  //MessageBoxFree(nppData._nppHandle,smprintf("Running Command %s",vizsequence+vizpos),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
	switch(reveal=vizsequence[vizpos]) {
	case '\0': break;
  case '+':
  case '-':
  case '!':
    switch(vizsequence[vizpos+1]) {
    case ':':
      p=vizsequence+vizpos+2;
      searchflags=SCFIND_MATCHCASE;
      if (!(end=strchr(p,':'))) {MessageBoxFree(nppData._nppHandle,smprintf("Can't find : terminator for search text, be sure to use :: if there are no flags"),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION); break;}
      for(; p<end; p++) switch(*p) {
      	case 'w': searchflags |= SCFIND_WHOLEWORD; break;
      	case 'r': searchflags |= SCFIND_REGEXP; break;
      	case '^': searchflags &= ~SCFIND_MATCHCASE; break;
      	case '!': complementary=TRUE; break;
      }
      if (strlen(end+1)) robothidecliplines(currentEdit,end+1,reveal,complementary,searchflags);
      break;
    case '[':
      if (p=strdupsafe(vizsequence+vizpos+1,"pfvizsequencenext")) {
        L1=atol(strtok(p,"[-]"));
        L2=atol(strtok(NULL,"[-]"));
        freesafe(p,"pfvizsequencenext");
      }
    case '*': robotvizinvert(currentEdit,reveal,curline+1,L1,L2); break;
    default: MessageBoxFree(nppData._nppHandle,smprintf("Invalid target %c, try :[*",*(vizsequence+vizpos+1)),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION); break;
    }
    break;
  default: MessageBoxFree(nppData._nppHandle,smprintf("Invalid reveal %c, try +-!",reveal),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION); break;
	}
	vizsequence[vizpos=eol]=chtemp;
	while(vizsequence[vizpos]=='\r' || vizsequence[vizpos]=='\n') vizpos++;
  SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos,0);
}

PFUNCPLUGINCMD pfvizsequencerest(){
	if (!vizsequence) MessageBox(nppData._nppHandle,"You have not performed any show hide operations since your last Show All Lines",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
	else while(vizsequence[vizpos]) pfvizsequencenext();
}

PFUNCPLUGINCMD pfvizsequencestart() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  int curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
	vizpos=0; // we can do this even if the sequence is invalid
	robotvizinvert(currentEdit,'+',0,0,0);
  SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos,0);
}

PFUNCPLUGINCMD pfvizsequenceall(){
	pfvizsequencestart();
	pfvizsequencerest();
}

PFUNCPLUGINCMD pfvizselectassequence(){
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  char *rv,*end;
  unsigned st,ln;
  if ((st=SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, NULL))>1 && (rv=(char *)mallocsafe(st,"pfvizselectassequence")) ) {
  	SENDMSGTOCED(currentEdit, SCI_GETSELTEXT, 0, rv);
  	if (ln=strlen(rv)) {
  	  for(end=rv+ln-1; end>=rv; end--) if (*end=='\r' || *end=='\n') *end='\0';  // trim off eol that the user may or may not have included
      if (ln=strlen(rv)) {
        if (vizsequence) freesafe(vizsequence,"pfvizselectassequence");
        vizsequence=rv;
        vizsz=st;
        vizlen=ln;
        strcpyarmsafe(&vizsequence,&vizsz,&vizlen,"\r\n","pfvizselectassequence"); // guarantee a \r\n ending
        pfvizsequencestart();
      }
  	}
  } else MessageBox(nppData._nppHandle,"No text selected",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
}

// which: *=all lines, +:visible only, -:invisible only
BOOL VizClipboardAlwaysCRLF=FALSE;
BOOL VizClipboardReplaceNulls=FALSE;
BOOL VizClipboardCopyAlsoUTF8=FALSE;
BOOL VizClipboardNotUnicode=FALSE;
BOOL VizPasteBinary=FALSE;
BOOL VizPasteToEditorEOL=TRUE;
void copycutdeletevisible(unsigned flags,char which) {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  char *buf=NULL; unsigned bufsz=0,buflen;
  unsigned *lps=NULL,*lpe=NULL; unsigned lpssz=0,lpesz=0,lplen;
  int p1=SENDMSGTOCED(currentEdit, SCI_GETSELECTIONSTART, 0, 0);
  int p2=SENDMSGTOCED(currentEdit, SCI_GETSELECTIONEND, 0, 0);
  if (p1<p2) {
    unsigned eoltype=SENDMSGTOCED(currentEdit, SCI_GETEOLMODE, 0, 0);
    if (eoltype>=NELEM(eoltypes)) eoltype=NELEM(eoltypes)-1;
    if (SENDMSGTOCED(currentEdit, SCI_SELECTIONISRECTANGLE, 0, 0)) flags |= SCDS_COPYRECTANGULAR; else flags &= ~SCDS_COPYRECTANGULAR;
    unsigned blockstart=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,p1, 0);
    unsigned blocklines=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,p2, 0)-blockstart+1;
    unsigned ln; for(lplen=ln=0; ln<blocklines; ln++) {
    	int vis=SENDMSGTOCED(currentEdit, SCI_GETLINEVISIBLE, (blockstart+ln),0);
    	if (which=='*' || vis && which=='+' || !vis && which=='-'){
        int ls; if (INVALID_POSITION==(ls=SENDMSGTOCED(currentEdit, SCI_GETLINESELSTARTPOSITION, (blockstart+ln),0))) continue;
        int le; if (INVALID_POSITION==(le=SENDMSGTOCED(currentEdit, SCI_GETLINESELENDPOSITION, (blockstart+ln),0))) continue;
        if (!lplen || lpe[lplen-1]!=ls) {
          armreallocsafe((char **)&lps,&lpssz,(lplen+1)*sizeof(*lps),ARMSTRATEGY_INCREASE,0,"pfvizcopyvisible"); if (!lps) goto failbreak;
          armreallocsafe((char **)&lpe,&lpesz,(lplen+1)*sizeof(*lps),ARMSTRATEGY_INCREASE,0,"pfvizcopyvisible"); if (!lpe) goto failbreak;
          lps[lplen]=ls;
          lpe[lplen]=le;
          lplen++;
          //MessageBoxFree(nppData._nppHandle,smprintf("[%u] %s line:%u from:%u to:%u",lplen,"Added New",blockstart+ln+1,ls,le),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
        } else {
          lpe[lplen-1]=le;
          //MessageBoxFree(nppData._nppHandle,smprintf("[%u] %s line:%u from:%u to:%u",lplen,"Appended",blockstart+ln+1,ls,le),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
        }
    	}
    }
    if (!lplen) goto failmessage;
    if (flags & SCDS_COPY) {
    	for(buflen=0,ln=0; ln<lplen; ln++) {
        struct TextRange tr;
        tr.chrg.cpMin= lps[ln];
        tr.chrg.cpMax= lpe[ln];
        //MessageBoxFree(nppData._nppHandle,smprintf("[%u] Copy from:%u to:%u",ln,tr.chrg.cpMin,tr.chrg.cpMax),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
        armreallocsafe(&buf,&bufsz,buflen+tr.chrg.cpMax-tr.chrg.cpMin+1,ARMSTRATEGY_INCREASE,0,"pfvizcopyvisible"); if (!buf) goto failbreak;
        tr.lpstrText=buf+buflen; buflen += SENDMSGTOCED(currentEdit, SCI_GETTEXTRANGE,0,&tr);
        if (flags&SCDS_COPYRECTANGULAR) {
          strcpyarmsafe(&buf,&bufsz,&buflen,eoltypes[eoltype],"pfvizcopyvisible"); if (!buf) goto failbreak;
        }
    	}
      if (!buflen) goto failmessage;
      if (!SetClipboardDataSingle(&buf,&bufsz,&buflen,flags|
        (VizClipboardAlwaysCRLF?SCDS_COPYEOLALWAYSCRLF:0)|
        (VizClipboardReplaceNulls?SCDS_COPYCONVERTNULLSTOSPACE:0)|
        (VizClipboardCopyAlsoUTF8?SCDS_COPYALSOUTF8:0)|
        (VizClipboardNotUnicode?SCDS_COPYNOTUNICODE:0)|
        (VizPasteBinary?SCDS_COPYPASTEBINARY:0)|
        (VizPasteToEditorEOL?SCDS_PASTETOEDITOREOL:0)|
        (SENDMSGTOCED(currentEdit, SCI_SELECTIONISRECTANGLE, 0, 0)?SCDS_COPYRECTANGULAR:0)|
        (SENDMSGTOCED(currentEdit, SCI_GETCODEPAGE, 0, 0)==SC_CP_UTF8?SCDS_UNICODEMODE:0)
        ,SENDMSGTOCED(currentEdit, SCI_GETEOLMODE, 0, 0)) ) goto failbreak;
    }
    if (flags & SCDS_DELETE) {
      int curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
      SENDMSGTOCED(currentEdit, SCI_BEGINUNDOACTION, 0, 0);
    	for(ln=lplen; ln>0; ln--) {
    		SENDMSGTOCED(currentEdit, SCI_SETTARGETSTART,lps[ln-1],0);
    		SENDMSGTOCED(currentEdit, SCI_SETTARGETEND,lpe[ln-1],0);
    		SENDMSGTOCED(currentEdit, SCI_REPLACETARGET,0,"");
    		if (curpos>=lps[ln-1]) curpos -= lpe[ln-1]-lps[ln-1];
    	}
      SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos,0);
      SENDMSGTOCED(currentEdit, SCI_ENDUNDOACTION, 0, 0);
    }
  } else {
//failmessage: ;
    //MessageBox(nppData._nppHandle,"No text selected",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  }
failmessage:
failbreak:
  if (buf) freesafe(buf,"pfvizcopyvisible");
  if (lps) freesafe(lps,"pfvizcopyvisible");
  if (lpe) freesafe(lpe,"pfvizcopyvisible");
}

PFUNCPLUGINCMD pfcopyallnoappend()    {copycutdeletevisible(SCDS_COPY,'*');}
PFUNCPLUGINCMD pfcopyallappend()      {copycutdeletevisible(SCDS_COPYAPPEND|SCDS_COPY,'*');}
PFUNCPLUGINCMD pfcutallnoappend()     {copycutdeletevisible(SCDS_COPY|SCDS_DELETE,'*');}
PFUNCPLUGINCMD pfcutallappend()       {copycutdeletevisible(SCDS_COPYAPPEND|SCDS_COPY|SCDS_DELETE,'*');}
//PFUNCPLUGINCMD pfdeleteall() {copycutdeletevisible(SCDS_DELETE,'*');} // Copyall and cutall are there for append and other features. Delete gets nothing from append!
PFUNCPLUGINCMD pfvizdeletevisible()   {copycutdeletevisible(SCDS_DELETE,'+');}
PFUNCPLUGINCMD pfvizdeleteinvisible() {copycutdeletevisible(SCDS_DELETE,'-');}
BOOL VizCutCopyAppend=FALSE;
PFUNCPLUGINCMD pfvizcopyvisible()     {copycutdeletevisible(SCDS_COPY|(VizCutCopyAppend?SCDS_COPYAPPEND:0),'+');}
PFUNCPLUGINCMD pfvizcutvisible()      {copycutdeletevisible(SCDS_COPY|(VizCutCopyAppend?SCDS_COPYAPPEND:0)|SCDS_DELETE,'+');}
PFUNCPLUGINCMD pfvizcopyinvisible()   {copycutdeletevisible(SCDS_COPY|(VizCutCopyAppend?SCDS_COPYAPPEND:0),'-');}
PFUNCPLUGINCMD pfvizcutinvisible()    {copycutdeletevisible(SCDS_COPY|(VizCutCopyAppend?SCDS_COPYAPPEND:0)|SCDS_DELETE,'-');}
BOOL VizPasteRetainsPosition=FALSE;
void VizPaste(unsigned flags) {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned txsz;
  char *tx;
  BOOL isrectangular;

  if (tx=strdupClipboardText(&txsz,flags|
      (VizPasteBinary?SCDS_COPYPASTEBINARY:0)|
      (VizPasteToEditorEOL?SCDS_PASTETOEDITOREOL:0)|
      (SENDMSGTOCED(currentEdit, SCI_GETCODEPAGE, 0, 0)==SC_CP_UTF8?SCDS_UNICODEMODE:0)
      ,SENDMSGTOCED(currentEdit, SCI_GETEOLMODE, 0, 0),&isrectangular)) {
    int curpos; if (VizPasteRetainsPosition) curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
    if (isrectangular) {
      SENDMSGTOCED(currentEdit, SCI_PASTE, 0, 0); // we can't handle binary rectangular text and may never need to
    } else {
      SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, "");
      SENDMSGTOCED(currentEdit, SCI_ADDTEXT, txsz, tx);
    }
    freesafe(tx,"pfVizPaste");
    if (VizPasteRetainsPosition) SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos, 0);
  }
}
PFUNCPLUGINCMD pfVizPasteUTF8()    { VizPaste(SCDS_PASTEANSIUTF8); }
PFUNCPLUGINCMD pfVizPasteUNICODE() { VizPaste(0); }

// SUBSECTION: End
#if ENABLE_PUSHPOP /* SUBSECTION: Poplists */
/* new feature: this needs a CRC-32 hash path instead of the real path */
struct _POPLISTS {
  char path[MAX_PATH];
  unsigned mru;
  unsigned poplist[30];
  unsigned popcount;
  unsigned poptextlen;
} poplists[5];
unsigned poplistno;

void detectprplist() {
  char path[MAX_PATH];

  SendMessage(nppData._nppHandle, WM_GET_FULLCURRENTPATH, 0, (LPARAM)path);
  int found;unsigned i,minmru,maxmru; for(found=0,minmru=(unsigned)-1,maxmru=0,i=0; i<NELEM(poplists); i++) {
    if (!found && minmru>poplists[i].mru) {minmru=poplists[i].mru; poplistno=i;}
    if (maxmru<poplists[i].mru) maxmru=poplists[i].mru;
    if (!strcmp(path,poplists[i].path)) {found=1; poplistno=i;}
  }
  if (!found) {
    memset(poplists+poplistno,0,sizeof(poplists[0]));
    strcpy(poplists[poplistno].path,path);
  }
  poplists[poplistno].mru=maxmru+1;
  //MessageBoxFree(nppData._nppHandle,smprintf(buf,"Detected %s in poplist:%u",path,poplistno),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
}

// new feature: It would be nice to destroy poplists when files are closed but we probably won't be able to do that.
void adjustprplist(INT_CURRENTEDIT,unsigned curpos) {
  unsigned t1=SENDMSGTOCED(currentEdit, SCI_GETLENGTH,0,0);
  if (poplists[poplistno].poptextlen) {
    unsigned i; for(i=0; i<poplists[poplistno].popcount; i++) if (poplists[poplistno].poplist[i]>curpos) poplists[poplistno].poplist[i]+=t1-poplists[poplistno].poptextlen;
  }
  poplists[poplistno].poptextlen=t1;
}

void pushpositionex(INT_CURRENTEDIT){
  detectprplist();
  unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  adjustprplist(currentEdit,curpos);
  if (poplists[poplistno].popcount>=NELEM(poplists[poplistno].poplist)) {
    memmove(poplists[poplistno].poplist,poplists[poplistno].poplist+1,sizeof(poplists[poplistno].poplist)-sizeof(poplists[poplistno].poplist[0]));
    poplists[poplistno].popcount--;
  }
  poplists[poplistno].poplist[poplists[poplistno].popcount]=curpos;
  poplists[poplistno].popcount++;
}

PFUNCPLUGINCMD pfpushposition(){ INT_CURRENTEDIT; GET_CURRENTEDIT; pushpositionex(currentEdit); }

PFUNCPLUGINCMD pfpushjumpposition(){
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  struct TextToFind tr;
  tr.chrg.cpMin=0; tr.chrg.cpMax=SENDMSGTOCED(currentEdit, SCI_GETLENGTH,0,0); if (tr.lpstrText=strdupClipboardText(NULL,0)) {
    int findpos=SENDMSGTOCED(currentEdit, SCI_FINDTEXT,SCFIND_MATCHCASE,&tr);
     if (findpos>=0) {
      pushpositionex(currentEdit);
      SENDMSGTOCED(currentEdit, SCI_GOTOPOS, findpos, 0);
     }
    freesafe(tr.lpstrText,"pfpushjumpposition");
  }
}

PFUNCPLUGINCMD pfpopposition(){
#define linedx 1
  detectprplist();
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
  adjustprplist(currentEdit,curpos);
  if (poplists[poplistno].popcount) {
    poplists[poplistno].popcount--;
    unsigned curline=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,curpos,0);
    unsigned goline=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,(poplists[poplistno].poplist[poplists[poplistno].popcount]),0);
    if (poplists[poplistno].popcount && ((goline>curline)?(goline-curline):(curline-goline))<=linedx) {
      SENDMSGTOCED(currentEdit, SCI_GOTOPOS, (poplists[poplistno].poplist[poplists[poplistno].popcount-1]),0);
    } else {
      SENDMSGTOCED(currentEdit, SCI_GOTOPOS, (poplists[poplistno].poplist[poplists[poplistno].popcount]),0);
      poplists[poplistno].popcount++;
    }
  }
#undef linedx
}
#endif /*SUBSECTION: End */

// SUBSECTION: Multiplex Subclass Menu System
#define PFUNCPLUGINCMX void __cdecl /* Some compilers cannot use the typedef in every case. This define works to alter function declarations with all compilers */
#if !defined(__WATCOMC__) || defined(__cplusplus)
typedef void (__cdecl PFUNCPLUGINCMX_MSC)(unsigned); /* MSC is unable to declare variables without a typedef. Compilers other than Watcom prefer this method also but often work with warnings with the #define. */
#else
#define PFUNCPLUGINCMX_MSC PFUNCPLUGINCMX /* Watcom C does not allow conversion between typedefs and nontypedefs whether or not they are identical. */
#endif
#define PFUNCPLUGINCMY USHORT __cdecl /* Some compilers cannot use the typedef in every case. This define works to alter function declarations with all compilers */
#if !defined(__WATCOMC__) || defined(__cplusplus)
typedef USHORT (__cdecl PFUNCPLUGINCMY_MSC)(unsigned,unsigned); /* MSC is unable to declare variables without a typedef. Compilers other than Watcom prefer this method also but often work with warnings with the #define. */
#else
#define PFUNCPLUGINCMY_MSC PFUNCPLUGINCMY /* Watcom C does not allow conversion between typedefs and nontypedefs whether or not they are identical. */
#endif
extern struct _wIDLOOKUP {
	PFUNCPLUGINCMD_MSC *_pFunc;
	PFUNCPLUGINCMX_MSC *buildfunc;
	PFUNCPLUGINCMY_MSC *usefunc; // returns 0 if fully processed or replacement wParamLo to pass call onto
  USHORT wParamLo,len;
  char *appdata1;
  HMENU   hSubMenu;
  unsigned dummy,dummy2;
} wIDlookup[];
void mpxbuildmenu(char *buf,unsigned idx,WORD ic);
unsigned findmpxmenuitem(PFUNCPLUGINCMX_MSC *_pFunc);

PFUNCPLUGINCMX pxmpxhelpmpxbuild(unsigned idx) {
  //MessageBoxFree(nppData._nppHandle,smprintf("wParamLo:%u wParamHi:%u",wParamLo,wParamHi),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
}
PFUNCPLUGINCMD pfmpxhelprebuild() {pxmpxhelpmpxbuild(findmpxmenuitem(pxmpxhelpmpxbuild));}

PFUNCPLUGINCMY pympxhelpmpxuse(unsigned idx,unsigned ic) {
  //MessageBoxFree(nppData._nppHandle,smprintf("wParamLo:%u wParamHi:%u",wParamLo,wParamHi),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
return(0);
}

#if ENABLE_TIDYDLL
PFUNCPLUGINCMD pfhtmltidy()       { convertall('T',CAFLAG_GETALLWHENNOSELECTION|CAFLAG_DENYBLOCK|CAFLAG_DENYBINARY,NULL,NULL,NULL,NULL);  }

char *tidydll_error=NULL;
HINSTANCE HItidydll=NULL;

extern unsigned miDisableSubclassing;
#define NPPTIDY_INI "TidyCFG.INI"
PFUNCPLUGINCMD pfabouttidy() {
  if (!HItidydll) MessageBox(nppData._nppHandle,tidydll_error?tidydll_error:"Unknown HTMLtidy error",PLUGIN_NAME, MB_OK|MB_ICONSTOP);
  else MessageBoxFree(nppData._nppHandle,smprintf("HTML Tidy Release Date: %s\r\nThanks to PSPAD for %s\r\n%s",TIDYDLLCALL(tidyReleaseDate)(),NPPTIDY_INI,funcItem[miDisableSubclassing]._init2Check?"The Tidy menu is partly disabled.\r\nEnable Subclassing for full functionality":""),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
}

PFUNCPLUGINCMD pfhtmltidyweb() {LaunchURL("http://tidy.sourceforge.net/");}
PFUNCPLUGINCMD pfgethtmltidyweb() {LaunchURL("http://notepad-plus.sourceforge.net/uk/download.php");}

void control_tidy(int);
PFUNCPLUGINCMD pfmpxtidyrebuild();
PFUNCPLUGINCMD pfreloadtidydll() {
  if (!HItidydll) {
  	control_tidy(0);
  	if (!HItidydll) pfabouttidy();
  	pfmpxtidyrebuild();
  }
}

void control_tidy(int stop) { // FALSE to start, TRUE to stop.
  char *path=NULL,*errfunc=NULL;
  if (!stop) {
  	if (tidydll_error) {freesafe(tidydll_error,"control_tidy"); tidydll_error=NULL;}
    path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,"libTidy.dll");
    if ((HItidydll=LoadLibrary("libTidy.dll")) || (HItidydll=LoadLibrary(path))) {
      TIDYDLLINIT
      DeleteMenu(GetMenu(nppData._nppHandle),funcItem[findmenuitem(pfgethtmltidyweb)]._cmdID, MF_BYCOMMAND);
      DeleteMenu(GetMenu(nppData._nppHandle),funcItem[findmenuitem(pfreloadtidydll)]._cmdID, MF_BYCOMMAND);
      EnableMenuItem(GetMenu(nppData._nppHandle), funcItem[findmenuitem(pfhtmltidy)]._cmdID, MF_BYCOMMAND);
      EnableMenuItem(GetMenu(nppData._nppHandle), funcItem[findmenuitem(pfmpxtidyrebuild)]._cmdID, MF_BYCOMMAND);
    } else { // Only disable the menu on startup. The menu may no longer be valid when exiting the DLL
fail: if (errfunc) tidydll_error=smprintf("Exported function %s not found in %s.\r\nThis "PLUGIN_NAME" build requires "TIDY_CALLQT" exports.",errfunc,"libTidy.dll");
      else tidydll_error=smprintf("Unable to find %s in the system path\r\nor %s","libTidy.dll",path);
      EnableMenuItem(GetMenu(nppData._nppHandle), funcItem[findmenuitem(pfhtmltidy)]._cmdID, MF_BYCOMMAND | MF_GRAYED);
      EnableMenuItem(GetMenu(nppData._nppHandle), funcItem[findmenuitem(pfmpxtidyrebuild)]._cmdID, MF_BYCOMMAND | MF_GRAYED);
      stop=TRUE;
    }
    DrawMenuBar(nppData._nppHandle);
  }
  if (path) freesafe(path,"control_tidy");
  if (stop && HItidydll) {
    FreeLibrary(HItidydll);
    HItidydll=NULL;
  }
}

PFUNCPLUGINCMX pxmpxtidympxbuild(unsigned idx) {
  //MessageBoxFree(nppData._nppHandle,smprintf("wParamLo:%u wParamHi:%u",wParamLo,wParamHi),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
  char *buf=NULL; unsigned bufsz=0,buflen=0;
  USHORT *bufs=NULL; unsigned bufssz=0;
  unsigned lim;

  if (HItidydll) {
    FILE *fp; char *path; if (!(path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,NPPTIDY_INI)) || !(fp=fopen(path,"rt"))) goto failbreak;
    freesafe(path,"pxmpxtidympxbuild");
    WORD ic; char *t; char ln[256];for(ic=0,lim=50; lim && fgets(ln,sizeof(ln)-1,fp); )  if (ln[0]=='[' && strlen(t=strtok(ln,"[]"))) {
      lim--;
      armreallocsafe((char **)&bufs,&bufssz,(ic+1)*sizeof(*bufs),ARMSTRATEGY_INCREASE,0,"pxmpxtidympxbuild"); if (!bufs) goto failbreak;
      bufs[ic++]=buflen;
      strcpyarmsafe(&buf,&bufsz,&buflen,t,"pxmpxtidympxbuild"); if (!buf) goto failbreak;
      buflen++; // preserve the \0
    }
    fclose(fp);
    WORD i; for(i=0; i<ic; i++) bufs[i]+=ic*sizeof(*bufs);
    memmovearmtest(&buf,&bufsz,&buflen,buf+ic*sizeof(*bufs),buf,1); if (!buf) goto failbreak;
    memcpy(buf,bufs,ic*sizeof(*bufs));
    //INT_CURRENTEDIT; GET_CURRENTEDIT; SENDMSGTOCED(currentEdit, SCI_ADDTEXT, buflen, buf); //SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, "\r\n");SENDMSGTOCED(currentEdit, SCI_ADDTEXT, ic*sizeof(*bufs), bufs);
    mpxbuildmenu(buf,idx,ic);
    buf=NULL;
failbreak:
    if (buf) freesafe(buf,"pxmpxtidympxbuild");
    if (bufs) freesafe(bufs,"pxmpxtidympxbuild");
  }
}
PFUNCPLUGINCMD pfmpxtidyrebuild() {pxmpxtidympxbuild(findmpxmenuitem(pxmpxtidympxbuild));}

PFUNCPLUGINCMY pympxtidympxuse(unsigned idx,unsigned ic) {
  USHORT rv=0;
  char *path=NULL;

  do {
    char *c=wIDlookup[idx].appdata1; USHORT *cs=(USHORT *)c;
    char *opt=smprintf("[%s]",c+cs[ic]); if (!opt) break;
    unsigned optsz=strlen(opt);
    FILE *fp; if (!(path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,NPPTIDY_INI)) || !(fp=fopen(path,"rt"))) break;
    freesafe(path,"pympxtidympxuse");
    if (!(path=smprintfpath("%s%?\\%s%?\\%s",pluginpath,SUPPORT_PATH,NPPTIDY_CFG)) ) break;
    FILE *fo=NULL;
    char *failed=NULL; unsigned failedsz=0,failedlen;
    int fnd=FALSE; char ln[256]; while(fgets(ln,sizeof(ln)-1,fp)) {
      if (ln[0]!=';') {
        if (!fnd) {
          if (ln[0]=='[' && !memcmp(ln,opt,optsz)) {
            fnd=TRUE;
            if (!(fo=fopen(path,"wt"))) goto failbreak;
          }
        } else {
          if (ln[0]=='[') break;
          fputs(ln,fo);
          if (!isspace(*ln)) {
            char *t=strchr(ln,':');
            if (t) {
              *t='\0';
              TidyOptionId tid=TIDYDLLCALL(tidyOptGetIdForName)(ln);
              //MessageBoxFree(nppData._nppHandle,smprintf("tid:%u=tidyOptGetIdForName(%s)",tid,ln),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
              if (N_TIDY_OPTIONS==tid || TidyUnknownOption==tid) sarmprintf(&failed,&failedsz,&failedlen,"%s\r\n",ln);
              *t=':';
            }
          }
        }
      }
    }
    if (fo) {
      fclose(fo);
      if (failed) {
        MessageBoxFree(nppData._nppHandle,smprintf("Invalid TIDY options:\r\n%s",failed),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
        freesafe(failed,"pympxtidympxuse");
      } else rv=funcItem[findmenuitem(pfhtmltidy)]._cmdID;
    } else MessageBoxFree(nppData._nppHandle,smprintf("Not Found idx:%u ic:%u option:%s",idx,ic,opt),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
    freesafe(opt,"pympxtidympxuse");
  } while(0);
failbreak:
  if (path) freesafe(path,"pympxtidympxuse");
  return(rv);
}
#endif

struct _wIDLOOKUP wIDlookup[]= {
#if ENABLE_TIDYDLL
{pfmpxtidyrebuild,pxmpxtidympxbuild,pympxtidympxuse},
#endif
{pfmpxhelprebuild,pxmpxhelpmpxbuild,pympxhelpmpxuse},
};

unsigned findmpxmenuitem(PFUNCPLUGINCMX_MSC *_pFunc) {
  unsigned itemno; for(itemno=0; itemno<NELEM(wIDlookup); itemno++) if (_pFunc == wIDlookup[itemno].buildfunc) return(itemno);
  return(0);
}

void freeIDLOOKUP(unsigned id) {
  if (wIDlookup[id].wParamLo) {
  	WORD wID; for(wID=0; wID<wIDlookup[id].len; wID++) DeleteMenu(wIDlookup[id].hSubMenu,wIDlookup[id].wParamLo+wID,MF_BYCOMMAND);
	  if (wIDlookup[id].appdata1) {freesafe(wIDlookup[id].appdata1,"freeIDLOOKUP"); wIDlookup[id].appdata1=NULL; /*wIDlookup[id].appdata1sz=0;*/}
	  //if (wIDlookup[id].appdata2) {freesafe(wIDlookup[id].appdata2,"freeIDLOOKUP"); wIDlookup[id].appdata2=NULL; }
	  wIDlookup[id].wParamLo=0;
	  wIDlookup[id].len=0;
  }
}

USHORT minwID=0,nextwID=0;
void mallocIDLOOKUP(unsigned rv,WORD len) {
  if (len) {
    WORD wID; for(wID=minwID; wID<nextwID; ) {
      unsigned wln; for(wln=0; wln<NELEM(wIDlookup); wln++) {
      	if (wln != rv && wIDlookup[wln].wParamLo) {
      		WORD mn=wIDlookup[wln].wParamLo,mx=mn+wIDlookup[wln].len-1;
          //MessageBoxFree(nppData._nppHandle,smprintf("wID:%u wID+len:%u -- mn:%u mx:%u",wID,wID+len,mn,mx),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
          //INT_CURRENTEDIT; GET_CURRENTEDIT; char *t;t=smprintf("mn:%u >= wID:%u && mn:%u < wID+len-1:%u || mx:%u >= wID:%u && mx:%u < wID+len-1:u\r\n",mn,wID,mn,wID+len-1,mx,wID,mx,wID+len-1);  SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, t); freesafe(t,"mallocIDLOOKUP");
          if (mn>=wID && mn<wID+len-1 || mx>=wID && mx<=wID+len-1) {wID=mx+1; goto cnt;}
        }
      }
      goto found;
cnt: ;
    }
found:
    wIDlookup[rv].wParamLo=wID; wIDlookup[rv].len=len; if (nextwID<wID+len) nextwID=wID+len;
  }
}

void mpxbuildmenu(char *buf,unsigned idx,WORD ic) {
  MENUITEMINFO mi;
	freeIDLOOKUP(idx);
  mallocIDLOOKUP(idx,ic);
  wIDlookup[idx].appdata1=buf;
  //DeleteMenu(wIDlookup[idx].hSubMenu,funcItem[wIDlookup[idx].mi]._cmdID,MF_BYCOMMAND);
  char *c=wIDlookup[idx].appdata1; USHORT *cs=(USHORT *)c;
  WORD i; for(i=0; i<ic; i++) {
    //MessageBoxFree(nppData._nppHandle,smprintf("i:%u str:(%x + %u)",i,c,cs[i]),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
    mi.cbSize=sizeof(mi); mi.fMask=MIIM_ID|MIIM_TYPE; mi.wID=wIDlookup[idx].wParamLo+i; mi.dwTypeData=c+cs[i]; mi.cch=strlen(mi.dwTypeData); mi.fType=MFT_STRING; InsertMenuItem(wIDlookup[idx].hSubMenu,GetMenuItemCount(wIDlookup[idx].hSubMenu),TRUE,&mi);
  }
}

#include "nppsrc/resource.h" /* From Notepad ++ */
// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/winui/windowsuserinterface/windowing/windowprocedures/usingwindowprocedures.asp
WNDPROC wpNPPOrigProc=NULL;
// This init() is correct but I think wpNPPOrigProc should be set before SetWindowLong() sets the new proc. This would mean doing it in two steps.
#define NPPSubclassProc_init() if (!wpNPPOrigProc) wpNPPOrigProc=(WNDPROC)SetWindowLong(nppData._nppHandle,GWL_WNDPROC,(LONG)NPPSubclassProc)
#define NPPSubclassProc_close() if (wpNPPOrigProc) do {SetWindowLong(nppData._nppHandle,GWL_WNDPROC,(LONG)wpNPPOrigProc); wpNPPOrigProc=NULL;} while(0)
LRESULT APIENTRY NPPSubclassProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
  if (uMsg==WM_COMMAND) {
    WORD wParamLo=LOWORD(wParam),wParamHi=HIWORD(wParam);
    if (wParamLo>=minwID && wParamLo<nextwID) {
    	unsigned i; for(i=0; i<NELEM(wIDlookup); i++) if (wParamLo>=wIDlookup[i].wParamLo && wParamLo<wIDlookup[i].wParamLo+wIDlookup[i].len) {
    		if (!(wParamLo=wIDlookup[i].usefunc(i,wParamLo-wIDlookup[i].wParamLo))) return(0);
    		wParam=MAKEWPARAM(wParamLo,wParamHi);
    		break;
    	}
    } //else if (wParamLo == IDM_EDIT_COPY) wParam=MAKEWPARAM(funcItem[findmenuitem(pfcopyall)]._cmdID,wParamHi);
    //else if (wParamLo == IDM_EDIT_LINE_UP  ) wParam=MAKEWPARAM(funcItem[findmenuitem(pfMarkWordFindReverse)]._cmdID,wParamHi);
    //else if (wParamLo == IDM_EDIT_LINE_DOWN) wParam=MAKEWPARAM(funcItem[findmenuitem(pfMarkWordFindForward)]._cmdID,wParamHi);
    else if (wParamLo == IDM_SEARCH_GOTOMATCHINGBRACE) wParam=MAKEWPARAM(funcItem[findmenuitem(pffindmatchchar)]._cmdID,wParamHi);
  }
  return CallWindowProc(wpNPPOrigProc, hwnd, uMsg, wParam, lParam);
}

PFUNCPLUGINCMD freeallIDLOOKUP(void) {
  unsigned wln; for(wln=0; wln<NELEM(wIDlookup); wln++) freeIDLOOKUP(wln);
}

PFUNCPLUGINCMD buildallIDLOOKUP(void) {
  unsigned wln; for(wln=0; wln<NELEM(wIDlookup); wln++) wIDlookup[wln].buildfunc(wln);
}

#if NPPDEBUG
PFUNCPLUGINCMD pfdisplaywIDlookup() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  char *dt=NULL; unsigned dtsz=0,dtlen;
  sarmprintf(&dt,&dtsz,&dtlen,"minwID:%u nextwID:%u\r\n",minwID,nextwID);
  int wln; for(wln=0; wln<NELEM(wIDlookup); wln++) sarmprintf(&dt,&dtsz,&dtlen,"dummy:0x%X build:0x%X use:0x%X mi:%u wParamLo:%d len:%d appdata1:%X hSubMenu:%x\r\n",
    wIDlookup[wln]._pFunc,wIDlookup[wln].buildfunc,wIDlookup[wln].usefunc,wIDlookup[wln].wParamLo,wIDlookup[wln].len,wIDlookup[wln].appdata1,wIDlookup[wln].hSubMenu);
  if (dt) {
    SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, dt);
    freesafe(dt,"pfdisplaywIDlookup");
  }
}

PFUNCPLUGINCMD pfdisplaywIDtest() {
#if 0
  INT_CURRENTEDIT; GET_CURRENTEDIT;
	char *t;
  int i;for(i=1; i<1000; i+=7) {
    if (i&1) mallocIDLOOKUP(0,10);
    else if (i&2) mallocIDLOOKUP(0,20);
    if (i&4) mallocIDLOOKUP(1,20);
    else if (i&8) mallocIDLOOKUP(1,10);
    USHORT mn0=wIDlookup[0].wParamLo,mx0=mn0+wIDlookup[0].len-1,mn1=wIDlookup[1].wParamLo,mx1=mn1+wIDlookup[1].len-1;
    if (mn0 && mn1) {
      if (mn0>=mn1 && mn0<=mx1 || mx0>=mn1 && mx0<=mx1) {t=smprintf("Fail mn0:%u mx0:%u mn1:%u mx1:%u",mn0,mx0,mn1,mx1); SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, t); freesafe(t,"pfdisplaywIDtest"); break;}
    }
  }
#endif
  buildallIDLOOKUP();pfdisplaywIDlookup();//mallocIDLOOKUP(0,20);
#if 0
	mallocIDLOOKUP(1,20);pfdisplaywIDlookup();mallocIDLOOKUP(1,10);mallocIDLOOKUP(0,20);pfdisplaywIDlookup();mallocIDLOOKUP(1,20);mallocIDLOOKUP(0,10);
#endif
	pfdisplaywIDlookup();
}
#endif
// SUBSECTION: END

BOOL altercheck(int itemno,char action) {
  switch(action) {
  case '1': // set
    funcItem[itemno]._init2Check = TRUE;
    break;
  case '0': // clear
    funcItem[itemno]._init2Check = FALSE;
    break;
  case '!': // invert
    funcItem[itemno]._init2Check = !funcItem[itemno]._init2Check;
    break;
  case '-': // change nothing but apply the current check value
    break;
  }
  CheckMenuItem(GetMenu(nppData._nppHandle), funcItem[itemno]._cmdID, MF_BYCOMMAND | ((funcItem[itemno]._init2Check)?MF_CHECKED:MF_UNCHECKED));
  return(funcItem[itemno]._init2Check);
}

// these are used to instantly access the menu items
unsigned miSortAscending; PFUNCPLUGINCMD pfSortAscending()    {altercheck(miSortAscending,'!');}
PFUNCPLUGINCMD pfqsortlinesnc()   { convertall('q',CAFLAG_EXTENDTOLINES|CAFLAG_DENYBINARY,"n",funcItem[miSortAscending]._init2Check?"a":"d",NULL,NULL); }
PFUNCPLUGINCMD pfqsortlinesc()    { convertall('q',CAFLAG_EXTENDTOLINES|CAFLAG_DENYBINARY,"c",funcItem[miSortAscending]._init2Check?"a":"d",NULL,NULL); }

unsigned miBlockOverwrite; PFUNCPLUGINCMD pfBlockOverwrite() {
  if (altercheck(miBlockOverwrite,'!')) {
    INT_CURRENTEDIT; GET_CURRENTEDIT;
    SENDMSGTOCED(currentEdit, SCI_SETOVERTYPE, 0, 0);
  }
}

unsigned miDisableSubclassing;    PFUNCPLUGINCMD pfDisableSubclassing()  {
	if (altercheck(miDisableSubclassing,'!')) {
      NPPSubclassProc_close();
      freeallIDLOOKUP();
	} else {
      buildallIDLOOKUP();
      NPPSubclassProc_init();
	}
}

#ifndef SCMOD_NORM
#define SCMOD_NORM 0
#endif
void WrapFriendlyHomeEnd(BOOL enable) {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  if (enable) {
    SENDMSGTOCED(currentEdit,SCI_ASSIGNCMDKEY, MAKEWPARAM(SCK_HOME,SCMOD_NORM) , SCI_HOMEWRAP);
    SENDMSGTOCED(currentEdit,SCI_ASSIGNCMDKEY, MAKEWPARAM(SCK_HOME,SCMOD_SHIFT), SCI_HOMEWRAPEXTEND);
    SENDMSGTOCED(currentEdit,SCI_ASSIGNCMDKEY, MAKEWPARAM(SCK_END,SCMOD_NORM)  , SCI_LINEENDWRAP);
    SENDMSGTOCED(currentEdit,SCI_ASSIGNCMDKEY, MAKEWPARAM(SCK_END,SCMOD_SHIFT) , SCI_LINEENDWRAPEXTEND);
  } else {
    SENDMSGTOCED(currentEdit,SCI_ASSIGNCMDKEY, MAKEWPARAM(SCK_HOME,SCMOD_NORM) , SCI_HOME);
    SENDMSGTOCED(currentEdit,SCI_ASSIGNCMDKEY, MAKEWPARAM(SCK_HOME,SCMOD_SHIFT), SCI_HOMEEXTEND);
    SENDMSGTOCED(currentEdit,SCI_ASSIGNCMDKEY, MAKEWPARAM(SCK_END,SCMOD_NORM)  , SCI_LINEEND);
    SENDMSGTOCED(currentEdit,SCI_ASSIGNCMDKEY, MAKEWPARAM(SCK_END,SCMOD_SHIFT) , SCI_LINEENDEXTEND);
  }
}
unsigned miWrapFriendlyHomeEnd; PFUNCPLUGINCMD pfWrapFriendlyHomeEnd() {WrapFriendlyHomeEnd(altercheck(miWrapFriendlyHomeEnd,'!'));}

BOOL CaptureCutCopyPaste(BOOL capture) {
#if NPPFUNCITEMARRAYVERSION>=2
  if (!capture) {
    funcItem[findmenuitem(pfcopyallnoappend)]._pShKey=NULL;
    funcItem[findmenuitem(pfcutallnoappend)]._pShKey=NULL;
    funcItem[findmenuitem(pfVizPasteUNICODE)]._pShKey=NULL;
  }
#endif
  return(capture);
}
unsigned miCaptureCutCopyPaste; PFUNCPLUGINCMD pfCaptureCutCopyPaste() {
  MessageBoxFree(nppData._nppHandle,smprintf("Next time you load Notepad++, Ctrl X,C,V will%s be captured!",CaptureCutCopyPaste(altercheck(miCaptureCutCopyPaste,'!'))?"":" not"),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  ;}

unsigned miSeparateQuickMenus       ;PFUNCPLUGINCMD pfSeparateQuickMenus()       {
  MessageBoxFree(nppData._nppHandle,smprintf("Next time you load Notepad++, quick menus will%s be separated!",altercheck(miSeparateQuickMenus,'!')?"":" not"),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
}

unsigned miAutoCloseHTMLtag         ;PFUNCPLUGINCMD pfAutoCloseHTMLtag()         {altercheck(miAutoCloseHTMLtag,'!');}
unsigned miAutoCloseBrace           ;PFUNCPLUGINCMD pfAutoCloseBrace()           {altercheck(miAutoCloseBrace,'!');}
unsigned miAutoSpace2Tab            ;PFUNCPLUGINCMD pfAutoSpace2Tab()            {altercheck(miAutoSpace2Tab,'!');}
unsigned miAutoShowMatchline        ;PFUNCPLUGINCMD pfAutoShowMatchline()        {if (altercheck(miAutoShowMatchline,'!')) pfshowmatchline();}
unsigned miAutoConvertHTML          ;PFUNCPLUGINCMD pfAutoConvertHTML()          {altercheck(miAutoConvertHTML,'!');}
unsigned miSortLinesUnique          ;PFUNCPLUGINCMD pfSortLinesUnique()          {SortLinesUnique=altercheck(miSortLinesUnique,'!');}
unsigned miMarkWordFindCaseSensitive;PFUNCPLUGINCMD pfMarkWordFindCaseSensitive(){MarkWordFindCaseSensitive=altercheck(miMarkWordFindCaseSensitive,'!');}
#ifndef HighPerformance
unsigned miHighPerformance          ;PFUNCPLUGINCMD pfHighPerformance()          {HighPerformance=altercheck(miHighPerformance,'!');}
#endif
unsigned miVizCaseSensitive         ;PFUNCPLUGINCMD pfVizCaseSensitive()         {VizCaseSensitive=altercheck(miVizCaseSensitive,'!');}
unsigned miVizWholeWords            ;PFUNCPLUGINCMD pfVizWholeWords()            {VizWholeWords=altercheck(miVizWholeWords,'!');}
unsigned miVizRegex                 ;PFUNCPLUGINCMD pfVizRegex()                 {VizRegex=altercheck(miVizRegex,'!');}
unsigned miVizCutCopyAppend         ;PFUNCPLUGINCMD pfVizCutCopyAppend()         {VizCutCopyAppend=altercheck(miVizCutCopyAppend,'!');}
unsigned miVizClipboardAlwaysCRLF   ;PFUNCPLUGINCMD pfVizClipboardAlwaysCRLF()   {VizClipboardAlwaysCRLF=altercheck(miVizClipboardAlwaysCRLF,'!');}
unsigned miVizClipboardReplaceNulls ;PFUNCPLUGINCMD pfVizClipboardReplaceNulls() {VizClipboardReplaceNulls=altercheck(miVizClipboardReplaceNulls,'!');}
unsigned miVizClipboardCopyAlsoUTF8 ;PFUNCPLUGINCMD pfVizClipboardCopyAlsoUTF8() {VizClipboardCopyAlsoUTF8=altercheck(miVizClipboardCopyAlsoUTF8,'!');}
unsigned miVizClipboardNotUnicode   ;PFUNCPLUGINCMD pfVizClipboardNotUnicode()   {VizClipboardNotUnicode=altercheck(miVizClipboardNotUnicode,'!');}
unsigned miVizPasteRetainsPosition  ;PFUNCPLUGINCMD pfVizPasteRetainsPosition()  {VizPasteRetainsPosition=altercheck(miVizPasteRetainsPosition,'!');}
unsigned miVizPasteBinary           ;PFUNCPLUGINCMD pfVizPasteBinary()           {VizPasteBinary=altercheck(miVizPasteBinary,'!');}
unsigned miVizPasteToEditorEOL      ;PFUNCPLUGINCMD pfVizPasteToEditorEOL()      {VizPasteToEditorEOL=altercheck(miVizPasteToEditorEOL,'!');}

void iniSaveSettings(char *ppath,BOOL save) {
  char iniFilePath[MAX_PATH],*i2;
  WIN32_FIND_DATA wfd;
  HANDLE h;

	miSeparateQuickMenus       =findmenuitem(pfSeparateQuickMenus);
	miBlockOverwrite           =findmenuitem(pfBlockOverwrite);
	miAutoCloseHTMLtag         =findmenuitem(pfAutoCloseHTMLtag);
	miAutoCloseBrace           =findmenuitem(pfAutoCloseBrace);
	miAutoSpace2Tab            =findmenuitem(pfAutoSpace2Tab);
	miAutoShowMatchline        =findmenuitem(pfAutoShowMatchline);
	miAutoConvertHTML          =findmenuitem(pfAutoConvertHTML);
	miSortAscending            =findmenuitem(pfSortAscending);
	miSortLinesUnique          =findmenuitem(pfSortLinesUnique);
	miVizCaseSensitive         =findmenuitem(pfVizCaseSensitive);
	miVizWholeWords            =findmenuitem(pfVizWholeWords);
	miVizRegex                 =findmenuitem(pfVizRegex);
	miVizCutCopyAppend         =findmenuitem(pfVizCutCopyAppend);
	miVizClipboardAlwaysCRLF   =findmenuitem(pfVizClipboardAlwaysCRLF);
	miVizClipboardReplaceNulls =findmenuitem(pfVizClipboardReplaceNulls);
	miVizClipboardCopyAlsoUTF8 =findmenuitem(pfVizClipboardCopyAlsoUTF8);
	miVizClipboardNotUnicode   =findmenuitem(pfVizClipboardNotUnicode);
	miVizPasteRetainsPosition  =findmenuitem(pfVizPasteRetainsPosition);
	miVizPasteBinary           =findmenuitem(pfVizPasteBinary);
	miVizPasteToEditorEOL      =findmenuitem(pfVizPasteToEditorEOL);
	miDisableSubclassing       =findmenuitem(pfDisableSubclassing);
	miMarkWordFindCaseSensitive=findmenuitem(pfMarkWordFindCaseSensitive);
	miWrapFriendlyHomeEnd      =findmenuitem(pfWrapFriendlyHomeEnd);
	miCaptureCutCopyPaste      =findmenuitem(pfCaptureCutCopyPaste);
#ifndef HighPerformance
	miHighPerformance          =findmenuitem(pfHighPerformance);
#endif
  //mimpxhelpdummy  =findmenuitem(pfmpxhelprebuild);
  ITEMIDLIST *pidl; SHGetSpecialFolderLocation(NULL, CSIDL_APPDATA, &pidl);
  i2=smprintfpath("%s%?\\%s",ppath,"..\\doLocalConf.xml");
  //MessageBoxFree(0/*nppData._nppHandle*/,smprintf("Looking for %s",i2),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
  if (i2 && INVALID_HANDLE_VALUE!=(h=FindFirstFile(i2,&wfd))) {
    FindClose(h);
    snprintfX(iniFilePath,sizeof(iniFilePath),ppath); // snprintfX() is bug free, strncpy() is permanently bugged
  } else {
    SHGetPathFromIDList(pidl, iniFilePath);
    /* PathAppend() has thrown compilers for a loop, even MSVC */
    PathAppend(iniFilePath, "Notepad++\\" );
  }
  if (i2) freesafe(i2,"iniSaveSettings");
  PathAppend(iniFilePath, "NPP" PLUGIN_NAME ".ini");
  //MessageBoxFree(0/*nppData._nppHandle*/,smprintf("Result: %s",iniFilePath),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
  if (!save) { //
    altercheck(miSeparateQuickMenus,GetPrivateProfileInt("Settings", "SeparateQuickMenus", 0, iniFilePath)?'1':'0');
    altercheck(miBlockOverwrite    ,GetPrivateProfileInt("Settings", "BlockOverwrite"    , 1, iniFilePath)?'1':'0');
    altercheck(miAutoCloseBrace    ,GetPrivateProfileInt("Settings", "AutoCloseBrace"    , 1, iniFilePath)?'1':'0');
    altercheck(miAutoCloseHTMLtag  ,GetPrivateProfileInt("Settings", "AutoCloseHTMLtag"  , 1, iniFilePath)?'1':'0');
    altercheck(miAutoSpace2Tab     ,GetPrivateProfileInt("Settings", "AutoSpace2Tab"     , 1, iniFilePath)?'1':'0');
    altercheck(miAutoShowMatchline ,GetPrivateProfileInt("Settings", "AutoShowMatchline" , 0, iniFilePath)?'1':'0');
    altercheck(miAutoConvertHTML   ,GetPrivateProfileInt("Settings", "AutoConvertHTML"   , 0, iniFilePath)?'1':'0');
    altercheck(miSortAscending     ,GetPrivateProfileInt("Settings", "SortAscending"     , 1, iniFilePath)?'1':'0');
    altercheck(miDisableSubclassing,GetPrivateProfileInt("Settings", "DisableSubclassing", 0, iniFilePath)?'1':'0');
    SortLinesUnique          =altercheck(miSortLinesUnique          ,GetPrivateProfileInt("Settings", "SortLinesUnique"          , 0, iniFilePath)?'1':'0');
    MarkWordFindCaseSensitive=altercheck(miMarkWordFindCaseSensitive,GetPrivateProfileInt("Settings", "MarkWordFindCaseSensitive", 0, iniFilePath)?'1':'0');
    altercheck(miWrapFriendlyHomeEnd,GetPrivateProfileInt("Settings", "WrapFriendlyHomeEnd", 0, iniFilePath)?'1':'0');
    CaptureCutCopyPaste(altercheck(miCaptureCutCopyPaste,GetPrivateProfileInt("Settings", "CaptureCutCopyPaste", 0, iniFilePath)?'1':'0'));
#ifndef HighPerformance
    HighPerformance          =altercheck(miHighPerformance         ,GetPrivateProfileInt("Settings", "HighPerformance"         , 1, iniFilePath)?'1':'0');
#endif
    VizCaseSensitive        =altercheck(miVizCaseSensitive        ,GetPrivateProfileInt("Viz", "VizCaseSensitive"        , 0, iniFilePath)?'1':'0');
    VizWholeWords           =altercheck(miVizWholeWords           ,GetPrivateProfileInt("Viz", "VizWholeWords"           , 0, iniFilePath)?'1':'0');
    VizRegex                =altercheck(miVizRegex                ,GetPrivateProfileInt("Viz", "VizRegex"                , 0, iniFilePath)?'1':'0');
    VizCutCopyAppend        =altercheck(miVizCutCopyAppend        ,GetPrivateProfileInt("Viz", "VizCutCopyAppend"        , 0, iniFilePath)?'1':'0');
    VizClipboardAlwaysCRLF  =altercheck(miVizClipboardAlwaysCRLF  ,GetPrivateProfileInt("Viz", "VizClipboardAlwaysCRLF"  , 1, iniFilePath)?'1':'0');
    VizClipboardReplaceNulls=altercheck(miVizClipboardReplaceNulls,GetPrivateProfileInt("Viz", "VizClipboardReplaceNulls", 0, iniFilePath)?'1':'0');
    VizClipboardCopyAlsoUTF8=altercheck(miVizClipboardCopyAlsoUTF8,GetPrivateProfileInt("Viz", "VizClipboardCopyAlsoUTF8", 0, iniFilePath)?'1':'0');
    VizClipboardNotUnicode  =altercheck(miVizClipboardNotUnicode  ,GetPrivateProfileInt("Viz", "VizClipboardNotUnicode"  , 0, iniFilePath)?'1':'0');
    VizPasteRetainsPosition =altercheck(miVizPasteRetainsPosition ,GetPrivateProfileInt("Viz", "VizPasteRetainsPosition" , 0, iniFilePath)?'1':'0');
    VizPasteBinary          =altercheck(miVizPasteBinary          ,GetPrivateProfileInt("Viz", "VizPasteBinary"          , 0, iniFilePath)?'1':'0');
    VizPasteToEditorEOL     =altercheck(miVizPasteToEditorEOL     ,GetPrivateProfileInt("Viz", "VizPasteToEditorEOL"     , 1, iniFilePath)?'1':'0');
  } else {
    WritePrivateProfileString("Settings","SeparateQuickMenus"       ,(funcItem[miSeparateQuickMenus       ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","BlockOverwrite"           ,(funcItem[miBlockOverwrite           ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","AutoCloseBrace"           ,(funcItem[miAutoCloseBrace           ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","AutoCloseHTMLtag"         ,(funcItem[miAutoCloseHTMLtag         ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","AutoSpace2Tab"            ,(funcItem[miAutoSpace2Tab            ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","AutoShowMatchline"        ,(funcItem[miAutoShowMatchline        ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","AutoConvertHTML"          ,(funcItem[miAutoConvertHTML          ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","SortAscending"            ,(funcItem[miSortAscending            ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","SortLinesUnique"          ,(funcItem[miSortLinesUnique          ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","MarkWordFindCaseSensitive",(funcItem[miMarkWordFindCaseSensitive]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","DisableSubclassing"       ,(funcItem[miDisableSubclassing       ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","WrapFriendlyHomeEnd"      ,(funcItem[miWrapFriendlyHomeEnd      ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Settings","CaptureCutCopyPaste"      ,(funcItem[miCaptureCutCopyPaste      ]._init2Check)?"1":"0",iniFilePath);
#ifndef HighPerformance
    WritePrivateProfileString("Settings","HighPerformance"          ,(funcItem[miHighPerformance          ]._init2Check)?"1":"0",iniFilePath);
#endif
    WritePrivateProfileString("Viz"     ,"VizCaseSensitive"         ,(funcItem[miVizCaseSensitive         ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizWholeWords"            ,(funcItem[miVizWholeWords            ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizRegex"                 ,(funcItem[miVizRegex                 ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizCutCopyAppend"         ,(funcItem[miVizCutCopyAppend         ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizClipboardAlwaysCRLF"   ,(funcItem[miVizClipboardAlwaysCRLF   ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizClipboardReplaceNulls" ,(funcItem[miVizClipboardReplaceNulls ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizClipboardCopyAlsoUTF8" ,(funcItem[miVizClipboardCopyAlsoUTF8 ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizClipboardNotUnicode"   ,(funcItem[miVizClipboardNotUnicode   ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizPasteRetainsPosition"  ,(funcItem[miVizPasteRetainsPosition  ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizPasteBinary"           ,(funcItem[miVizPasteBinary           ]._init2Check)?"1":"0",iniFilePath);
    WritePrivateProfileString("Viz"     ,"VizPasteToEditorEOL"      ,(funcItem[miVizPasteToEditorEOL      ]._init2Check)?"1":"0",iniFilePath);
  }
}

// partial string case insensitive compare skipping &
// returns 0 if the same or at least equal to where s2 ends
// if & is found in s1, it is skipped
// compare is case insenstive
// new feature: bug: This skips all & signs, it should only skip one at a time to allow for && escapes
int strxicmpamp(char *s1,char *s2) {
  int rv=0;
  if (!*s2 || !*s1) return(-1);
  while(1) {
    if (*s1=='&') s1++;
    else if (!*s2) break;
    else if (rv=(toupper(*s1) - toupper(*s2))) break;
    else {s1++; s2++; }
  }
  return(rv);
}

struct _MENUWALK {// Set-by-caller values marked with (*) do not or may not under some circumstances persist between calls and must be reset each time
  char cmd;       // set by caller 'f' for find, 'd' for display; 'w' to find largest wId
  unsigned level; // set by caller to 0
  BOOL quit;      //*set by caller to FALSE, will be set to TRUE if recursion needs to be aborted
  BOOL recurse;   // set by caller: FALSE, only search the level specified in the function call; TRUE; additionally search all submenus
  char *ms;       // set by caller to string to store the menu item text
  unsigned szms;  // set by caller to sizeof(ms)
  char *b;        // cmd='d': set by caller to NULL or a malloc()'d buffer, may be NULL; cmd='f': small malloc()'d buffer 'text to find'
  unsigned bl;    // set by caller to the malloc()'d size of the buffer, or the desired starting size of the buffer to be malloc()'d if b is NULL.
  unsigned blen;  //*cmd='d': does not need to be set if b starts as NULL, if b starts as a malloc()'d buffer, must be set to starting point to append new stuff; cmd='f', not used
  HMENU foundmenu;//*cmd='f': set by caller to NULL, will return HMENU when item is found, or NULL if nothing found
  int founditem;  // set to the item number during 'f' when item is found
  MENUITEMINFO mi;// not set by caller, used constantly during operation and will return MENUITEM details for cmd='f' upon valid foundmenu
  USHORT maxwID;   //*set by caller to 0, returns largest wID for non submenus on completion for cmd='w'
} ;

void menuwalk(HMENU hmn,struct _MENUWALK *mx) {
  mx->level++;
  if (mx->cmd=='d') sarmprintf(&mx->b,&mx->bl,&mx->blen,"%-*s%2u:Menu:0x%X:%d\n",mx->level*2,"",mx->level,hmn,GetMenuItemCount(hmn));
  int pos; for(pos=0; !mx->quit && pos<GetMenuItemCount(hmn); pos++) {
    mx->mi.fMask=/*MIIM_CHECKMARKS*/MIIM_DATA|MIIM_ID|MIIM_STATE|MIIM_SUBMENU|MIIM_TYPE;
    mx->mi.cbSize=sizeof(MENUITEMINFO)-sizeof(HBITMAP); /* This is for NT4 compatibility */ mx->mi.dwTypeData=mx->ms; mx->mi.cch=mx->szms;
    GetMenuItemInfo(hmn,pos,TRUE,&(mx->mi));
    switch (mx->cmd) {
    case 'd':
      sarmprintf(&mx->b,&mx->bl,&mx->blen,"%-*s%2u-dwTypeData:'%-12s'@cch:%u wID=%d dwItemData=0x%x Submenu=0x%x ",mx->level*2,"",pos,mx->mi.dwTypeData,mx->mi.cch,mx->mi.wID,mx->mi.dwItemData,mx->mi.hSubMenu);
      if (!mx->b) {mx->quit=TRUE; break;}
      if (mx->mi.fType&MFT_BITMAP       ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFT_BITMAP"      ,"menuwalk");
      if (mx->mi.fType&MFT_MENUBARBREAK	) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFT_MENUBARBREAK","menuwalk");
      if (mx->mi.fType&MFT_MENUBREAK    ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFT_MENUBREAK"   ,"menuwalk");
      if (mx->mi.fType&MFT_OWNERDRAW    ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFT_OWNERDRAW"   ,"menuwalk");
      if (mx->mi.fType&MFT_RADIOCHECK   ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFT_RADIOCHECK"  ,"menuwalk");
      if (mx->mi.fType&MFT_RIGHTJUSTIFY ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFT_RIGHTJUSTIFY","menuwalk");
      if (mx->mi.fType&MFT_SEPARATOR    ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFT_SEPARATOR"   ,"menuwalk");
      if (mx->mi.fType&MFT_STRING       ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFT_STRING"      ,"menuwalk");
      if (mx->mi.fType&MFS_CHECKED      ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFS_CHECKED"     ,"menuwalk");
      if (mx->mi.fState&MFS_DEFAULT     ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFS_DEFAULT"     ,"menuwalk");
      if (mx->mi.fState&MFS_DISABLED    ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFS_DISABLED"    ,"menuwalk");
      if (mx->mi.fState&MFS_ENABLED     ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFS_ENABLED"     ,"menuwalk");
      if (mx->mi.fState&MFS_GRAYED      ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFS_GRAYED"      ,"menuwalk");
      if (mx->mi.fState&MFS_HILITE      ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFS_HILITE"      ,"menuwalk");
      if (mx->mi.fState&MFS_UNCHECKED   ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFS_UNCHECKED"   ,"menuwalk");
      if (mx->mi.fState&MFS_UNHILITE    ) strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"|MFS_UNHILITE"    ,"menuwalk");
      if (!mx->b) mx->quit=TRUE;
      else {
        strcpyarmsafe(&mx->b,&mx->bl,&mx->blen,"\n","menuwalk"); if (!mx->b) mx->quit=TRUE;
      }
      break;
    case 'f':
      if (!mx->ms) mx->quit=TRUE;
      else if (!strxicmpamp(mx->ms,mx->b)) {
        mx->foundmenu=hmn;
        mx->founditem=pos;
        mx->quit=TRUE;
      }
      break;
    case 'w':
      if (!(mx->mi.hSubMenu) && mx->maxwID < LOWORD(mx->mi.wID)) {
     	  mx->maxwID = LOWORD(mx->mi.wID);
        //MessageBoxFree(nppData._nppHandle,smprintf("wId:%u",mx->mi.wID),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
      }
      break;
    }
    if (mx->recurse && !mx->quit && mx->mi.hSubMenu) menuwalk(mx->mi.hSubMenu,mx);
  }
  mx->level--;
}

#if NPPDEBUG
PFUNCPLUGINCMD pfshowmenuinfo() {
  char ms[128];
  struct _MENUWALK mx;
  mx.cmd='d'; mx.level=0; mx.recurse=TRUE; mx.ms=ms; mx.szms=sizeof(ms);
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  mx.b=NULL; mx.bl=512; mx.foundmenu=NULL; mx.quit=FALSE; menuwalk(GetMenu(nppData._nppHandle),&mx);
  if (mx.b) {
    SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, mx.b);
    freesafe(mx.b,"pfshowmenuinfo");
  }
}
#endif

struct _MENUMOVE {
  int location; //0=item goes into the plugin menu; 1=a new menu under pluginmenu; 2=a new menu on the menu bar
  char prefix;  // character: of menu to detect
  char menuname[26]; // applies to 1&2
} menumove[]= {
{2,'Q',PLUGIN_NAME " Quick"},
{1,'E',PLUGIN_NAME " Edit"},
{1,'C',PLUGIN_NAME " Convert"},
//{1,'H',PLUGIN_NAME " Online Help"},
{1,'D',PLUGIN_NAME " HTML Tidy"},
{1,'T',PLUGIN_NAME " Tools"},
{2,'V',PLUGIN_NAME " Viz"},
{1,'W',PLUGIN_NAME " Viz Settings"},
{1,'S',PLUGIN_NAME " Settings"},
};

#if 0 /* This is not NT4 compatible; it works fine in XP */
MenuItemConvertUTF8toUCS2(HMENU hMenu,UINT uItem,BOOL fByPosition) {
  MENUITEMINFOA mia;
  char buf[128];
  mia.fMask=MIIM_TYPE; mia.cbSize=sizeof(mia)-sizeof(HBITMAP); /* This is for NT4 compatibility */ mia.dwTypeData=buf; mia.cch=sizeof(buf);
  if (GetMenuItemInfoA(hMenu,uItem,fByPosition,&mia) && (mia.dwTypeData) && mia.cch>0) {
    wchar_t ubuf[128];
    MENUITEMINFOW miw;
    miw.cch=UCS2FromUTF8(buf,mia.cch+1,ubuf,sizeof(ubuf)); ubuf[sizeof(ubuf)-1]=L'\0';
    //MessageBoxW(nppData._nppHandle,ubuf,L"Plugin Name", MB_OK|MB_ICONWARNING);
    miw.fMask=MIIM_TYPE; miw.cbSize=sizeof(miw)-sizeof(HBITMAP); /* This is for NT4 compatibility */ miw.dwTypeData=ubuf;
    SetMenuItemInfoW(hMenu,uItem,fByPosition,&miw);
  }
}
#endif

// Rearrange the plugin menu and apply separator mods
// new feature: ! makes menu item disappear
PFUNCPLUGINCMD pfbuildmenu() {
  char ms[128];
  struct _MENUWALK mx;
#if NPPDEBUG /* { */
    if (!powcheck(sizeof(struct _MENUMOVE)))
      MessageBoxFree(nppData._nppHandle,smprintf("sizeof(struct _MENUMOVE)==%u is not a power of two",sizeof(struct _MENUMOVE)),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
    if (!powcheck(sizeof(struct _wIDLOOKUP)))
      MessageBoxFree(nppData._nppHandle,smprintf("sizeof(struct _wIDLOOKUP)==%u is not a power of two",sizeof(struct _wIDLOOKUP)),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
#endif /* } */
  unsigned wln; for(wln=0; wln<NELEM(wIDlookup); wln++) wIDlookup[wln].wParamLo=funcItem[findmenuitem(wIDlookup[wln]._pFunc)]._cmdID;
  mx.cmd='f'; mx.level=0; mx.recurse=FALSE; mx.ms=ms; mx.szms=sizeof(ms);
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  mx.b=NULL; mx.bl=0;
  do {
    if (!funcItem[miSeparateQuickMenus]._init2Check) {unsigned i; for(i=0; i<NELEM(menumove); i++) if (menumove[i].location>1) menumove[i].location=1;}
    strcpyarmsafe(&mx.b,&mx.bl,NULL,"Plugin","pfbuildmenu"); if (!mx.b) goto failbreak; mx.foundmenu=NULL; mx.quit=FALSE; menuwalk(GetMenu(nppData._nppHandle),&mx); if (!mx.b || !mx.foundmenu) break;
    HMENU pluginsubmenu=mx.mi.hSubMenu;
    HMENU pluginmenubar=mx.foundmenu;
    int pluginmenubaritem=mx.founditem;
    int pluginmenubaroffset=0;
    mx.cmd='w'; mx.maxwID=0; mx.recurse=TRUE; mx.foundmenu=NULL; mx.quit=FALSE; menuwalk(pluginsubmenu,&mx); nextwID=minwID=mx.maxwID+1; mx.cmd='f'; mx.recurse=FALSE; // Safe this is not! When notepad updates, inspection it will need!
    strcpyarmsafe(&mx.b,&mx.bl,NULL,"_"PLUGIN_NAME,"pfbuildmenu"); if (!mx.b) goto failbreak; mx.foundmenu=NULL; mx.quit=FALSE; menuwalk(pluginsubmenu,&mx); if (!mx.b || !mx.foundmenu) break;
    HMENU pluginmenu=mx.foundmenu;
    int pluginmenuitem=mx.founditem;
    int pluginmenuitemoffset=0;
    mx.mi.fMask=MIIM_TYPE; memmove(mx.mi.dwTypeData,mx.mi.dwTypeData+1,mx.mi.cch); mx.mi.cch--; // remove the underline
    SetMenuItemInfo(pluginmenu,pluginmenuitem,TRUE,&mx.mi);
    HMENU textfxorig=mx.mi.hSubMenu;
    HMENU newmenu=NULL;
    unsigned i;
    for(i=0; i<NELEM(menumove); i++) {
      char ck[3];
      snprintfX(ck,sizeof(ck),"%c:",menumove[i].prefix);
      goto bottest; do {
        DeleteMenu(mx.foundmenu,mx.founditem,MF_BYPOSITION);
        if (!GetMenuItemCount(textfxorig)) DeleteMenu(pluginmenu,pluginmenuitem,MF_BYPOSITION);
        memmovetest(mx.mi.dwTypeData,mx.mi.dwTypeData+2,mx.mi.cch-2+1);
        if (mx.mi.dwTypeData[0]=='-') mx.mi.fType=MFT_SEPARATOR;
        if (!menumove[i].location) {
          InsertMenuItem(pluginmenu,pluginmenuitem+pluginmenuitemoffset,TRUE,&mx.mi);
          //MenuItemConvertUTF8toUCS2(pluginmenu,pluginmenuitem+pluginmenuitemoffset,TRUE);
          pluginmenuitem++;
        } else {
          if (!newmenu) newmenu=CreateMenu();
          InsertMenuItem(newmenu,GetMenuItemCount(newmenu),TRUE,&mx.mi);
          //MenuItemConvertUTF8toUCS2(newmenu,GetMenuItemCount(newmenu)-1,TRUE);
        }
        for(wln=0; wln<NELEM(wIDlookup); wln++) if (mx.mi.wID==wIDlookup[wln].wParamLo) wIDlookup[wln].hSubMenu=newmenu;
        //char buf2[16000]; mx.cmd='d'; mx.b=buf2; mx.bl=sizeof(buf2); mx.quit=FALSE; menuwalk(mx.mi.hSubMenu,&mx); SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, buf2); mx.cmd='f';
bottest:
        strcpyarmsafe(&mx.b,&mx.bl,NULL,ck,"pfbuildmenu"); if (!mx.b) goto failbreak; mx.foundmenu=NULL; mx.quit=FALSE; menuwalk(textfxorig,&mx);
      } while (mx.foundmenu);
      if (newmenu) {
        MENUITEMINFO mi; memset(&mi,0,sizeof(mi)); mi.cbSize=sizeof(MENUITEMINFO)-sizeof(HBITMAP); /* NT4 compatiblity */
        mi.fMask=MIIM_SUBMENU|MIIM_TYPE; mi.fType=0;
        mi.dwTypeData=menumove[i].menuname;
        mi.hSubMenu=newmenu;
        if (menumove[i].location==1) InsertMenuItem(pluginmenu,pluginmenuitem+(++pluginmenuitemoffset),TRUE,&mi);
        else                         InsertMenuItem(pluginmenubar,pluginmenubaritem+(++pluginmenubaroffset),TRUE,&mi);
        newmenu=NULL;
      }
    }
  } while(0);
failbreak:
  for(wln=0; wln<NELEM(wIDlookup); wln++) wIDlookup[wln].wParamLo=0;
  if (mx.b) freesafe(mx.b,"pfbuildmenu");
}

// As we write this and other programs, we think of needed features and list them here.
// Features that are pretty easy:
//  new feature: text to decimal, decimal to text
//  new feature: never hide the last line of the document
//  new feature: sort: output only non duplicated lines (exclusive)
//  new feature: sort compare ends at first whitespace: http://www.ultraedit.com/index.php?name=Forums&file=viewtopic&t=1106
//  new feature: insert line number of width #
//  new feature: Autoblock editing before this column: http://www.ultraedit.com/index.php?name=Forums&file=viewtopic&t=1282
//  new feature: BIGG find and replace; options, ignore trivial crlf differences, starts..ends with
//  new feature: HTML tags to lowercase http://www.crimsoneditor.com/english/board/CrazyWWWBoard.cgi?db=forum&mode=read&num=5226&page=43&ftype=6&fval=&backdepth=1
//  new feature: viz search between columns
//  new feature: sort: remove duplicates, preserve order
//  new feature: multiple copy buffers
//  new feature: copy line
//  new feature: insert line length, sort by line length. http://www.textpad.info/forum/viewtopic.php?t=6687
//  new feature: insert random number: sort by random number
//  new feature: CSV sorting: http://www.textpad.info/forum/viewtopic.php?t=6040
//  new feature: offer stable sort http://en.wikipedia.org/wiki/Sort_algorithm#Stable (can be done if unique length is different than sort length) http://www.auto.tuwien.ac.at/~blieb/woop/merge.html
//                http://linux.wku.edu/~lamonml/algor/sort/merge.html
//  new feature: delete brace pair
//  new feature: read shortcut keys from file

// Features that are harder but still possible
//  new feature: analyze C code function usage summary
//  new feature: save recent copy blocks, present them as a screen, click on each block in order that you want them pasted
//  new feature: Search&Replace Column http://www.crimsoneditor.com/english/board/CrazyWWWBoard.cgi?db=forum&mode=read&num=5438&page=34&ftype=6&fval=&backdepth=1
//  new feature: NiceSQL, tidy SQL http://home.broadpark.no/~ihalvor/
//  new feature: http://cscope.sourceforge.net/ http://ctags.sourceforge.net/ http://forum.pspad.com/read.php?f=2&i=2664&t=2664
//  new feature: xmlformat http://forum.pspad.com/read.php?f=2&i=1449&t=1449
//  new feature: chars to html entity http://forum.pspad.com/read.php?f=2&i=2200&t=2200 (watch for license)
//  new feature: show duplicate lines (Viz)
//  new feature: PSPAD: Reformat HTML; is this separate from HTML Tidy?
//  new feature: Copy lines with line numbers: http://www.ultraedit.com/index.php?name=Forums&file=viewtopic&t=1745
//  new feature: Swap-Cut-Paste http://www.ornj.net/forums/viewtopic.php?t=576
//  new feature: close tag and text: http://developers.evrsoft.com/forum/showthread.php?t=523
//  new feature: definition finder http://sourceforge.net/tracker/index.php?func=detail&aid=988438&group_id=58130&atid=486612
//  new feature: find unmatched brace
//  new feature: launch HTML
//  new feature: prefab HTML templates
//  new feature: View in browser
//  new feature: codeblocks plugin
//  new feature: insert/remove ULAII (Universal Language Auto Include Insert) strings
//  new feature: find text after comma #, used to search through comma delimited text
//  new feature: sort lines at comma number (think delimited text)
//  new feature: paste-overwrite lines for multi line text,paste-overwrite characters for single line text
// Features that won't work properly until N++ allows us to retrieve editor settings
//  new feature: auto find bookmarks preferrably on file load
//  New Feature: Prefab search & replace strings
// Features that are easy enough but won't be useful until plugins can add custom keyboard accelerators
// Features that are too hard for me
//  new feature: column editing http://www.crimsoneditor.com/english/board/CrazyWWWBoard.cgi?db=forum&mode=read&num=2900&page=141&ftype=6&fval=&backdepth=2
//  new feature: Spellcheck, only comments area http://www.crimsoneditor.com/english/board/CrazyWWWBoard.cgi?db=forum&mode=read&num=3858&page=104&ftype=6&fval=&backdepth=1
//  new feature: Detect SDF, write conversion rule line, perform convertion from SDF to comma or tab DELI
//  new feature: BIGG-Edit Live MagnaDiff
//  new feature: find code block that is __% similar to ___
//  new feature: Goto column
//  new feature: Copy as HTML
//  new feature: #include files should be made into hotlinks
//  new feature: label or relabel with comment end of #if,#ifdef,switch,if,while,do { ... } /* autolabel: if:else (... */
//  new feature: vb to jscript converter
//  new feature: diff: compare clipboard to highlighted text, possibly hightligh, WinDiff, KDiff3
//  new feature: FC.EXE/diff/WinDiff Compare two open files or two selected text blocks in two files, write result into 3rd file

// Features that N++ will need to implement
/*Scintilla is pretty good about providing all it's internals but I need a few more things from N++ to get plugins to be more functional.
* The ability to retrieve or set styles translated by Notepad++, the ability to identify regions
* I need access to the dialog properties for Find, Replace, FindNext, FindPrevious. I need to be able to set or retrieve text, set or retrieve options, or make them perform. While I could use the SCI_ find functions, I want to make automated Search & Replace interact with users without making my own dialogs.
* I need to retrieve the N++ version numbers. At this time N++ is actively developed and later versions are universally more desirable than earlier versions. Should this not be the case in the future, I would like plugins to add or not add functionality based on version number.
* I need another DLLEXPORT callback. It would be called when the menus and screens are created so menuitems, hotkeys, and other stuff can be altered. A RunOnce in the Notify callback seems to work but it should be standardised.
* I need a callback for displaying menus. I'd like to disable menu options if no text is selected and for other reasons. I'd also like to alter the text of some menu selections depending on the language selected.
* N++ needs a find next/previous bar just like Firefox.
* setinfo() needs to accept a sizeof(struct) parameter so the struct can evolve and old and new plugins will be supported
* N++ needs a way for plugins to generate and regenerate dynamic menus; wParamLo should be sent to PFUNCPLUGINS
* Clone View Multi Monitor
* Jump to column number
* Print Dialog: Print Line Numbers, ^L pagenates http://www.crimsoneditor.com/english/board/CrazyWWWBoard.cgi?db=forum&mode=read&num=645&page=242&ftype=6&fval=&backdepth=1
* Scroll Lock locks clone views together
* Regular expression tool, #1, jump to web page, #2: Quick Reference, #3: Interactive Builder
* Search and replace static filters for syntax highlighting
* Search & Replace between columns
* Scintilla: I need access to the lexer styles so I can know if text is a comment, a string, code, brace, or whatever
* Scintilla find options need to be [x] do not search comments, [x] do not search quoted strings (language dependent)
*/

PFUNCPLUGINCMD pftemp() {
  INT_CURRENTEDIT; GET_CURRENTEDIT;
  //int codepage=SENDMSGTOCED(currentEdit, SCI_GETCODEPAGE, 0, 0);
  //MessageBoxFree(nppData._nppHandle,smprintf("CP:%d",codepage),PLUGIN_NAME, MB_OK|MB_ICONWARNING);
}

// CTRL ALT SHIFT
#if (NPPFUNCITEMARRAYVERSION >= 2)
struct ShortcutKey skMarkWordFindReverse={TRUE,TRUE,FALSE,VK_UP};
struct ShortcutKey skMarkWordFindForward={TRUE,TRUE,FALSE,VK_DOWN};
struct ShortcutKey skfindmatchchar={TRUE,FALSE,FALSE,'B'};
struct ShortcutKey skmarkmatchchar={TRUE,FALSE,TRUE,'B'};
struct ShortcutKey skdeletebracepair={TRUE,TRUE,FALSE,'B'};
//struct ShortcutKey skdeletebracepaircontents={TRUE,TRUE,TRUE,'B'};
struct ShortcutKey skmarkmatchline={TRUE,TRUE,TRUE,'B'};
struct ShortcutKey skcopyallnoappend={TRUE,FALSE,FALSE,'C'};
struct ShortcutKey skcopyallappend={TRUE,FALSE,TRUE,'C'};
struct ShortcutKey skcutallnoappend={TRUE,FALSE,FALSE,'X'};
struct ShortcutKey skcutallappend={TRUE,FALSE,TRUE,'X'};
struct ShortcutKey skVizPasteUNICODE={TRUE,FALSE,FALSE,'V'};
struct ShortcutKey skindentlines={FALSE,FALSE,FALSE,VK_TAB};
#endif

struct FuncItem funcItem[]={
#if NPPDEBUG /* { it wasn't automatic at first */
{"(Experimental) Temp Function",pftemp,0,FALSE},
{"(Experimental) Display Clipboard Info",pfinsertclipinfo,0,FALSE},
//{"Convert EOLs",pfconverteol,0,FALSE},
{"(Experimental) Insert Menu Info",pfshowmenuinfo,0,FALSE},
//{"(Experimental) Build New Menu",pfbuildmenu,0,FALSE},
//{"(Experimental) Display wID",pfdisplaywIDtest,0,FALSE},
//{"(Experimental) Build Dynamic Menus",buildallIDLOOKUP,0,FALSE},
//{"(Experimental) Destroy Dynamic Menus",freeallIDLOOKUP,0,FALSE},
#endif /* } */
{"Convert quotes to \"",pfconvert1q2q,0,FALSE},
{"Convert quotes to '",pfconvert2q1q,0,FALSE},
{"Swap quotes (\" <-> ')",pfconvertqtswap,0,FALSE},
{"Drop quotes \" && '",pfconvertqtdrop,0,FALSE},
{"-",pfdummy,0,FALSE}, // Notepad++ does not allow NULL functions so we pick a dummy
{"Escape \" to \\\"",pfconvertescapesq,0,FALSE},
{"Escape ' to \\'",pfconvertescape1qs1q,0,FALSE},
{"Escape ' to \\\"",pfconvertescape1qsq,0,FALSE},
{"Escape both \"&&' to \\\"&&\\'",pfconvertescapeboth,0,FALSE},
{"unEscape \\\" to \"",pfconvertunescapesq,0,FALSE},
{"unEscape \\' to '",pfconvertunescapes1q1q,0,FALSE},
{"unEscape \\\" to '",pfconvertunescapesq1q,0,FALSE},
{"unEscape both \\\"&&\\' to \"&&'",pfconvertunescapeboth,0,FALSE},
{"Escape \" to \"\"",pfconvertescape2q22q,0,FALSE},
{"Escape ' to \"\"",pfconvertescape1q22q,0,FALSE},
{"unEscape \"\" to \"",pfconvertunescape22q2q,0,FALSE},
{"unEscape \"\" to '",pfconvertunescape22q1q,0,FALSE},
{"-",pfdummy,0,FALSE},
{"Proper Case",pfconvertpropercase,0,FALSE},
{"Sentence case.",pfconvertsentencecase,0,FALSE},
{"iNVERT cASE",pfconvertinvertcase,0,FALSE},
{"Zap all characters to space",pfzapspace,0,FALSE},
{"Zap all non printable characters to #",pfzapnonprint,0,FALSE},
{"Q:Mark Word or Find Reverse"/*" 繁体支持"*/,pfMarkWordFindReverse,0,FALSE NPPPLUGINACCELERATOR(skMarkWordFindReverse)},
{"Q:Mark Word or Find Forward",pfMarkWordFindForward,0,FALSE  NPPPLUGINACCELERATOR(skMarkWordFindForward)},
{"Q:+Mark Word or Find Case Sensitive",pfMarkWordFindCaseSensitive,0,FALSE},
//{"Q:Show Style",pfshowstyle,0,FALSE},
//{"Q:Find In Other View",pffindinotherview,0,FALSE},
//{"Q:Switch selection to rectangular",pfswitchseltorectangular,0,FALSE},
{"Q:Find matching {([<Brace>])}",pffindmatchchar,0,FALSE  NPPPLUGINACCELERATOR(skfindmatchchar)},//Ctrl-B in Notepad++
{"Q:Mark to matching {([<Brace>])}",pfmarkmatchchar,0,FALSE  NPPPLUGINACCELERATOR(skmarkmatchchar)},
{"Q:Delete Marked {([<Brace>])} Pair",pfdeletebracepair,0,FALSE  NPPPLUGINACCELERATOR(skdeletebracepair)},
//{"Q:Delete Between Marked {([<Brace>])} Pair",pfdeletebracepaircontents,0,FALSE  NPPPLUGINACCELERATOR(skdeletebracepaircontents)},
{"Q:Mark lines to matching {([<Brace>])}",pfmarkmatchline,0,FALSE  NPPPLUGINACCELERATOR(skmarkmatchline)},
{"Q:Show line matching Brace])}",pfshowmatchline,0,FALSE},
#if ENABLE_PUSHPOP
{"Q:-",pfdummy,0,FALSE},
{"Q:Push position",pfpushposition,0,FALSE},
{"Q:Push position and jump to first occurance of (Clipboard)",pfpushjumpposition,0,FALSE},
{"Q:Recall/Pop position",pfpopposition,0,FALSE},
#endif
//{"V:Show/Hide 'All' Set Line Range",pfvizshowselectedalllines,0,FALSE}, // proposed
//{"V:Show/Hide 'All' Clear Range",pfvizshowselectedalllines,0,FALSE}, // proposed
{"V:Show Between-Selected or All-Reset Lines",pfvizshowselectedalllines,0,FALSE},
{"V:Hide Between-Selected or All-Reset Lines",pfvizhideselectedalllines,0,FALSE},
{"V:Invert Visibility Between-Selected or All Lines",pfvizinvertselectedalllines,0,FALSE},
{"V:Hide Lines with (Clipboard) text",pfvizhidecliplines,0,FALSE},
{"V:Hide Lines without (Clipboard) text",pfvizhideclipclines,0,FALSE},
{"V:Show Lines with (Clipboard) text",pfvizshowcliplines,0,FALSE},
{"V:Show Lines without (Clipboard) text",pfvizshowclipclines,0,FALSE},
{"V:Show More Lines around my position...",pfvizshowmorelines,0,FALSE},
{"V:-",pfdummy,0,FALSE},
{"V:Hide/Show sequence all steps",pfvizsequenceall,0,FALSE},
{"V:Hide/Show sequence singlestep start",pfvizsequencestart,0,FALSE},
{"V:Hide/Show sequence singlestep next",pfvizsequencenext,0,FALSE},
{"V:Hide/Show sequence singlestep rest",pfvizsequencerest,0,FALSE},
{"V:Select as Hide/Show sequence",pfvizselectassequence,0,FALSE},
{"V:Insert Show/Hide Sequence",pfvizinsertsequence,0,FALSE},
{"V:-",pfdummy,0,FALSE},
{"V:Copy Visible Selection",pfvizcopyvisible,0,FALSE},
{"V:Cut Visible Selection",pfvizcutvisible,0,FALSE},
{"V:Delete Visible Selection",pfvizdeletevisible,0,FALSE},
{"V:Copy Invisible Selection",pfvizcopyinvisible,0,FALSE},
{"V:Cut Invisible Selection",pfvizcutinvisible,0,FALSE},
{"V:Delete Invisible Selection",pfvizdeleteinvisible,0,FALSE},
{"V:Copy Entire Selection (no append)",pfcopyallnoappend,0,FALSE  NPPPLUGINACCELERATOR(skcopyallnoappend)},
{"V:Cut Entire Selection (no append)",pfcutallnoappend,0,FALSE  NPPPLUGINACCELERATOR(skcutallnoappend)},
{"V:Copy && Append Entire Selection",pfcopyallappend,0,FALSE  NPPPLUGINACCELERATOR(skcopyallappend)},
{"V:Cut && Append Entire Selection",pfcutallappend,0,FALSE  NPPPLUGINACCELERATOR(skcutallappend)},
{"V:Paste as UTF-8/ANSI",pfVizPasteUTF8,0,FALSE},
{"V:Paste",pfVizPasteUNICODE,0,FALSE  NPPPLUGINACCELERATOR(skVizPasteUNICODE)},
//{"V:-",pfdummy,0,FALSE},
{"W:+Viz Text Search Case Sensitive",pfVizCaseSensitive,0,FALSE},
{"W:+Viz Text Search Whole Words",pfVizWholeWords,0,FALSE},
{"W:+Viz Text Search Regex",pfVizRegex,0,FALSE},
{"W:+Viz Copy-Cut Appends to clipboard",pfVizCutCopyAppend,0,FALSE},
{"W:+Viz Copy-Cut always converts to CRLF",pfVizClipboardAlwaysCRLF,0,FALSE},
{"W:+Viz Copy-Cut replace [NUL] with space",pfVizClipboardReplaceNulls,0,FALSE},
{"W:+Viz Copy-Cut also in UTF-8",pfVizClipboardCopyAlsoUTF8,0,FALSE},
{"W:+Viz Copy-Cut not in UNICODE",pfVizClipboardNotUnicode,0,FALSE},
{"W:+Viz Paste retains position",pfVizPasteRetainsPosition,0,FALSE},
{"W:+Viz Paste/Append binary",pfVizPasteBinary,0,FALSE},
{"W:+Viz Paste converts EOL to editor",pfVizPasteToEditorEOL,0,FALSE},
{"W:+Viz Capture Keyboard Ctrl-C,X,V",pfCaptureCutCopyPaste,0,FALSE},
{"E:Fill Down Insert",pffilldownins,0,FALSE},
{"E:Fill Down Overwrite",pffilldownover,0,FALSE},
{"E:Insert (Clipboard) through lines",pfinsertclipboardcolumn,0,FALSE},
{"E:-",pfdummy,0,FALSE},
{"E:Reindent C++ code",pfreindentcode,0,FALSE},
{"E:Leading space to tabs or tabs to spaces",pfspace2tabs,0,FALSE},
{"E:Leading space to tabs or tabs to spaces width=8",pfspace2tabs8,0,FALSE},
{"E:Trim Trailing Spaces",pftrimtrailingspace,0,FALSE}, // Notepad++ 3.2 is slow and only trims the entire file
{"E:Indent text sticky left margin",pfindentlines,0,FALSE  NPPPLUGINACCELERATOR(skindentlines)},
{"E:Indent && surround { text lines }",pfindentlinessurround,0,FALSE},
{"E:Delete Blank Lines",pfdeleteblanklines,0,FALSE},
{"E:Delete Surplus Blank Lines",pfdeleteblanklines2,0,FALSE},
//{"E:Delete Every Other Line",pfdeleteevery2lines,0,FALSE}, // completely handled by viz now
{"E:-",pfdummy,0,FALSE},
{"E:Strip unquoted text (VB) separate by (Clipboard<=20)",pffindqtstringvb,0,FALSE},
{"E:Strip unquoted text (C) separate by (Clipboard<=20)",pffindqtstringc,0,FALSE},
{"E:Kill unquoted (VB) whitespace",pfkillwhitenonqtvb,0,FALSE},
{"E:Kill unquoted (C) whitespace",pfkillwhitenonqtc,0,FALSE},
{"E:Split lines at (clipboard character) or , (VB)",pfsplitlinesatchvb,0,FALSE},
{"E:Split lines at (clipboard character) or , (C)",pfsplitlinesatchc,0,FALSE},
{"E:Line up multiple lines by (,)",pflineupcomma,0,FALSE},
{"E:Line up multiple lines by (=)",pflineupequals,0,FALSE},
{"E:Line up multiple lines by (Clipboard Character)",pflineupclipboard,0,FALSE},
{"E:-",pfdummy,0,FALSE},
{"E:Unwrap Text",pfunwraptext,0,FALSE},
{"E:ReWrap Text to (Clipboard or 72) width",pfrewraptext,0,FALSE},
//{"E:Extend selection as rectangular to end of file",pfextendblock,0,FALSE},
{"E:Pad rectangular selection with spaces",pfextendblockspaces,0,FALSE},
{"C:Encode URI Component",pfencodeURIcomponent,0,FALSE},
{"C:Encode HTML (&&<>\")",pfencodeHTML,0,FALSE},
{"C:Strip HTML tags table tabs",pfstripHTMLtags,0,FALSE},
{"C:Strip HTML tags table nontabs",pfstripHTMLnotabs,0,FALSE},
{"C:Submit to W3C HTML Validator",pfsubmitHTML,0,FALSE},
{"C:Submit to W3C CSS Validator",pfsubmitCSS,0,FALSE},
{"C:Convert text to code command(\"text=\\\"value\\\"\");",pfprepostpendlines,0,FALSE},
{"C:-",pfdummy,0,FALSE},
{"C:Convert Decimal Number to Binary",pfdecimal2binary,0,FALSE},
{"C:Convert Decimal Number to Octal",pfdecimal2octal,0,FALSE},
{"C:Convert Decimal Number to Hex",pfdecimal2hex,0,FALSE},
{"C:Convert Hex Number to Decimal",pfhex2decimal,0,FALSE},
{"C:Convert Octal Number to Decimal",pfoctal2decimal,0,FALSE},
{"C:Convert Binary Number to Decimal",pfbinary2decimal,0,FALSE},
{"C:Convert C-style Number to Decimal",pfcnum2decimal,0,FALSE},
{"C:-",pfdummy,0,FALSE},
{"C:Convert text to Hex-16",pftohex16,0,FALSE},
{"C:Convert text to Hex-32",pftohex32,0,FALSE},
{"C:Convert text to Hex-64",pftohex64,0,FALSE},
{"C:Convert text to Hex-128",pftohex128,0,FALSE},
{"C:Convert hex byte runs into LE-WORDS",pfhexbyterunstolittlendian2,0,FALSE},
{"C:Convert hex byte runs into LE-DWORDS",pfhexbyterunstolittlendian4,0,FALSE},
{"C:Convert LE-words to hex byte runs",pflittlendiantohexbyteruns,0,FALSE},
{"C:Convert Hex to text",pffromhex,0,FALSE},
{"C:ROT13 Text",pfrot13,0,FALSE},
{"C:Convert EBCDIC to ASCII",pfEBCDIC2ascii,0,FALSE},
{"C:Convert ASCII to EBCDIC",pfascii2EBCDIC,0,FALSE},
{"T:Sort lines case sensitive (at column)",pfqsortlinesc,0,FALSE},
{"T:Sort lines case insensitive (at column)",pfqsortlinesnc,0,FALSE},
{"T:+Sort ascending",pfSortAscending,0,TRUE},
{"T:+Sort outputs only UNIQUE (at column) lines",pfSortLinesUnique,0,TRUE},
{"T:-",pfdummy,0,FALSE},
{"T:Insert Ascii Chart or Character",pfinsertasciichart,0,FALSE},
{"T:Insert Ruler",pfinsertruler,0,FALSE},
{"T:Insert Line Numbers",pfinsertlinenumbers,0,FALSE},
{"T:Delete Line Numbers or First Word",pfdeletefirstword,0,FALSE},
{"T:Clean eMail >Quoting",pfcleanemailquoting,0,FALSE},
{"T:UUdecode",pfuudecode,0,FALSE},
{"T:Base64 Decode",pfbase64decode,0,FALSE},
{"T:Word Count",pfwordcount,0,FALSE},
{"T:Add up numbers",pfaddup,0,FALSE},
{"T:-",pfdummy,0,FALSE},
{"T:Empty Undo Buffer (be sure to save)",pfemptyundobuffer,0,FALSE},
#ifndef HighPerformance
{"S:+Go Faster, use more memory",pfHighPerformance,0,TRUE},
#endif
{"S:+Cancel Overwrite Mode moving from current line",pfBlockOverwrite,0,TRUE},
{"S:+Autoclose XHTML/XML <Tag>",pfAutoCloseHTMLtag,0,TRUE},
{"S:+Autoclose {([Brace",pfAutoCloseBrace,0,TRUE},
{"S:+Autoconvert typed leading spaces to tabs",pfAutoSpace2Tab,0,TRUE},
{"S:+Autoshow line matching Brace])}",pfAutoShowMatchline,0,TRUE},
{"S:+Autoconvert typed HTML/XML to &&entities;",pfAutoConvertHTML,0,TRUE},
{"S:+Disable Subclassing && advanced features",pfDisableSubclassing,0,FALSE},
{"S:+Move quick menus out of 'Plugins' menu",pfSeparateQuickMenus,0,FALSE},
{"S:+Improve Home-End Movement for Wrapped Text",pfWrapFriendlyHomeEnd,0,FALSE},
{"S:-",pfdummy,0,FALSE},
{"S:Visit Notepad++ && "PLUGIN_NAME" website",pfhNotepadweb,0,FALSE},
{"S:Help",pfhelp,0,FALSE},
{"S:About "PLUGIN_NAME,pfabout,0,FALSE},
#if NPPDEBUG
{"H:>Online Help",pfmpxhelprebuild,0,FALSE},
#endif
#if ENABLE_TIDYDLL
{"D:About Tidy",pfabouttidy,0,FALSE},
{"D:Visit HTML Tidy SourceForge website",pfhtmltidyweb,0,FALSE},
{"D:Download libTidy.DLL from SourceForge",pfgethtmltidyweb,0,FALSE},
{"D:Reload libTidy.DLL",pfreloadtidydll,0,FALSE},
{"D:Refresh Menu from "NPPTIDY_INI,pfmpxtidyrebuild,0,FALSE},
{"D:-",pfdummy,0,FALSE},
{"D:Tidy (most recent "NPPTIDY_CFG")",pfhtmltidy,0,FALSE},
#endif
};

unsigned findmenuitem(PFUNCPLUGINCMD_MSC *_pFunc) {
  unsigned itemno; for(itemno=0; itemno<NELEM(funcItem); itemno++) if (_pFunc == funcItem[itemno]._pFunc) return(itemno);
  return(0);
}

/*
 *--------------------------------------------------
 * The 4 extern functions are mandatory
 * They will be called by Notepad++ plugins system
 *--------------------------------------------------
*/

// The setInfo function gets the needed infos from Notepad++ plugins system
void DLLIMPORT __cdecl setInfo(struct NppData notpadPlusData) {
  //MessageBox(0 , "setInfo",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  nppData = notpadPlusData;
#if NPPDEBUG /* { */
    testsprintfuncs();
#endif /* } */
}

// The getName function tells Notepad++ plugins system its name
const char DLLIMPORT * __cdecl getName(void) {
  //MessageBox(0 , "getName",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  return "_" PLUGIN_NAME
#if NPPDEBUG /* { */
  " (debug)"
#endif /* } */
  " Characters";
}

// The getFuncsArray function gives Notepad++ plugins system the pointer FuncItem Array
// and the size of this array (the number of functions)
struct FuncItem DLLIMPORT * __cdecl getFuncsArray(int *nbF) {
  //MessageBox(0 , "getFuncsArray",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  *nbF = NELEM(funcItem);
  return funcItem;
}

// If you don't need get the notification from Notepad++,
// just let it be empty.
// new feature: block multiline selections when lines are hidden
void DLLIMPORT __cdecl beNotified(struct SCNotification *notifyCode) {
  static unsigned runonce=0;
  char *rch;
  static unsigned prevline;
  static BOOL block=FALSE;
  BOOL kscapital;

if (!block) { // with enough delay, beNotified ends up rentrant
  block=TRUE;
  if (!runonce && loadonce) {
    pfbuildmenu();
    MENUITEMINFO miosep={sizeof(MENUITEMINFO),MIIM_TYPE,MFT_SEPARATOR,0,0,0,0,0,0,NULL,0};
    for(runonce=0; runonce<NELEM(funcItem); runonce++) if (funcItem[runonce]._itemName[0]=='-') SetMenuItemInfo(GetMenu(nppData._nppHandle),funcItem[runonce]._cmdID,FALSE,&miosep);
#if ENABLE_TIDYDLL
    control_tidy(FALSE);
#endif
    DrawMenuBar(nppData._nppHandle);
    CloseHandle(loadonce); // closing here will allow another invocation of Notepad++ + NPPTextFX to load if N++ allows this
    loadonce=NULL;
    if (!funcItem[miDisableSubclassing]._init2Check) {
      buildallIDLOOKUP(); NPPSubclassProc_init(); // must be after pfbuildmenu();
    }
    /*for(runonce=1; runonce<3; runonce++) {
      SENDMSGTOCED(runonce==1, SCI_CLEARCMDKEY, MAKEWPARAM('C',SCMOD_CTRL), 0); // Notepad++ isn't handling these keys but will give them to us through the new features
      SENDMSGTOCED(runonce==1, SCI_CLEARCMDKEY, MAKEWPARAM('V',SCMOD_CTRL), 0);
      SENDMSGTOCED(runonce==1, SCI_CLEARCMDKEY, MAKEWPARAM('X',SCMOD_CTRL), 0);
      SENDMSGTOCED(runonce==1, SCI_CLEARCMDKEY, MAKEWPARAM(SCK_TAB,SCMOD_NORM), 0); // Convert All needs to get the current line to make this work
    }*/
    WrapFriendlyHomeEnd(funcItem[miWrapFriendlyHomeEnd]._init2Check);
  }

  switch (notifyCode->nmhdr.code) {
  case SCN_UPDATEUI:
    kscapital=(GetKeyState(VK_CAPITAL)&0x8000)?TRUE:FALSE;
    if (funcItem[miBlockOverwrite]._init2Check || kscapital) {
      INT_CURRENTEDIT; GET_CURRENTEDIT;
      unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
      unsigned curline=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,curpos,0);
      if (prevline!=curline) {
        if (funcItem[miBlockOverwrite]._init2Check && SENDMSGTOCED(currentEdit, SCI_GETOVERTYPE, 0, 0)) {
        	SENDMSGTOCED(currentEdit, SCI_SETOVERTYPE, 0, 0);
          //MessageBoxFree(nppData._nppHandle,smprintf("First unhandled code after insert change %d",notifyCode->nmhdr.code),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
        }
        int dir=curline>prevline?1:-1;
        if (kscapital && prevline+dir != curline && !SENDMSGTOCED(currentEdit, SCI_GETLINEVISIBLE, prevline+dir, 0)) {
          unsigned col=SENDMSGTOCED(currentEdit, SCI_GETCOLUMN, curpos, 0);
          //MessageBoxFree(nppData._nppHandle,smprintf("%X Jumped from line %d to %d dir %d maintain column:%u",GetKeyState(VK_CAPITAL),prevline,curline,dir,col),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
          curline=prevline+dir;
          SENDMSGTOCED(currentEdit, SCI_SHOWLINES, curline, curline);
          SENDMSGTOCED(currentEdit, SCI_GOTOPOS,SENDMSGTOCED(currentEdit, SCI_FINDCOLUMN, curline, col), 0);
        }
      }
      prevline=curline;
    }
    if (funcItem[miAutoShowMatchline]._init2Check) pfshowmatchline();
    break;
  case SCN_CHARADDED:
    switch(notifyCode->ch) {
    case ' ': // Autoconvert spaces to tabs
      if (funcItem[miAutoSpace2Tab]._init2Check) {
        INT_CURRENTEDIT; GET_CURRENTEDIT;
        unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
        unsigned bol=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE,SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION,curpos,0),0);
        unsigned buflen=curpos-bol,bufsz=buflen+1; char *buf; if (!(buf=(char *)mallocsafe(bufsz,"scNotify-space"))) break;
        int tabwidth;
        if (SENDMSGTOCED(currentEdit, SCI_GETUSETABS, 0, 0) && (tabwidth=SENDMSGTOCED(currentEdit, SCI_GETTABWIDTH, 0, 0)) /*&& curpos-bol<sizeof(buf)*/) {
          struct TextRange tr; tr.chrg.cpMin=bol; tr.chrg.cpMax=curpos; tr.lpstrText=buf;
          if (tr.chrg.cpMin>=0 && SENDMSGTOCED(currentEdit, SCI_GETTEXTRANGE, 0, &tr) == tr.chrg.cpMax-tr.chrg.cpMin) {
            curpos-=bol;
            unsigned curposcol; char *d; for(d=buf,curposcol=0; *d && curpos!=(unsigned)-1; curposcol+=(*d=='\t'?tabwidth:1),d++,curpos--);
            if (space2tabs(&buf,&bufsz,&buflen,1,tabwidth,1)) {
              for(d=buf; *d && curposcol>0; curposcol-=(*d=='\t'?tabwidth:1),d++);
              //MessageBoxFree(nppData._nppHandle,smprintf("curpos:%d bol:%d curposcol:%d cpmin:%d cpmax:%d buf\r\n%s",curpos,bol,curposcol,tr.chrg.cpMin,tr.chrg.cpMax,tr.lpstrText),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
              SENDMSGTOCED(currentEdit, SCI_BEGINUNDOACTION, 0, 0);
              SENDMSGTOCED(currentEdit, SCI_SETTARGETSTART, tr.chrg.cpMin, 0);
              SENDMSGTOCED(currentEdit, SCI_SETTARGETEND, tr.chrg.cpMax, 0);
              SENDMSGTOCED(currentEdit, SCI_REPLACETARGET, -1,buf);
              SENDMSGTOCED(currentEdit, SCI_GOTOPOS, (d-buf+bol),0);
              SENDMSGTOCED(currentEdit, SCI_ENDUNDOACTION, 0, 0);
            }
          }
        }
        freesafe(buf,"scNotify-space");
      }
      break;
    case '(': rch=")"; goto acb;
    case '[': rch="]"; goto acb;
    case '{': rch="}";
acb:  if (funcItem[miAutoCloseBrace]._init2Check) { // auto close brace
        enum LangType docType; SendMessage(nppData._nppHandle, WM_GETCURRENTLANGTYPE, 0, (LPARAM)&docType);
        if (docType == L_PHP || docType == L_C || docType == L_CPP || docType == L_CS || docType == L_OBJC || docType == L_JAVA || docType == L_ASP || docType ==  L_SQL || docType ==  L_JS || docType ==  L_PERL || docType ==  L_LUA || docType ==  L_FLASH) {
          INT_CURRENTEDIT; GET_CURRENTEDIT;
          unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
          unsigned curposl=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION, curpos, 0);
          //if (curpos!=SENDMSGTOCED(currentEdit, SCI_GETLINEENDPOSITION, curposl, 0)) break; // can only autoclose at end of line http://www.textpad.info/forum/viewtopic.php?t=2838
          char *buf=NULL; unsigned bufsz=2,buflen;
          if (notifyCode->ch=='{') {
            unsigned eoltype=SENDMSGTOCED(currentEdit, SCI_GETEOLMODE, 0, 0); if (eoltype>=NELEM(eoltypes)) eoltype=NELEM(eoltypes)-1;
            unsigned extra=strlen(eoltypes[eoltype]);
            bufsz=extra+SENDMSGTOCED(currentEdit, SCI_GETLINE,curposl,NULL)+1;
            strcpyarmsafe(&buf,&bufsz,&buflen,eoltypes[eoltype],"scNotify-acb"); if (!buf) break;
            buflen += SENDMSGTOCED(currentEdit, SCI_GETLINE,curposl,buf+buflen); buf[buflen]='\0';
            //MessageBoxFree(nppData._nppHandle,smprintf("reqbufsz:%u bufsz:%u buflen:%u strlen:%u\r\n%s",extra+SENDMSGTOCED(currentEdit, SCI_GETLINE,curposl,NULL)+1,bufsz,buflen,strlen(buf),buf),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
            buflen=extra+strspn(buf+extra," \t");
            strcpyarmsafe(&buf,&bufsz,&buflen,rch,"scNotify-acb"); if (!buf) break;
            SENDMSGTOCED(currentEdit, SCI_GOTOPOS, SENDMSGTOCED(currentEdit, SCI_GETLINEENDPOSITION,curposl,0), 0);
          } else {strcpyarmsafe(&buf,&bufsz,&buflen,rch,"scNotify-acb"); if (!buf) break;}
          SENDMSGTOCED(currentEdit, SCI_BEGINUNDOACTION, 0, 0);
          SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, buf);
          SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos, 0);
          SENDMSGTOCED(currentEdit, SCI_ENDUNDOACTION, 0, 0);
          freesafe(buf,"acb");
        }
      }
      break;
    case '>': // Autoclose HTML tag
      if (funcItem[miAutoConvertHTML]._init2Check) {rch="&gt;"; goto acH;} // ach: will pass back to act2: if > is not to be converted to an entity because in a tag
      if (funcItem[miAutoCloseHTMLtag]._init2Check ) {
        enum LangType docType; SendMessage(nppData._nppHandle, WM_GETCURRENTLANGTYPE, 0, (LPARAM)&docType);
        if ((docType == L_HTML || docType == L_XML /*|| docType == L_PHP || docType == L_ASP*/)) {
act2:     INT_CURRENTEDIT; GET_CURRENTEDIT;
          unsigned curpos,bktpos; if (bracematch(currentEdit,&curpos,&bktpos,FLAG_INCLUDEBRACKETS) && bktpos<curpos && curpos-bktpos>2 /* 2 ensures no null <> tags */) {
            bktpos++;
#define SZPREFIX 2 /* sizeof("<\"); */
            char *buf=NULL; unsigned bufsz=SZPREFIX+curpos-bktpos+1,buflen;
            //MessageBoxFree(nppData._nppHandle,smprintf("curpos:%u bktpos:%u curpos-bktpos+%u+1:%u bufsz:%u buflen:%u",curpos,bktpos,SZPREFIX,curpos-bktpos+SZPREFIX+1,bufsz,buflen),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
            strcpyarmsafe(&buf,&bufsz,&buflen,"</","scNotify-act2"); if (!buf) break;
            //MessageBoxFree(nppData._nppHandle,smprintf("curpos:%u bktpos:%u curpos-bktpos+%u+1:%u bufsz:%u buflen:%u strlen:%u\r\n%s",curpos,bktpos,SZPREFIX,curpos-bktpos+SZPREFIX+1,bufsz,buflen,strlen(buf),buf),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
            struct TextRange tr; tr.chrg.cpMin=bktpos; tr.chrg.cpMax=curpos; tr.lpstrText=buf+SZPREFIX;
            if ( (buflen+=SENDMSGTOCED(currentEdit, SCI_GETTEXTRANGE, 0, &tr))>1+SZPREFIX /* 1 to prevent close from null <> tags*/
                && buf[buflen-2]!='/' /* prevent from self-closing XHTML tags */
                && isgraph(buf[2])    /* prevent close from strange characters */
                && memicmp(buf+2,"BR",2) // http://codex.wordpress.org/HTML_to_XHTML
                && memicmp(buf+2,"HR",2)
                && memicmp(buf+2,"META",4)
                && memicmp(buf+2,"LINK",4)
                && memicmp(buf+2,"IMG",3)
                && !strchr("/%?!",buf[2])  /* prevent from close and special tags */ ) {
              //MessageBoxFree(nppData._nppHandle,smprintf("curpos:%u bktpos:%u curpos-bktpos+%u+1:%u bufsz:%u buflen:%u strlen:%u\r\n%s",curpos,bktpos,SZPREFIX,curpos-bktpos+SZPREFIX+1,bufsz,buflen,strlen(buf),buf),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
              char *sp; if (sp=strchr(buf+2,' ')) strcpy(sp,">");
              SENDMSGTOCED(currentEdit, SCI_BEGINUNDOACTION, 0, 0);
              SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, buf);
              SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos, 0);
              SENDMSGTOCED(currentEdit, SCI_ENDUNDOACTION, 0, 0);
            }
            freesafe(buf,"scNotify-act2");
#undef SZPREFIX
          }
        }
      }
      break;
    case '"': rch="&quot;"; goto acH; // " not converted if in a tag
    case '&': rch="&amp;";  goto acH; // & always converted
    case '<': rch="&lt;";   goto acH; // < not converted if not in a tag;
    case 13 : rch="<br/>";            // > not converted if in a tag
acH:  if (funcItem[miAutoConvertHTML]._init2Check) { // auto convert HTML entities
        enum LangType docType; SendMessage(nppData._nppHandle, WM_GETCURRENTLANGTYPE, 0, (LPARAM)&docType);
        if ((docType == L_HTML || docType == L_XML /*|| docType == L_PHP || docType == L_ASP*/)) {
          INT_CURRENTEDIT; GET_CURRENTEDIT;
          unsigned curpos=SENDMSGTOCED(currentEdit, SCI_GETCURRENTPOS, 0, 0);
          unsigned curposl=SENDMSGTOCED(currentEdit, SCI_LINEFROMPOSITION, curpos, 0);
          unsigned bol=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE,curposl,0);
          BOOL intag=FALSE;
          if (curpos-1>bol) {
            char *t; if (t=(char *)mallocsafe(curpos-bol,"scNotify-acH")) {
              int sln=SENDMSGTOCED(currentEdit, SCI_GETCURLINE,curpos-bol,t);
              //MessageBoxFree(nppData._nppHandle,smprintf("curpos-1:%u bol:%u curpos-bol:%u test:%u lastchs:%d:%d:%d:%d:%d\r\n%s",curpos-1,bol,curpos-bol,test,t[curpos-bol-1],t[curpos-bol],t[curpos-bol+1],t[curpos-bol+2],t[curpos-bol+3],t),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
              char *tx; for(tx=t+sln-1; tx>=t; tx--) if (*tx=='<') {intag=TRUE; break;} else if (*tx=='>') break;
              freesafe(t,"scNotify-acH");
            }
          }
          //MessageBoxFree(nppData._nppHandle,smprintf("intag:%s",intag?"TRUE":"FALSE"),PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
          if (notifyCode->ch=='>' && intag) {
             if (funcItem[miAutoCloseHTMLtag]._init2Check) goto act2; // act gave up this character for us and we need to send it back validated if we can't use it
             else break;
          } else if (notifyCode->ch=='<' && !intag) { break;
          } else if (notifyCode->ch=='"' &&  intag) { break;
          }
          if (curpos>0) {
            unsigned s1,s2;
            if (notifyCode->ch==13) {
              s1=s2=SENDMSGTOCED(currentEdit, SCI_GETLINEENDPOSITION, curposl-1, 0);
              curpos=SENDMSGTOCED(currentEdit, SCI_POSITIONFROMLINE, curposl,0);
            } else {
            	curpos--;
              s1=curpos;
              s2=curpos+1;
            }
            SENDMSGTOCED(currentEdit, SCI_BEGINUNDOACTION, 0, 0);
            SENDMSGTOCED(currentEdit, SCI_SETSEL, s1, s2);
            SENDMSGTOCED(currentEdit, SCI_REPLACESEL, 0, rch);
            SENDMSGTOCED(currentEdit, SCI_GOTOPOS, curpos+strlen(rch), 0);
            SENDMSGTOCED(currentEdit, SCI_ENDUNDOACTION, 0, 0);
          }
        }
      }
      break;
    }
    break;
  }
  block=FALSE;
}
}
// SECTION: END
// SECTION: Beginning of DllMain

//----------------------------------------------------------//
// Standard Windows DLL Entry Point
//----------------------------------------------------------//

BOOL APIENTRY DllMain(HINSTANCE hInst, DWORD reason, LPVOID lpvReserved) {
  char *t,*path,pathtemp[MAX_PATH];
  OSVERSIONINFO osv = {sizeof(OSVERSIONINFO)};

  //MessageBox(0 , "DllMain",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
  switch (reason) {
  case DLL_PROCESS_ATTACH:
#ifdef mallocsafeinit
    mallocsafeinit(); // must be first
    testmallocsafe();
#endif
    GetVersionEx(&osv); onNT = osv.dwPlatformId == VER_PLATFORM_WIN32_NT;
    if (!(loadonce=CreateFileMapping( (HANDLE) 0xFFFFFFFF, NULL,PAGE_READWRITE, 0,4,"Notepad++" PLUGIN_NAME))) return FALSE;
    // Windows automatically closes this FileMapping when the last DLL quits.
    if (GetLastError() == ERROR_ALREADY_EXISTS) {
      MessageBox(nppData._nppHandle,"You can't load "PLUGIN_NAME" twice. Please delete the extra DLL in your Notepad ++ plugins folder",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
      return(FALSE);
    }
#if ENABLE_PUSHPOP
    memset(poplists,0,sizeof(poplists));
#endif
    GetModuleFileName(hInst, pathtemp, sizeof(pathtemp));
    for(t=pathtemp+strlen(pathtemp); t>=pathtemp && *t != '\\'; t--);
    t[1]='\0'; //MessageBox(0 , pathtemp,PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    if (!(pluginpath=strdupsafe(pathtemp,"DllMain-pluginpath"))) return(FALSE);
    iniSaveSettings(pluginpath,FALSE); // must be after pluginpath is set
    cfColumnSelect = RegisterClipboardFormat("MSDEVColumnSelect");
    break;
  case DLL_PROCESS_DETACH:
    //MessageBox(0 , "DLL_PROCESS_DETACH",PLUGIN_NAME, MB_OK|MB_ICONINFORMATION);
    NPPSubclassProc_close();
    iniSaveSettings(pluginpath,TRUE);
    freeallIDLOOKUP();
    if (loadonce) CloseHandle(loadonce);
    if (pluginpath) {
      if (path=smprintfpath("%s%?\\%s",pluginpath,"temp.htm\0Extra unused space for l33t haxors to specify their own filenames\0")) {
        DeleteFile(path);
        freesafe(path,"DllMain-path");
      }
      freesafe(pluginpath,"DllMain-pluginpath");
    }
    if (vizsequence) freesafe(vizsequence,"DllMain");
#if ENABLE_TIDYDLL
    control_tidy(TRUE);
    if (tidydll_error) freesafe(tidydll_error,"DllMain-tidydll_error");
#endif
#ifdef mallocsafeinit
    mallocsafedone(); // must be after any safe calls
#endif
    break;
  case DLL_THREAD_ATTACH:
    break;
  case DLL_THREAD_DETACH:
    break;
  }
  return TRUE;
}

/* Execution sequence:
  1) DllMain is called when Notepad++ loads the plugin
  2) N++ calls getFuncsArray, setInfo, getName to get initial operating info
  3) beNotified is called constantly while editing.
  4) Occasionally the user will select a plugin menu item.
  5) The user will quit N++ which will call DllMain for the last time. */

// SECTION: END
